/**
* UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
* AUTOR LUIZ ANGELO BARCHET ESTEFANEL
* CLASSE AwarenessEventsWindow
* PACOTE BW.UML
* DATA DE CRIACAO 16 / 01 / 2001
*/

package BW.UML;

import java.awt.*;
import java.awt.event.*;
import BW.kernel.*;
import BW.Interface.*;
import java.util.Vector;
import java.util.Calendar;

/**
* A classe AwarenessEventsWindow exibe os detalhes de um conjunto de eventos semelhantes.
* @author Luiz Angelo Barchet Estefanel
* @version 1.0
*/
public class AwarenessEventsWindow extends Frame implements UI_GUIEvent, WindowListener, ItemListener {

   ////////////////////////// ATRIBUTOS
   
   /** variavel que contem o prototipo do evento a ser apresentado
   */
   private BW_Event prototype;

   /** vetor que contem a lista de eventos exibidos
   */
   private Vector events;
   
   /**  referencia ao elemento que gerencia esta classe
   */
   private UI_GUIElement element;

   /** Variaveis da Interface grafica
   */
      private Frame frameInfo;
      private List eventsList;
      private TextArea eventInfo;      

   ////////////////////////// CONSTRUTOR  
   
   /**
	 * construtor
	 * @param UI_GUIElement element	referencia a janela principal da interface
	 * @param BW_Event prototype  prototipo dos eventos que serao exibidos
	 * @param Vector events  conjunto de eventos do tipo do prototype
	 */
   public AwarenessEventsWindow (UI_GUIElement element, BW_Event prototype, Vector events) {
	super(prototype.getName());   
      this.element = element;
      this.prototype = (BW_Event)prototype.clone();
      this.events = (Vector)events.clone();

      this.setSize(300, 250);
      addWindowListener (this);
      eventsList = new List (10);
      eventsList.setSize(this.getSize());
	Vector newList = mountList();
	for (int i=0; i<newList.size(); ++i)
	   eventsList.add((String)newList.elementAt(i));
      add(eventsList);
	eventsList.addItemListener(this);
      setVisible(true);
   }


   ////////////////////////  METODOS

   /** metodo mountList: gera a lista de eventos
    *  @return Vector lista de nomes dos eventos
    */
   public Vector mountList () {
      Vector list = new Vector();
	String eventDescription;
	BW_Member responsible;
      for (int i=0; i<events.size(); ++i) 
	{
	   eventDescription = new String ("Event: " + ((BW_Event)events.elementAt(i)).getDescription());
	   responsible = ((BW_Event)events.elementAt(i)).getResponsible();
	   eventDescription=eventDescription.concat(" - User: " + responsible.getName());
	   list.addElement(eventDescription);
	}
	return list;
   }


   /**
	 * inclui novos eventos na lista de eventos a serem exibidos
	 * @param Vector newEvents	lista de novos eventos a serem anexados aos demais
	 */
   public void newEvents(Vector newEvents)
   {
      for (int i=0; i<newEvents.size(); ++i)
      {
         events.addElement((BW_Event)newEvents.elementAt(i));
      }
      updateList();
   }


	/**
	 * redefine o conjunto de eventos exibidos
	 * @param Vector events	lista de eventos que substituira a anterior
	 */
   public void update(Vector events)
   {
      this.events = (Vector)events.clone();
	updateList();
   }

   /**
	 * permite alterar o prototipo dos eventos exibidos
	 * @param BW_Event newProt	novo prototipo de eventos
	 */
   public void setEventPrototype (BW_Event newProt)
   {
      prototype = (BW_Event)newProt.clone();
   }


   /**
	 * permite alterar a referencia ao elemento de exibicao (janela principal)
	 * @param UI_GUIElement control	novo elemento
	 */
   public void setUI_GUIElement (UI_GUIElement control)
   {
      element = control;
   }


   /**
	 * retorna o prototipo de eventos atualmente em exibicao
	 * @return BW_Event prototipo de eventos
	 */

   public BW_Event getEventPrototype ()
   {
      return prototype;
   }
   
   
   /**
	 * retorna informacoes especificas de um determinado evento selecionado
	 * @param int i	indice do evento selecionado na lista
	 * @return Vector lista de informacoes sobre o evento selecionado
 	 */
   public Vector getEventInfo(int i)
   {
	BW_Event selectedEvent;
	Vector info = new Vector();
      BW_Member user;
	BW_TimeInterval interval;
	String field;

	selectedEvent = (BW_Event)events.elementAt(i);
	info.addElement(new String("Event Name: " + selectedEvent.getName()));
	info.addElement(new String("Description: " + selectedEvent.getDescription()));
	user = selectedEvent.getResponsible();
	info.addElement(new String("Responsible: " + user.getName()));
	info.addElement(new String("Mail: " + user.getMail()));
        info.addElement(new String("Homepage: " + user.getHomePage()));
	info.addElement(new String("Details: " + selectedEvent.getDetails()));
	interval = selectedEvent.getInterval();
        Calendar c1 = interval.getBegin();
        info.addElement(new String("Begin: " + c1.get(Calendar.DATE) + "/" + (c1.get(Calendar.MONTH)+1) + "/" + c1.get(Calendar.YEAR) + " - " + c1.get (Calendar.HOUR_OF_DAY) + ":" + c1.get(Calendar.MINUTE) + ":" + c1.get(Calendar.SECOND)));
	Calendar c2 = interval.getEnd(); 
        info.addElement(new String("End: " + c2.get(Calendar.DATE) + "/" + (c2.get(Calendar.MONTH)+1) + "/" + c2.get(Calendar.YEAR) + " - " + c2.get (Calendar.HOUR_OF_DAY) + ":" + c2.get(Calendar.MINUTE) + ":" + c2.get(Calendar.SECOND)));

  	return info;
   }


   /**
	 * retorna o UI_GUIElement atualmente em uso
	 * @return UI_GUIElement 
	 */

   public UI_GUIElement getUI_GUIElement()
   {
      return element;
   }


   /**
	 * atualiza a lista de eventos em exibicao
	 */
   public void updateList () {
      eventsList = new List (12);
	Vector newList = mountList();
	for (int i=0; i<newList.size(); ++i)
	   eventsList.add((String)newList.elementAt(i));
      eventsList.repaint();
   }

   /**
	 * abre uma janela com as informacoes especificas de um evento selecionado
	 * @param int index	indice do evento, na lista de exibicao
	 */
   public void showEventInfo (int index) {
      int selected = index;
      Vector info = getEventInfo(selected);
      frameInfo = new Frame ("Event Detail");
      frameInfo.addWindowListener(this);
      frameInfo.setSize(300,200);
      eventInfo = new TextArea();
      eventInfo.setEditable(false);
      frameInfo.add(eventInfo);
      for (int i=0; i<info.size(); ++i)
	{
	   eventInfo.append((String)info.elementAt(i) + "\n");
	}
	frameInfo.setVisible(true);
   }

///////// metodo do ItemListener

   public void itemStateChanged (ItemEvent e) {
      int selected = ((Integer)e.getItem()).intValue();
      showEventInfo(selected); 
   }

/////////// metodos do WindowListener

   public void windowClosing (WindowEvent evt){
	Frame closedObject = (Frame)evt.getWindow();
        if (closedObject==this)
        {
           if (frameInfo!=null)
           {
              frameInfo.setVisible(false);
              frameInfo.dispose();
           }
        }
        closedObject.setVisible(false);
	closedObject.dispose();
   }

   
   public void windowClosed (WindowEvent evt) {
   }

   public void windowOpened (WindowEvent evt) {
   }

   public void windowIconified (WindowEvent evt) {
   }

   public void windowActivated (WindowEvent evt) {
   }

   public void windowDeactivated (WindowEvent evt) {
   }

   public void windowDeiconified (WindowEvent evt) {
   }

   public void actionPerformed (WindowEvent evt) {
   }

}