/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTOR LUIZ ANGELO BARCHET ESTEFANEL
 * CLASSE AwarenessMainWindow
 * PACOTE BW.UML
 * DATA DE CRIACAO 24 / 01 / 2001
 */


package BW.UML;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import BW.Interface.*;
import BW.kernel.*;
import BW.UML.*;
import BW.UML.ClickBar.*;


/**
 * classe AwarenessMainWindow - classe principal do ambiente de
 * visualizacao dos eventos registrados no framework BW
 * @author Luiz Angelo Barchet Estefanel
 * @version 1.0
 */

public class AwarenessMainWindow extends Frame implements UI_GUIElement, ActionListener, WindowListener {

  /** referencia  classe base da interface */
  private UI_Interface manager;
  
  /** lista de eventos a serem visualizados */
  private Hashtable UI_Events; 

  /** relacao entre tipos de eventos e suas cores */
  private Hashtable Events_Color;

  //////////// variaveis da interface grafica
  private MenuBar mb;
  private Menu memberMenu;
  private Menu eventsMenu;
  private Menu eventsMenu_Profile;
  private Menu exitMenu;
  private Menu helpMenu;
  private Scroller sc;
  private boolean isSet;

  private CheckboxMenuItem	menuItem;


  ////////////////  construtor
  
  
  /** construtor recebe apenas a referencia da UI_Interface do BW
   * @param	UI_Interface manager	gerente da exibicao dos eventos
   */
  public AwarenessMainWindow(UI_Interface manager) {

    super("Past Awareness Window");
    setSize(400, 280);

    this.menuItem = null;

    this.manager = manager;
    makeMenu();  
    
    resetAllEvents();

    sc = null;
    isSet = false;
    
    addWindowListener (this);
 }


 /** construtor recebe apenas a referencia da UI_Interface do BW
  * @param	UI_Interface manager	gerente da exibicao dos eventos
  */
  public AwarenessMainWindow() {

    super("Past Awareness Window");
    setSize(400, 280);

    this.menuItem = null;

    this.manager = null;
    makeMenu();

    resetAllEvents();

    sc = null;
    isSet = false;

    addWindowListener (this);
 }



 ////////////////// metodos


   /** limpa o hashtable que contem os vetores de eventos, 
   *   deixando apenas o indice dos prototipo
   */ 
   public void resetAllEvents () {
      UI_Events = new Hashtable();
      Events_Color = new Hashtable();

      Color color[] = new Color[9];
      color[0] = Color.blue;
      color[1] = Color.black;
	   color[2] = Color.green;
	   color[3] = Color.orange;
	   color[4] = Color.cyan;
	   color[5] = Color.red;
	   color[6] = Color.yellow;
	   color[7] = Color.magenta;
	   color[8] = Color.gray;	


      Vector prototypes = UML_BW_Mediator.getPrototypes(); 
      for (int i=0; i<prototypes.size(); ++i) {
         try {
	      UI_Events.put (this.mountKey((BW_Event)prototypes.elementAt(i)),new Vector());
	      Events_Color.put (this.mountKey((BW_Event)prototypes.elementAt(i)),color[i]);
	   } catch (NullPointerException np) {
              System.out.println("Ocorreu um null-pointer");
         }
      }
   }

   /** monta a chave que sera usada como indice no hashtable de eventos
   * @param BW_Event obj prototipo de evento que sera' incluido na lista de eventos
   * @return String chave do objeto
   */

   public String mountKey (BW_Event obj) {
		   Class classe = obj.getClass();
		   StringBuffer key = new StringBuffer();
		   //key.append ((obj.getClass().getName()).toUpperCase());
		   key.append ((obj.getClass().getName()));
		   key.append("-");
		   key.append(obj.getTypeId());	         	
		   return key.toString();
	}

  /** torna visivel a janela principal da interface 
  */
  public void Show ()
  {
     this.setVisible(true);
  }
  
  /** esconde a janela
  */
  public void Hide()
  {
     this.setVisible(false);
  }

  /** verifica se a janela esta visivel ou nao
   */
  public boolean IsVisible () {
  	return this.isVisible();
  }

  /** inicia e preenche os menus da janela, delegando suas funcoes
  */
  public void makeMenu () {
     mb = new MenuBar();
     eventsMenu = new Menu ("Events");
     eventsMenu.add(new MenuItem("Get Events"));
     eventsMenu_Profile = new Menu ("Profiles");
     eventsMenu_Profile.add(new MenuItem("Personal Profiles"));
     eventsMenu_Profile.add(new MenuItem("Personal Paper"));
     eventsMenu.add(eventsMenu_Profile);
     mb.add(eventsMenu);
     memberMenu = new Menu ("Member");
     memberMenu.add(new MenuItem("Search Info"));
     memberMenu.add(new MenuItem("Modify Info"));
     mb.add(memberMenu);
     exitMenu = new Menu ("Close");
     exitMenu.add(new MenuItem("Really?"));
     mb.add(exitMenu);
     helpMenu = new Menu ("Help");
     helpMenu.add(new MenuItem("Fundamental"));
     helpMenu.add(new MenuItem("Advanced"));
     mb.add(helpMenu);

     eventsMenu.addActionListener(this); 
     eventsMenu_Profile.addActionListener(this); 
	memberMenu.addActionListener(this); 
	exitMenu.addActionListener(this); 
	helpMenu.addActionListener(this); 
     

     setMenuBar(mb);

     System.out.println ("[DBG]AwarenessMainWindow.makeMenu: menus ok");
  }

   
   

   private void click_getEvents () {
      Notify();
   }

   private void click_personalProfiles() {
      AwarenessProfileWindow APW = new AwarenessProfileWindow(manager,manager.getMember(),true);     
   }

   private void click_personalPaper() {
      AwarenessProfileWindow APW = new AwarenessProfileWindow(manager,manager.getMember(),false);     
   }

   private void click_searchInfo() {
      AwarenessGroupInfoWindow AGIW = new AwarenessGroupInfoWindow(manager.getGroup());     
   }
 
   private void click_modifyInfo() {
      AwarenessUserInfoWindow AUIW = new AwarenessUserInfoWindow(manager,manager.getMember());     
   }

   private void click_fundamental() {
      String fundamental = new String("fundamental.txt");
      AwarenessHelpWindow AHW = new AwarenessHelpWindow(fundamental);
   }

   private void click_advanced() {
      String advanced = new String("advanced.txt");
      AwarenessHelpWindow AHW = new AwarenessHelpWindow(advanced);
   }


  /** metodo chamado pelos outros pacotes do framework BW, faz com que
  *   sejam atualizadas as listas de eventos que estao sendo exibidas
  */
  
  public void Notify ()
  {
      String key;
      Vector evtPointer; 

      try {
	      //Vector events = (Vector)manager.getEvents(this).clone();
	      Vector events = (Vector)manager.getEvents(this);
	      resetAllEvents();

	      for (int i=0; i<events.size(); ++i)
	      {
        	 key = mountKey((BW_Event)events.elementAt(i));
	         evtPointer = (Vector)UI_Events.get(key);
	         evtPointer.addElement(events.elementAt(i));
	      }
	      setBarObjects();
	
	      this.repaint();
	
      } catch (Exception e) {
	      	System.out.println ("No events available");
      }
  }

  
  /** permite a alteracao da referencia ao objeto UI_Interface que gerencia esta
   *  interface
   * @param UI_Interface manager o novo objeto que gerencia a interface grafica
   */

  public void setUI_Interface (UI_Interface manager)
  {
    this.manager = manager;
  }


  /** retorna a referencia ao objeto que gerencia esta interface grafica
   * @return UI_Interface manager
   */

  public UI_Interface getUI_Interface ()
  {
     return manager;
  }


  /** metodo que permitiria a inclusao de outros UI_GUIEvent na lista de controle
   *  devido 'as caracteristicas da aplicacao (CUTE), este metodo nao foi implementado
   *  por nao haver a necessidade de outros UI_GUIEvent
   * @param UI_GUIEvent guievent 
   * @param BW_Event prototype
   */

  public void addUI_GUIEvent (UI_GUIEvent guievent, BW_Event prototype)
  {
     /// nao faz nada, pois a estrutura hashtable nao foi construida para suportar diversos GUIEVents
  }

  private void exit() {
     try {
     	menuItem.setState (false);
     } catch (Exception e) {
     }

    setVisible(false); // hide the Frame
    //dispose(); // tell windowing system to free resources
  }

  /** seta o item de menu dentro do groupware COPSE/CUTE relacionado a esta janela */
   public void setMenuItem (MenuItem menuItem) {
   	this.menuItem = (CheckboxMenuItem)menuItem;
   }


  /** metodo invocado por um objeto bar, dispara a exibicao de detalhes de um determinado evento
   * @param BW_Event prototype prototipo do evento a ser exibido em detalhes
  */
  public void showEvent (BW_Event prototype) {
     String chave = this.mountKey(prototype);
     Vector eventList = (Vector)UI_Events.get(chave);
     AwarenessEventsWindow eventWindow = new AwarenessEventsWindow(this,prototype,eventList);
  }

   /** atualiza a exibicao do grafico de barras
   */
   private void setBarObjects () {

      Vector evtPointer; 

      /*
      Color color[] = new Color[9];
      color[0] = Color.blue;
      color[1] = Color.black;
	   color[2] = Color.green;
	   color[3] = Color.orange;
	   color[4] = Color.cyan;
	   color[5] = Color.red;
	   color[6] = Color.yellow;
	   color[7] = Color.magenta;
	   color[8] = Color.lightGray;
	
      int colorIndex = 0;
      */

      Vector dados = new Vector();
      
	try {
	   Enumeration e1 = UI_Events.elements();
	   while (e1.hasMoreElements()) {
		evtPointer = (Vector) e1.nextElement();
            if (evtPointer.size()>0)
            {
		   //bar elementBar = new bar(this,(BW_Event)evtPointer.elementAt(0), String.valueOf(evtPointer.size()),((BW_Event)evtPointer.elementAt(0)).getName(), color[colorIndex],"SOLID");
		   BW_Event modelo = (BW_Event)evtPointer.elementAt(0);
		   bar elementBar = new bar(this,modelo, String.valueOf(evtPointer.size()),
		   				modelo.getName(), (Color) Events_Color.get (this.mountKey(modelo)),"SOLID");
		   dados.addElement(elementBar);
		   //colorIndex ++;
		}
            
         }
	   if (!isSet) {
	
		sc = new Scroller(this.getSize(),dados);
		sc.setSize(this.getSize());
		this.add(sc);
		repaint();
		isSet=true;
	   }
         else {
            sc.setObjects (dados);
         }
	} catch (NullPointerException npe) {
	       System.out.println("catch" + npe);	}

	repaint();
      sc.repaint ();
   }

  
  
 ////////  metodos da classe ItemListener  
  
  public void itemStateChanged (ItemEvent e)
  {
     int state = e.getStateChange();
     if (state==ItemEvent.SELECTED)
	this.Show();
     else {
	if (state==ItemEvent.DESELECTED)
	   this.Hide();
     }
   /*  try {
     	this.menuItem = (CheckboxMenuItem) e.getItem();
     } catch (Exception exp) {
     }
   */  	
  }

  /////// metodos da classe ActionListener
  
  public void actionPerformed(ActionEvent e) {
      String item = e.getActionCommand();
      if (item.equals("Get Events"))
         click_getEvents();
      else
      if (item.equals("Personal Profiles"))
         click_personalProfiles();
      else
      if (item.equals("Personal Paper"))
         click_personalPaper();
      else
      if (item.equals("Search Info"))
         click_searchInfo();
      else
      if (item.equals("Modify Info"))
         click_modifyInfo();
      else
      if (item.equals("Really?"))
	   exit();
      else
	if (item.equals("Fundamental"))
	   click_fundamental();
      else
	if (item.equals("Advanced"))
	   click_advanced();
  }
  
  
  /////////// metodos da classe WindowListener
  
  public void windowClosing (WindowEvent evt){
     exit();
   /*  try {
     	menuItem.setState (false);
     } catch (Exception e) {
     }
     this.setVisible(false);
     this.dispose();
    */
  }

  public void windowClosed (WindowEvent evt) {
  	//System.out.println ("[DBG]AwarenessMainWindow: WINDOWCLOSED");
  }

  public void windowOpened (WindowEvent evt) {
  }

  public void windowIconified (WindowEvent evt) {
  	//System.out.println ("[DBG]AwarenessMainWindow: WINDOWICONIFIED");
  }

  public void windowActivated (WindowEvent evt) {
  	//System.out.println ("[DBG]AwarenessMainWindow: WINDOWACTIVATED");
  }

  public void windowDeactivated (WindowEvent evt) {
  	//System.out.println ("[DBG]AwarenessMainWindow: WINDOWDEACTIVATED");
  }

  public void windowDeiconified (WindowEvent evt) {
  	//System.out.println ("[DBG]AwarenessMainWindow: WINDOWDEICONIFIED");
  }

}