/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * Classe BW_AwarenessProfile
 * PACOTE BW.KERNEL
 * DATA DE CRIACAO 12 / 11 / 2000
 */

package BW.kernel;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

import BW.typeIdDefault;

/**
 * Classe BW_AwarenessProfile: descreve a nocao do conjunto de
 * profiles do usuario dentro do framework BW
 *
 * @author Manuele Kirsch Pinheiro
 * @version 1.2
 */
public class BW_AwarenessProfile extends BW_Profile {

     ///////////////////////////////// ATRIBUTOS //////////////////

     /** pool de profiles pessoais */
     Vector personal;

     /** profiles pessoais por papel no formato (papel,profile)*/
     Hashtable personalpaper;


     ///////////////////////////////// CONSTRUTORES //////////////

     /**
       * construtor
       * @param	int typeId	identificador de tipo
       * @param int objId	identificador do objeto
       */
     public BW_AwarenessProfile (int typeId, int objId) {
     		super (typeId, objId);
                this.personal = new Vector();
                this.personalpaper = new Hashtable();
     }


     public BW_AwarenessProfile () {
     		//super (typeIdDefault.BW_AWARENESSPROFILE,typeIdDefault.BW_AWARENESSPROFILE+typeIdDefault.OBJID);
     		super (typeIdDefault.BW_AWARENESSPROFILE,typeIdDefault.BW_AWARENESSPROFILE);
                this.personal = new Vector();
                this.personalpaper = new Hashtable();     		
     }



     ///////////////////////////////// METODOS ////////////////////

     //////// manipulacao pool personal - profiles pessoais

     /**
      * insere um profile no pool de profiles pessoais
      * @param	BW_Profile prof	novo profile a ser inserido
      */
     public void insertPersonalProfile (BW_Profile prof) {
     //public void insertProfile (BW_Profile prof) {
     		personal.addElement (prof);
     }

     /**
      * verifica se um profile esta no pool de profiles pessoais
      * @param BW_Profile	profile a ser verificado
      */
      public boolean verifyPersonalProfile (BW_Profile prof) {
      //public boolean verifyProfile (BW_Profile prof) {
      		return personal.contains (prof);
      }

      /**
       * remove um profile especifico do pool de profiles pessoais
       * @param BW_Profile prof	profile a ser removido
       */
       public void removePersonalProfile (BW_Profile prof) {
       		personal.removeElement (prof);
       }

      /**
       * busca o n-esimo profile pessoal
       * @param		int n	posicao do profile pessoal desejado
       * @return        BW_Profile	n-esimo profile pessoal
       *		null se o indice i for invalido
       */
      public BW_Profile getPersonalProfile (int n) {
      	   BW_Profile prof = null;
      	   try {
      		prof =  (BW_Profile) personal.elementAt (n);
      	   } catch (ArrayIndexOutOfBoundsException aie)	{
      	        prof = null;      	
      	   }
      	   return prof;
      }

      /**
       * retorna o pool completo de profiles pessoais
       * @return Vector		pool de profiles pessoais
       */
      public Vector getPersonalProfile () {
      		return (Vector) personal.clone();
      }


      /**
       * seta o pool completo de profiles pessoais
       * @param Vector pool	novo pool de profiles pessoais
       */
      public void setPersonalProfile (Vector pool)  {
      		//this.personal = null;
      		this.personal = (Vector) pool.clone();
      }



     //////// manipulacao pool personalpaper - profiles pessoais por papel

     /**
      * insere um profile no pool de profiles pessoais por papel
      * @param	BW_Profile prof	novo profile a ser inserido
      * @param	BW_Paper paper	papel associado ao profile
      */
     public void insertPersonalPaperProfile (BW_Profile prof,BW_Paper paper) {
                personalpaper.put (paper,prof);     		
     }


     /**
      * remove o profile para um dado papel do pool de profiles pessoais por papel
      * @param BW_Paper paper	papel cujo profle se deseja remover do pool
      */
     public void removePersonalPaperProfile (BW_Paper paper) {
     		personalpaper.remove (paper);
     }


     /**
      * verifica se ha um profile para um determinado papel no pool de
      * profiles pessoais por papel
      * @param BW_Paper		papel a ser verificado
      */
      public boolean verifyPersonalPaperProfile (BW_Paper paper) {
      		return personalpaper.containsKey(paper);
      }

      /**
       * busca o profile pessoal do papel especificado
       * @param		BW_Paper paper	papel do profile desejado
       * @return        BW_Profile	profile pessoal do papel dado
       *		null se o papel for invalido
       */
      public BW_Profile getPersonalPaperProfile (BW_Paper paper) {
      	   BW_Profile prof = null;
      	   prof =  (BW_Profile) personalpaper.get (paper);
      	   return prof;
      }

      /**
       * retorna o pool completo de profiles pessoais por papel
       * indexado em uma hash de acordo com o papel, formando
       * tuplas (BW_Paper,BW_Profile)
       * @return Hashtable		pool de profiles pessoais por papel
       * @see BW_Paper
       * @see BW_Profile
       */
      public Hashtable getPersonalPaperProfile () {
      		return (Hashtable) personalpaper.clone();
      }


      /**
       * seta o pool completo de profiles pessoais por papel, atraves de
       * uma Hash com os papeis relacionados a seus respectivos profiles
       * pessoais
       * @param Hashtable pool	novo pool de profiles pessoais por papel
       * @see #getPersonalPaperProfile
       */
      public void setPersonalPaperProfile (Hashtable pool)  {
      		//this.personalpaper = null;
      		this.personalpaper = (Hashtable) pool.clone();
      }



      /////////// metodos default
   	
   	
      /** metodo getFields retorna um Hashtable com os
       * pares (atributo,valor) do objeto
       * @return Hashtable  - atributo,valor
       */
       public Hashtable getFields () {
       		Hashtable ht = super.getFields();
       		ht.put ("personal",(Vector)personal.clone());
       		ht.put ("personalpaper",(Hashtable)personalpaper.clone());
                return (ht);
       }


	/** metodo setFields seta os atributos do cobjeto
	 * com base em uma Hashtable no mesmo formato retornado
	 * pelo metodo getFields. Retorna true se tudo correu
	 * bem, e false caso contrario
	 * @param	Hashtable fields	atributos a serem preenchidos
	 * @return	boolean	true - se foi possivel ajustar os atribuitos
	 *			false - caso contrario
	 * @see	#getFields
	 */
       public boolean setFields (Hashtable fields) {
			Hashtable bkp = super.getFields();
       		boolean ret = super.setFields(fields);

       		if (ret && fields.containsKey ("personal")
       			&& fields.containsKey ("personalpaper")) {
       			//System.out.println ("********* setting ");
       			this.setPersonalProfile ((Vector)fields.get("personal"));
       			this.setPersonalPaperProfile ((Hashtable)fields.get("personalpaper"));
       			ret = true;	
       		}
       		else {
				ret = false;
				super.setFields (bkp);
			}
       		return ret;       			
       }


       public Object clone () {
       		BW_AwarenessProfile copy = new BW_AwarenessProfile (this.getTypeId(),this.getObjId());
       		copy.setInterval (this.getInterval());
       		copy.setEvents (this.getEvents());
       		copy.setPersonalProfile (this.getPersonalProfile());
       		copy.setPersonalPaperProfile (this.getPersonalPaperProfile());
       		return (Object) copy;
       }


       public String toString () {
       		StringBuffer sb = new StringBuffer();
       		
       		String s = super.toString();
                int i = s.length();
       		char c = s.charAt (i-1);
       		sb.append (s.substring(0,i-1));
       		
       		sb.append (", Personal Profiles=[");
       		Enumeration e = personal.elements();
       		while (e.hasMoreElements()) {
       		     BW_Profile p = (BW_Profile) e.nextElement();
       		     sb.append (p.toString());
       		}
                sb.append ("]");

       		sb.append (", Personal Paper Profiles=[");
       		e = personalpaper.elements();
       		while (e.hasMoreElements()) {
      		     BW_Profile p = (BW_Profile) e.nextElement();
       		     sb.append (p.toString());
       		}
                sb.append ("]");

       		sb.append (c);
       		
       		return sb.toString();
       }

       /////////////// metodos especificos

       /**
        * metodo getAllPersonalEvents retorna o conjunto completo de eventos
        * dos profiles pessoais presentes neste awareness profile.
        * Esta Hash e composta pelas tuplas (chave,evento), onde a chave,
        * obtida com ometodo mountKey, e formada pelo nome da classe e o
        * typeid do evento.
        * @see BW.kernel.BW_Event#events
        * @see BW.kernel.BW_Event#mountKey
        * @return Hashtable	hash com todos os eventos dos profiles pessoais
        */
        public Hashtable getAllPersonalEvents () {
        	//pega os eventos diretamente colocados neste profile
                Hashtable evts = (Hashtable)this.events.clone();

		//processa os eventos dos profiles no pool pessoal
		for (int i=0; i<personal.size();i++) {			
			BW_Profile prof = this.getPersonalProfile (i);
			if (prof == null)
				continue;
			Hashtable ht = prof.getEvents();
		        Enumeration elem = ht.elements();
		        while (elem.hasMoreElements()) {
		        	BW_Event e = (BW_Event) elem.nextElement();
		        	evts.put (this.mountKey(e),e);
		        }
		}				
		return evts;
        }


       /**
        * metodo getAllPaperEvents retorna todos os eventos presentes
        * em todos os profiles por papel deste awareness profile.
        * @return Hashtable	hash com todos os eventos dos profiles pessoais por papel
       */
       public Hashtable getAllPaperEvents () {
        	//processa os eventos dos profiles no pool pessoal por papel
        	Hashtable evts = new Hashtable();
		Enumeration papers = personalpaper.elements();
		while (papers.hasMoreElements())	 {	
			BW_Profile prof = (BW_Profile) papers.nextElement();		
			Hashtable ht = prof.getEvents();
		        Enumeration elem = ht.elements();
		        while (elem.hasMoreElements()) {
		        	BW_Event e = (BW_Event) elem.nextElement();
		        	evts.put (this.mountKey(e),e);
		        }
		}

		return evts;
	}
	
	/**
	 * metodo getGlobalInterval retorna um intervalo global, que
	 * engloba os intervalos de todos os profiles do pool de
	 * profiles pessoais.
	 * @return BW_TimeInterval	intervalo global	
	 * @see	BW_TimeInterval#merge
	 */
	 public BW_TimeInterval getGlobalInterval () {	
         	//pega o interval local
         	BW_TimeInterval global = this.getInterval();
         	
         	//para cada profile do pool de profiles pessoais
         	Enumeration enum = personal.elements();
         	while (enum.hasMoreElements()) {
         		//pega o intervalo e faz merge com o global
         	        BW_TimeInterval i = ((BW_Profile)enum.nextElement()).getInterval();
         	        //System.out.println ("[DBG] BW_AWARENESS_PROFILE.GETGLOBALINTERVAL: processing "+i+" GLOBAL "+global);
         	        global = global.merge (i);
         	}
         	
         	//retorna o global
	 	return global;
	 }
	
}