/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE BW_Group
 * PACOTE BW.KERNEL
 * DATA DE CRIACAO 16 / 07 / 2000
 */

package BW.kernel;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;

import BW.typeIdDefault;

/**
 * classe BW_Group representa a propria ideia
 * do grupo dentro do BW
 * @author Manuele K P
 * @version 1.2
 */
public class BW_Group extends BW_Element {

    ///////////////////////////////// ATRIBUTOS //////////////

	/** nome ou designaco para o grupo */
	String name;

	/** rapida descricao sobre o grupo */
	String description;

	/** participantes do grupo, classificados por uid*/
	Hashtable members;
	
	/** conjunto de papeis do do grupo
	 * o conjunto de papeis do grupo define quais os papeis possiveis
	 * de serem desempenhados pelos membros do grupo	
	 */
	Hashtable papers;

    //////////////////////////////// CONSTRUTORES ////////////

	/** construtor default - passa o id e o nome do grupo
         * @param String name 	nome do grupo
	 * @param int objId	identificador do grupo
	 */
	public BW_Group (String name, int objId) {
         	super (typeIdDefault.BW_GROUP,objId);
		this.name = new String (name);
		this.description = new String();
		this.members = new Hashtable();
		this.papers = new Hashtable();
	}

	/** construtor passando o identificador do grupo
	 * e um identificador de tipo para o grupo
         * @param int typeId	identificador de tipo do grupo
	 * @param int objId	identificador do grupo
	 */
	public BW_Group (int typeId, int objId) {
         	super (typeId,objId);
		this.name = new String ();
		this.description = new String();
		this.members = new Hashtable ();
		this.papers = new Hashtable();
	}
	
	
	/** construtor sem parametros
	 * seta todos os atributos para seus valores default
	 */
	public BW_Group () {
         	super (typeIdDefault.BW_GROUP,typeIdDefault.BW_GROUP);
		this.name = new String ("default");
		this.description = new String();
		this.members = new Hashtable();
		this.papers = new Hashtable();
	}
	
	

    /////////////////////////////// METODOS //////////////////
	/* monte a chave usada na hashtable members */
	public Object mountKey (BW_Member obj) {
            Integer key = new Integer(obj.getObjId());
	    return (Object) key;
	}
	
	/** monta a chave usada para referenciar o usuario */
	public Object  mountKey (int uid) {
            Integer key = new Integer(uid);
	    return (Object) key;
	}

	
	/* monte a chave usada na hashtable papers */
	public Object mountKey (BW_Paper obj) {
            String key = new String(obj.getName());
	    return (Object) key;
	}
	
	/** monta a chave usada para referenciar um papel pelo nome */
	public Object  mountKey (String obj) {
            String key = new String(obj);
	    return (Object) key;
	}
	
	
	/** atribui um novo nome ao grupo
	 * @param String newname	novo nome
	 */
	public void setName (String newname) {
         	this.name = new String (newname);
	}

	/** retorna o nome do grupo
	 * @param STRING	NOME DO GRUPO
	 */
	public String getName () {
         	return new String (this.name);
	}

	/** atribui uma nova descricao para o grupo
	 * @param String newdescrip	nova descricao
	 */
	public void setDescription (String newdescrip) {
         	this.description = new String (newdescrip);
	}

	/** retorna a descricao do grupo
	 * @param String	descricao do grupo
	 */
	public String getDescription () {
         	return new String (this.description);
	}


	////////////////////////// METODOS SOBRE MEMBROS /////////////////////////
	
	/** inclui um membro ao grupo. Retornar false caso
	 * o uid do novo membro (identificador do objeto
	 * BW_Member) ja exita no grupo
         * @param BW_Member newmb	novo membro a ser adicionado
	 * @return	boolean		true se o membro foi incluido
	 * 		boolean		false se o membri ja existia 	
	 */
	public boolean insertMember (BW_Member newmb) {	
		Object key = this.mountKey(newmb);
         	if (!members.containsKey(key)) {
                 	members.put (key,newmb);
			return true;
		} else return false;
	}

	/** verifica se um membro se encontra no grupo
	 * @param BW_Member member	membro a ser pesquisado
	 * @return	boolean	true se o membro esta no grupo
	 *			false caso contrario
	 */
	public boolean verifyMember (BW_Member member) {
		Object key = this.mountKey(member);
		if (this.members.containsKey(key)) return true;
		else return false;
	}

	/** verifica se um membro se encontra no grupo
	 * @param int member	uid do membro a ser pesquisado
	 * @return  boolean	true se o membro esta no grupo
	 *			false caso contrario
	 */
	public boolean verifyMember (int member) {
		Object key = this.mountKey(member);
		if (this.members.containsKey(key)) return true;
		else return false;
	}
	/** remove um membro do grupo
	 * @param BW_member member	membro a ser removido
	 */
	public void removeMember (BW_Member member) {
		Object key = this.mountKey(member);
		this.members.remove (key);
	}


	/** seta todo o conjunto de membros de uma unica vez
	 * @param 	Hashtable cjmemb	conjunto de membros
	 */
	public void setMembers (Hashtable cjmemb) {
         	this.members = (Hashtable) cjmemb.clone();
	}	

	/** retorna uma hashtable na forma (uid,BW_Member), com os membros do grupo
	 * @return 	Hashtable	hash de todos os membros do grupo
	 */
	public Hashtable getMembers () {
		return (Hashtable)members.clone();
	}


	/** retorna um unico membro do grupo.
	 * @param 	int uid	identificador numerico (uid) do membro
	 * @return	BW_Member	membro do grupo
	 *				null se o membro nao pertencer ao grupo
	 */
	public BW_Member getAMember (int uid) {
         	Object key = mountKey (uid);
                return (BW_Member) members.get (key);
	}


	/** altera as informacoes de um unico membro dentro do grupo.
	 * Nao faz nada caso o membro nao exista no grupo
	 * @param	int uid		uid do membro a ser alterado
	 * @param	BW_Member info	novas informacoes para o membro
	 */
	public void setAMember (int uid, BW_Member info) {
		Object key = this.mountKey(info);
         	if (members.containsKey(key)) {
                 	members.put (key,info);
		}
		//System.out.println ("[DBG] BW_GROUP.SETAMEMBER: MEMBER SET "+info);
	}
	
	
	
	////////////////////// METODOS SOBRE PAPEL //////////////////////
		
	
	/** insere um papel ao conjunto de papeis do grupo
	 * o conjunto de papeis do grupo define quais os papeis possiveis
	 * de serem desempenhados pelos membros do grupo
	 * @param 	BW_Paper newpaper	novo papel do grupo
	 */
	public void insertPaper (BW_Paper newpaper) {
		Object key = this.mountKey (newpaper);
		papers.put (key,newpaper);
	}

	/** remove um papel ao conjunto de papeis do grupo
	 * @param 	BW_Paper paper	papel a ser removido
	 * @see	#papers
	 */
	public void removePaper (BW_Paper paper) {
		Object key = this.mountKey (paper);
		papers.remove (key);
	}

	/** remove um papel ao conjunto de papeis do grupo
	 * CUIDADO! Remover um papel do grupo nao o removera
	 * da lista de papeis dos membros que podem desempenha-lo!
	 * @see	BW_Member#papers
	 * @param 	String paper	papel a ser removido
	 * @see	#papers
	 */
	public void removePaper (String paper) {
		Object key = this.mountKey (paper);
		papers.remove (key);
	}
	
	/** verifica se um determinado papel esta definido no grupo
	 * @param	BW_Paper paper	papel a ser verificado
	 * @return	boolean	true se o papel existe no grupo;
	 *			false caso contrario
	 */
	public boolean verifyPaper (BW_Paper paper) {
		Object key = this.mountKey (paper);
		return this.papers.containsKey(key);	
	}

	/** verifica se um determinado papel esta definido no grupo
	 * @param	String paper	papel a ser verificado
	 * @return	boolean	true se o papel existe no grupo;
	 *			false caso contrario
	 */
	public boolean verifyPaper (String paper) {
		Object key = this.mountKey (paper);
		return this.papers.containsKey(key);	
	}
		

	/** retorna um unico papel dentro do conjunto de papeis
	 *  definidos para o grupo.
	 *  @param 	String paper	papel a ser retornado
	 *  @return	BW_Paper	papel solicitado
	 *				null caso o papel nao esteja definido no grupo
	 */
	public BW_Paper getAPaper (String paper) {
		Object key = this.mountKey (paper);
		return (BW_Paper) this.papers.get (key);
	}

	/** altera os valores para um unico papel do grupo.
	 * Nao faz nada caso o papel nao esteja cadastrado no grupo.
	 * @param	String paper	atual nome do papel
	 * @param	BW_Paper info	novas informacoes do papel
	 */
	public void setAPaper (String paper, BW_Paper info) {
		Object key = this.mountKey (paper);
		if (this.papers.containsKey (key)) {
		 	this.papers.put (key,info);
		}
	}

        /** retorna o conjunto de papeis definido para o grupo
         * @return 	Vector	papeis definidos no grupo
         */
        public Vector getPapers () {
		Enumeration pps = papers.elements();
		Vector ret = new Vector ();
		while (pps.hasMoreElements ()) {
			ret.addElement ((BW_Paper) pps.nextElement());			
		}
		return ret;
        }

        /** seta o conjunto completo de papeis do grupo
         * @param	Vector papersset vetor com todos os papeis do grupo
         * @see #getPapers
        */
        public void setPapers (Vector papersset) {
        	this.papers = new Hashtable();
        	Enumeration pps = papersset.elements();
        	while (pps.hasMoreElements()) {
        		BW_Paper p = (BW_Paper)pps.nextElement();
        		Object key = this.mountKey (p);
        		this.papers.put (key,p);
        	}

        }


        ////////////////////////// METODOS DEFAULT //////////////////////////
	

   	/** metodo getFields retorna um Hashtable com os
	 * pares (atributo,valor) do objeto
    	 * @return Hashtable  - atributo,valor
    	 */
	public Hashtable getFields () {
		Hashtable ht = super.getFields();
		ht.put ("name",this.getName());
		ht.put ("description",this.getDescription());
		ht.put ("members",this.getMembers());
		ht.put ("papers",this.getPapers());
		return ht;			
	}
	
	/** metodo setFields seta os atributos do cobjeto
	 * com base em uma Hashtable no mesmo formato retornado
	 * pelo metodo getFields. Retorna true se tudo correu
	 * bem, e false caso contrario
	 * @param	Hashtable fields	atributos a serem preenchidos
	 * @return	boolean	true - se foi possivel ajustar os atribuitos
	 *			false - caso contrario
	 * @see	#getFields
	 */
       public boolean setFields (Hashtable fields) {
			Hashtable bkp = super.getFields();
       		boolean ret = super.setFields(fields);

       		if (ret && fields.containsKey ("name")
       			&& fields.containsKey ("description")
       			&& fields.containsKey ("members")
       			&& fields.containsKey ("papers")) {
       			//System.out.println ("********* setting ");
       			this.setName ((String)fields.get("name"));
       			this.setName ((String)fields.get("description"));
       			this.setMembers ((Hashtable)fields.get("members"));
       			this.setPapers ((Vector)fields.get("papers"));
       			ret = true;	
       		}
			else  {
				ret = false;
				super.setFields (bkp);
			}
       		

       		return ret;       			
       }	
	
	
	

	/** retorna um outro objeto BW_Group com todos os
	 * atributos iguais, exceto o objId que e acrescido de 1
	 */
	public Object clone () {
         	BW_Group  ng = new BW_Group (this.getTypeId(),this.getObjId());
		ng.setName (this.getName());
		ng.setDescription (this.getDescription());
		ng.setMembers (this.getMembers());
		ng.setPapers (this.getPapers());
		return (Object)ng;
	}
		
}
