/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE BW_Register
 * PACOTE BW.KERNEL
 * DATA DE CRIACAO 15 / 07 / 2000
 */

package BW.kernel;

import java.util.Hashtable;

import BW.typeIdDefault;


/**
 * a classe BW_Register mantem o registro dos eventos que
 * serao observados e passados para o BW.
 * Trata-se do registro dos eventos que sao de interesse
 * do groupware base, e sobre os quais sera feita a
 * contextualizacao (awareness) posteriormente
 * @author Manuele Kirsch Pinheiro
 * @version 1.2
 */
public class BW_Register extends BW_Element {

         ///////////////////////////////// ATRIBUTOS //////////////

	/** lista dos eventos registrados
	 * Esta lista e um Hashtable dos seguintes valores:
	 * A chave e uma string formada pelo nome da
	 * classe do evento mais o sinal de "-" (hifem), mais
	 * o typeId da classe;
	 * e o valor e o proprio typeId do evento
	 */
	Hashtable events;
        //optou-se assim, pq hashtables nao suportam chaves duplicadas
	//e priorizou-se uma busca rapida por valores, logo uma hash e o ideal

        //////////////////////////////// CONSTRUTORES ////////////
	public BW_Register () {
        	super(typeIdDefault.BW_REGISTER,typeIdDefault.BW_REGISTER);
		events = new Hashtable();
	}

         /////////////////////////////// METODOS //////////////////

	/** monta a chave usada na hash que mantem os eventos registrados
	 * A chave e uma string formada pelo nome da
	 * classe do evento mais o sinal de "-" (hifem), mais
	 * o typeId da classe;
	 * e o valor e o proprio typeId do evento
	 */
	public String mountKey (BW_Event obj) {
		   Class classe = obj.getClass();
		   StringBuffer key = new StringBuffer();
		   //key.append ((obj.getClass().getName()).toUpperCase());
		   key.append (obj.getClass().getName());
		   key.append("-");
		   key.append(obj.getTypeId());	         	
		   return key.toString();
	}

	/** inclui um tipo de evento no registro, a partir do
	 * objeto passado como parametro.
	 * Retorna true se insercao foi realizada, e false caso contrario
	 * @param 	BW_Event obj	objeto cujo tipo se deseja registrar
	 * @return	boolean 	true - insercao ok
	 */
	public boolean insertEvent (BW_Event obj){
		boolean r=false;
         	try {
		   events.put (this.mountKey(obj),new Integer(obj.getTypeId()));
		   r=true;
		} catch (NullPointerException np) {
                   //caso obj = null, retorna false
		   r=false;
		}
		return r;
	}

	/** verifica se um tipo de objeto esta registrado
	 *  @param	BW_Event obj	objeto cujo tipo esta ou nao registrado
	 *  @return	boolean	 retorna true se o tipo do objeto obj esta registrado
	 */
	public boolean verifyEvent (BW_Event obj) {
	     boolean r=false;
             try {	
		if (events.containsKey(this.mountKey(obj))) {
			r=true;
		}
	     } catch (NullPointerException np) {
              	r=false;
	     }
	     return r;
	}


        /** remove um tipo de evento registrado da lista dos monitoraveis
	 * @param 	BW_Event obj	objeto cujo tipo vai ser retirado da lista
	 * @return	boolean		true - tipo foi removido corretamente
	 */
	public boolean removeEvent (BW_Event obj) {
		boolean r=false;
         	try {
                  events.remove (this.mountKey(obj));
		  r=true;
		} catch (NullPointerException np){
			r=false;
		}
		return r;
	}	

	/** retorna uma hashtable com os eventos registrados
	 * Atencao! Esta hash e formada pelos pares (eventClassName-eventTypeId, eventTypeId)
	 * @return Hashtable 	tipos de eventos registrados
	 */
	public Hashtable getEvents () {
         	return (Hashtable)events.clone();
	}


	/** seta todo o conjunto de eventos de uma unica vez
	 * @param 	Hashtable cjevt	conjunto de eventos
	 */
	public void setEvents (Hashtable cjevt) {
         	this.events = (Hashtable) cjevt.clone();
	}	

	
   	/** metodo getFields retorna um Hashtable com os
	 * pares (atributo,valor) do objeto
    	 * @return Hashtable  - atributo,valor
    	 */
	public Hashtable getFields () {
		Hashtable ht = super.getFields();
		ht.put ("events",this.getEvents());
		return ht;			
	}

	/** metodo setFields seta os atributos do cobjeto
	 * com base em uma Hashtable no mesmo formato retornado
	 * pelo metodo getFields. Retorna true se tudo correu
	 * bem, e false caso contrario
	 * @param	Hashtable fields	atributos a serem preenchidos
	 * @return	boolean	true - se foi possivel ajustar os atribuitos
	 *			false - caso contrario
	 * @see	#getFields
	 */
	 public boolean setFields (Hashtable fields) {
		Hashtable bkp = super.getFields();
	 	boolean result = super.setFields(fields);
	 	
                if (result) {
                    if (fields.containsKey("events")) {
                    	this.events = (Hashtable)((Hashtable) fields.get("events")).clone();
                    	result = true;
                    }
                    else  {
				result = false;
				super.setFields (bkp);
			  }
                }
	 	return result;  	
	 } 		
}
