/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE FC_ST_CONTEXT
 * PACOTE BW.STORAGE
 * DATA DE CRIACAO 21 / 07 / 2000
 */

package BW.storage;

import BW.kernel.*;
import BW.control.*;

import java.util.Vector;
//import java.util.Hashtable;


/**
 * classe FC_ST_Context faz o interfaceamento entre
 * as camadas de controle (pacote control) e a
 * camada de armazenamento (pacote storage), com as
 * informacoes fluindo no sentido camada de armazenamento
 * (storage) para a camada de controle (control)
 * @autor Manuele K P
 * @version 1.0
 */
public class FC_ST_Context {

	///////////////////////////////////////
	////// FALTA MUITA COISA AINDA  //////
	//////////////////////////////////////

	//////////////////////////////// ATRIBUTOS //////////////////////////////
	
	/** gerente para contextualizacao */	
        CL_Awareness awareness;

        /** objeto de controle da camada de armazenamento*/
        ST_Controler control;

        /** objeto responsavel pela estrategia de armazenamento */
        ST_Strategist strgy;


        //////////////////////////// CONSTRUTOR ////////////////////////////////

        public FC_ST_Context () {
        	awareness = null;
        	control = null;
        	strgy = null;
        }
	

        ////////////////////////// METODOS //////////////////////////////////

	/////////////////// SAVES
	
	/** executa um statement SQL junto a base de dados
	 * @param 	String sql	String com o statement SQL
	 * @return	boolean		true se o statement foi executado com sucesso,
	 *				false, caso contrario.
	 */
	public boolean save (String sql) {
		boolean ret = false;
		try {
			ret = this.control.save (sql);
		}catch (Exception e) {
		
		}
		return ret;
	}

		
	/////////////////////METODOS READ
	
        /** le um objeto, identificado pelo paramentro
	 * objId passado, da classe BW_Element identificad
	 * pelo id de tipo passado como parametro.
	 * @param	int typeId	identificador de tipo da classe
	 * @param	int objId	identificador do objeto
	 * @return	BW_Element	objeto cujo objId e typeId foram passados
	 */
	public BW_Element read (int typeId, int objId) {
          	BW_Element ret = null;
          	try {
          		BW_Element elm = new BW_Element (typeId,objId);
          		ret = this.control.read (elm);
          	}catch (Exception e) {
          	
          	}
          	return ret;
	}

	
	/** executa uma QUERY SQL junto a base de dados
	 * @param 	String sql	query a ser executada
	 * @return	Vector 		resultados lidos da query (poder ser null)
	 */
	public Vector read (String sql) {
		Vector ret = null;
		try {
			ret = this.control.read (sql);
		} catch (Exception e) {
			ret = null;
		}
		return ret;
	}
	
	
	/** faz a carga atualizada de uma unica instancia generica de BW_Element
	 * @param	BW_Element what	instancia a ser atualizada
	 * @return	BW_Element	instancia lida da base de dados
	 */
	public BW_Element read (BW_Element what) {
		BW_Element ret = null;
		try {
			ret = this.control.read (what);
		} catch (Exception e) {
			ret = null;
		}
		return ret;
	}
	
		
	/** faz a carga atualizada de um unico evento ocorrido
	 * @param	BW_Event what	instancia a ser atualizada
	 * @return	BW_Event	instancia lida da base de dados
	 */
	public BW_Event read (BW_Event what) {
		BW_Event ret = null;
		try {
			ret = this.control.read (what);
		} catch (Exception e) {
			ret = null;
		}
		return ret;
	}
	

	
	/** faz a carga atualizada de uma unica instancia de BW_Member
	 * @param	BW_Member what	instancia a ser atualizada
	 * @return	BW_Member	instancia lida da base de dados
	 */
	public BW_Member read (BW_Member what) {
		BW_Member ret = null;
		try {
			ret = this.control.read  (what);
		} catch (Exception e) {
			ret = null;
		}
		return ret;
	}
	

	/** Le, de forma generica, todas as instancias armazenadas
	 * de um determinado tipo, identificado pelo typeid da instancia modelo.
	 * @param	BW_Element what	instancia modelo, com o typeid setado
	 * @return	Vector 		instancias lidas
	 **/
	public Vector readAll (BW_Element what) {
		Vector ret = null;
		try {
			ret = this.control.readAll (what);
		} catch (Exception e) {
		
		}
		return ret;	
	
	}

		
	/** Le todos os membros armazenadas, segundo o identificador
	 * de tipo passado na instancia modelo.
	 * @param	BW_Member what	instancia modelo, com o typeid setado
	 * @return	Vector 		instancias lidas
	 **/
	public Vector readAll (BW_Member what) {
		Vector ret = null;
		try {
			//System.out.println ("[DBG] FC_ST_Context.readAll(BW_Member): reading...");			
			ret = this.control.readAll (what);
		} catch (Exception e) {
		        System.out.println ("[DBG]FC_ST_Context.readAll(BW_Member): exception catch");
		        e.printStackTrace();
		}
		return ret;		
	}

	
	/** Le todas as instancias armazenadas de um determinado tipo
	 * de evento, segundo o identificador de tipo passado na instancia modelo
	 * @param	BW_Event what	instancia modelo, com o typeid setado
	 * @return	Vector 		instancias lidas
	 **/
	public Vector readAll (BW_Event what) {
		Vector ret = null;
		try {
			ret = this.control.readAll (what);
		} catch (Exception e) {
		
		}
		return ret;		
	}
	
	
	/** faz a leitura de todos os membros, de um determinado tipo, que
	 *  estiveram ativos pela ultima vez no sistema dentro do periodo
	 *  indicado pelo intervalo passado.
	 * @param	BW_Member what		instancia modelo, com o identificador do tipo desejado
	 * @param 	BW_TimeInterval when	intervalo de interesse
	 * @return	Vector			vetor com as instancias recuperadas
	 */
        public Vector readByTime (BW_Member what, BW_TimeInterval when) {
        	Vector ret = null;
        	try {
        		ret = this.control.readByTime (what, when);
        	}catch (Exception e) {
        	
        	}
        	return ret;
        }
	
	/** faz a leitura de todos os eventos, de um determinado tipo, que
	 *  ocorreram dentro do periodo indicado pelo intervalo passado.
	 * @param	BW_Event what		instancia modelo, com o identificador do tipo desejado
	 * @param 	BW_TimeInterval when	intervalo de interesse
	 * @return	Vector			vetor com as instancias recuperadas
	 */
        public Vector readByTime (BW_Event what, BW_TimeInterval when) {
        	Vector ret = null;
        	try {
        		ret = this.control.readByTime (what, when);
        	}catch (Exception e) {
        	
        	}
        	return ret;
        }

	/** faz a leitura de todos os profiles, de um determinado tipo, que
	 *  possuem o intervalo de interesse dentro do periodo
	 *  indicado pelo intervalo passado.
	 * @param	BW_Profile what		instancia modelo, com o identificador do tipo desejado
	 * @param 	BW_TimeInterval when	intervalo de interesse
	 * @return	Vector			vetor com as instancias recuperadas
	 */
        public Vector readByTime (BW_Profile what, BW_TimeInterval when) {
        	Vector ret = null;
        	try {
        		ret = this.control.readByTime (what, when);
        	}catch (Exception e) {
        	
        	}
        	return ret;
        }
        	


	///////////////////// READS EVENTOS
	
	/** Le todos os eventos que sao de interesse do profile passado.
	 * @param	BW_Profile what		profile descrevendo os tipos de eventos
	 *					e o intervalo de interesse.
	 * @return	Vector			todos os eventos lidos
	 */
	public Vector readEvents (BW_Profile what) {
		Vector ret = null;
		try {
			ret = this.control.readEvents (what);
		} catch (Exception e) {
		
		}
		return ret;
	
	}
	
	
	///////////////// SET E GETS
	
	/** seta o controlador da camada de armazenamento
	 * @param	ST_Cotnroler stc	novo controlador
	 */
        public void setControler (ST_Controler stc) {
        	this.control = stc;

        }

        /** seta o estrategista para a camada de armazenamento.
         * @param	ST_Strategist stg
         * @see #strategy
         */
        public void setStrategist (ST_Strategist stg) {
        	this.strgy = stg;

        }

        /** seta o objeto responsavel pela contextualizacao na camada de controle
	 * @param	CL_Awareness aware	gerente de contextualizacao
	 */
        public void setAwareness (CL_Awareness aware) {
        	this.awareness = aware;
        }

        /** retorna o controlador da camada de armazenamento utilizado
         * @return	ST_Controler	controlador usado
         */
        public ST_Controler getControler () {
        	return this.control;

        }

        /** retorna o estrategista usado.
         * @return	ST_Strategist	estrategista em uso
         */
        public ST_Strategist getStrategist () {
        	return this.strgy;
        }

        /** retorna o gerente de contextualizacao em uso
         * @erturn	CL_Awareness	gerente para awareness
         */
        public CL_Awareness getAwareness () {
        	return this.awareness;
        }



        //////////////////////////  OUTROS

        public void Notify () {
        	awareness.Notify();
        }


        /** Finaliza as operacoes */
	public void terminate () {
		System.out.println ("[DBG]FC_ST_Context.terminate: finalizing activities");	
	       control.terminate () ;
	}




}