/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * COPSE Client Application
 * Classe:  COPSEClientEventHandler
\******************************************************************************/

import java.io.*;
import java.util.*;
import java.net.*;


import COPSE.*;
import COPSE.CONTROL.*;
//import COPSE.CLIENT.*;


public class COPSEClientEventHandler
    extends CEventHandler
{
    COPSEClient                 pClient;

    public COPSEClientEventHandler ( COPSEClient client )
    {
        super();
        pClient = client;
    }

    protected void ProcessEvent( CEvent event )
    {
        COPSESessionManager pSessionManager = pClient.GetSessionManager();
        int type = event.GetType();

        System.err.println( "[DEBUG} COPSEClientEventHandler - Event ["+type+"]" );
        switch( type )
        {
            case CEvent.EvCOPSE_Server_Exit:
                {
                    pClient._Exit();

                    // TEMP ??
                    System.exit(0);
                }
                break;

            case CEvent.EvCOPSE_Connection_Close:
                {
                    Integer userId = (Integer) event.GetArgumentAt( 0 );
                    System.err.println( "[DEBUG} COPSEClientEventHandler - UserId ["+userId.intValue()+"] logging out..." );
                    System.err.println( "[DEBUG} COPSEClientEventHandler - Your Id is "+ pClient.GetProjectManager().GetUserId() );
                    System.err.flush();


                    COPSESession session = null;
                    if( pSessionManager != null )
                    {
                        session = pSessionManager.FindSession( userId.intValue() );
                        if( session == null )
                            System.err.println( "Session not found in sessionManager");
                    }
                    else
                        System.err.println( "SessionManager is null");



                    if( (userId.intValue()) == (pClient.GetProjectManager().GetUserId()) )
                    {
                        System.err.println( "[DEBUG} COPSEClientEventHandler - exiting client..." );
                        System.err.flush();
                        pClient._Exit();
                    }
                    else
                    {
                        // Remove session from SessionManager
                        pSessionManager.RemoveSession( session );
                    }
                    System.err.flush();
                }
                break;


            case CEvent.EvCOPSE_SessionManager_Load:
                {
                    pClient.SetSessionManager( (COPSESessionManager) event.GetArgumentAt(0) );
                    pSessionManager = pClient.GetSessionManager();
                }
                break;

            case CEvent.EvCOPSE_ProductManager_Load:
                {
                    COPSEClientProductManager productManager;
                    productManager = pClient.GetProjectManager().GetProductManager();

                    CFileController     productController;
                    Vector              documentControllers;

                    productController = (CFileController) event.GetArgumentAt( 0 );
                    documentControllers = (Vector)  event.GetArgumentAt( 1 );

                    productManager.SetProduct( productController, documentControllers );
                }
                break;

            case CEvent.EvCOPSE_UsersManager_Load:
                {
                    COPSEClientUsersManager usersManager;
                    usersManager = pClient.GetProjectManager().GetUsersManager();

                    CFileController     usersController;
                    usersController = (CFileController) event.GetArgumentAt( 0 );
                    usersManager.SetUsers( usersController );
                }
                break;

            case CEvent.EvCOPSE_ProcessManager_ServerSocket:
                {
                    String   host    = (String)   event.GetArgumentAt( 0 );
                    Integer  port    = (Integer)  event.GetArgumentAt( 1 );

                    System.err.println( "[DEBUG] COPSEClientEventHandler - Starting ProcessManager to ("+host+":"+port+")" );
                    System.err.flush();
                    pClient.GetProjectManager().StartProcessManager( host, port.intValue() );
                }
                break;

            default:
                // Ignore the event
        }

    }

}

