/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * COPSE Client Application
 * Classe:  COPSEClientProcessManager
\******************************************************************************/


import java.io.*;
import java.util.*;

import COPSE.*;
import FILE.*;
import PROCESS.*;
import PROCESS.MODEL.*;


// Event Handlers
import COPSE.CLIENT.*;


//***********************************************
//* Class: COPSEClientProcessEventHandler
//***********************************************
class COPSEClientProcessClientEventHandler
    extends CClientEventHandler
{
    public COPSEClientProcessClientEventHandler( COPSEClientProcessManager client )
    {
        super( client );
    }

    protected void ProcessEvent( CEvent event )
    {
        COPSEClientProcessManager manager = (COPSEClientProcessManager) pClient;
        int type = event.GetType();

        switch(type)
        {
            case CEvent.EvProcessServer_ProcessUsersSession_Load:
            {
                System.err.println( "[DEBUG] COPSEClientProcessClientEventHandler - ProcessUsersSession" );
                CProcessUsersSession    sessions;
                sessions = (CProcessUsersSession) event.GetArgumentAt(0);
                manager.SetProcessUsersSession( sessions );
            }
            break;

            default:
                super.ProcessEvent( event );
        }
    }
}


//*************************************************
//*  Class: COPSEClientProcessManagerEventHandler
//*************************************************
class COPSEClientProcessManagerEventHandler
    extends CClientManagerEventHandler
{
    public COPSEClientProcessManagerEventHandler( COPSEClientProcessManager manager, CClientEventHandler handler )
    {
        super( manager, handler );
    }

    protected void ProcessEvent( CEvent event )
    {
        switch( event.GetType() )
        {
            default:
                break;
        }

        // Passes the event to superclasse
        super.ProcessEvent( event );
    }
}




//***********************************************
//*  Class: COPSEClientProcessManager
//***********************************************
public class COPSEClientProcessManager
    extends CQueuedClientManager
//    extends CClientManager
{
    COPSEClientProjectManager   pProjectManager;

    CProcessUsersSession        pPrjUsersSession;


    Vector                      pManagerListeners;


    public static COPSEClientProcessManager New( COPSEClientProjectManager manager, String host, int port, CEventConnector globalClientConnector )
        throws IOException
    {
        pCOPSEClientConnector = globalClientConnector;
        return new COPSEClientProcessManager( manager, host, port );
    }


    protected COPSEClientProcessManager( COPSEClientProjectManager manager, String host, int port )
        throws IOException
    {
        super( manager.GetUserId(), host, port );

        pProjectManager = manager;

        pPrjUsersSession = null;

        pManagerListeners = new Vector();

        // Ask "Server" for extra needed information
        try {
            CEvent event = new CEvent( CEvent.EvProcessServer_ProcessUsersSession_Load );
            pServerConnector.Send( event );
        } catch ( Exception e ) {};
    }



    protected CClientEventHandler   CreateClientEventHandler()
    {
        return new COPSEClientProcessClientEventHandler ( this );
    }

    protected CClientManagerEventHandler CreateManagerEventHandler()
    {
        return new COPSEClientProcessManagerEventHandler( this, pEventHandler );
    }



//***********************************************************************

    public void addManagerListener( COPSEClientManagerListener listener )
    {
        pManagerListeners.add( listener );
    }

    public void removeManagerListener( COPSEClientManagerListener listener )
    {
        pManagerListeners.remove( listener );
    }

    public Enumeration getManagerListeners()
    {   return pManagerListener.elements(); }

    public CProcess GetProcess()
    {
        return (CProcess) pFile;
    }

/*
    public void AddUserToActivity( int actId, int userId )
    {
        pPrjUsersSession.AddEntry( actId, userId );
    }

    public void RemoveUserFromActivity( int actId, int userId )
    {
        pPrjUsersSession.RemoveEntry( actId, userId );
    }

    public Enumeration GetUsersInActivity( int actId )
    {
        return pPrjUsersSession.GetUsersInActivity( actId );
    }
*/

    public CProcessUsersSession GetProcessUsersSession()
    {
        return pPrjUsersSession;
    }

    public void SetProcessUsersSession (CProcessUsersSession processUsersSession)
    {
        pPrjUsersSession = processUsersSession;
    }


}


