/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * COPSE Client Application
 * Classe:  UserSessionManager
\******************************************************************************/

import java.io.*;
import java.util.*;
import java.net.*;


import COPSE.*;
import FILE.*;
import PROCESS.*;


class ProcessClient
    extends Thread
{
    UserSessionManager      pUserManager;
    CSocketStreamConnector  pConnector;

    CProcess                pProcess;
    CProcessUsersSession    pProcessUsers;

    boolean                 mExit;

    public ProcessClient( UserSessionManager umgr, String host, String port )
        throws IOException
    {
        pUserManager = umgr;
        mExit = false;

        Socket pSocket = new Socket( host, port );

        pConnector = CSocketStreamConnector.New( pSocket, false );

        // Start Communication to Process Server
        CEvent event = new CEvent( CEvent.EvUserConnection_Open );
        event.AddArgument( new Integer( pUserManager.GetUserId() ) );
        pConnector.Send( event );
        event = pConnector.Receive();
        ProcessEvent( event );

        // Load Process and the Users "Positions" in the Process
        CEvent event = new CEvent( CEvent.EvProcessServer_Process_Load );
        pConnector.Send( event );
        event = pConnector.Receive();
        ProcessEvent( event );

    }


    public CProcess GetProcess()        { return pProcess; }


    public void run()
    {
        while( !mExit )
        {
            try{
                CEvent event = pConnector.Receive();
                ProcessEvent( event );
            } catch(IOException ioe)
            {  // TODO: Verify if the connection is still alive
            }
            catch(Exception e) {}
        } // while( !mExit )

        //pConnector.close();
        //Exit();
    }



    protected void ProcessEvent( CEvent event )
    {
        switch( event.GetType() )
        {
            case CEvent.EvUserConnection_Open:
                OnEvent_UserConnectionOpen( event );
                //NotifyDisplay();
                break;

            case CEvent.EvUserConnection_Close:
                OnEvent_UserConnectionClose( event );
                //NotifyDisplay();
                break;

            case CEvent.EvProcessServer_Process_Load:
                pProcess = (CProcess)event.GetArgumentAt(0);
                pProcessUsers = (CProcessUsersSession) event.GetArgumentAt(1);
                break;

            default:
                break;
        } // end switch

    }

}







public class UserSessionManager
    extends Thread
{
    int         mUserId;

    CSocketStreamConnector  pConnector;

    boolean     mExit;

    CFile       pPrjUsers;
    Vector      pLoggedUsers;

    //ProcessClient   pPrcClient;


    public UserSessionManager( String serverHost, String serverPort, String login, String password )
        throws UnknownHostException, IOException
    {
        super();

        // try to connect to the COPSE serversocket (at UserManager)
        Socket pSocket = new Socket( serverHost, new Integer(serverPort).intValue() );

        try {
            pConnector = CSocketStreamConnector.New( pSocket );
        }
        catch( Exception e )
        {
            System.err.println("Error: UserSessionManager");
            e.printStackTrace( System.err );
            throw new IOException( e.toString() );
        }

        // Request Login to the COPSE Environment
        CEvent event = new CEvent( CEvent.EvUserConnection_Open );
        event.AddArgument( login );
        event.AddArgument( password );
        pConnector.Send( event );

        event = pConnector.Receive();
        switch( event.GetType() )
        {
            case CEvent.EvUserConnection_Open:
                {
                    Integer uid = (Integer) event.GetArgumentAt(0);
                    mUserId = uid.intValue();
                    mExit = false;

                    if( !_InitUserSessionManager() )
                        throw new IOException( "Error on starting User Session Manager!");
                }
                break;

            case CEvent.EvUserConnection_Error:
                {
                    pConnector.Close();
                    throw new IOException( (String)event.GetArgumentAt(0) );
                }

            default:
                {
                    pConnector.Close();
                    throw new IOException( "Unknown Error!" );
                }
        } // end switch

    }


    public int GetUserId()      { return mUserId; }



    private boolean _InitUserSessionManager()
    {
        CEvent event;

        try {
            // Load Project Users
            event = new CEvent( CEvent.EvUserManager_Users_LoadAll );
            pConnector.Send( event );
            event = pConnector.Receive();
            ProcessEvent( event );

            // Load Logged Users
            event = new CEvent( CEvent.EvUserManager_Users_LoadLogged );
            pConnector.Send( event );
            event = pConnector.Receive();
            ProcessEvent( event );

            // Load Process Server Socket
            event = new CEvent( CEvent.EvUserManager_ProcessServer_GetSocket );
            pConnector.Send( event );
            event = pConnector.Receive();
            ProcessEvent( event );



        } catch ( Exception e ) { return false;};
        return true;
    }




    public void run()
    {
        while( !mExit )
        {
            try{
                CEvent event = pConnector.Receive();
                ProcessEvent( event );
            } catch(IOException ioe)
            {  // TODO: Verify if the connection is still alive
            }
            catch(Exception e) {}
        } // while( !mExit )

        //pSocket.close();
        //Exit();
    }



    protected void ProcessEvent( CEvent event )
    {
        switch( event.GetType() )
        {
            case CEvent.EvUserConnection_Open:
                OnEvent_UserConnectionOpen( event );
                //NotifyDisplay();
                break;

            case CEvent.EvUserConnection_Close:
                OnEvent_UserConnectionClose( event );
                //NotifyDisplay();
                break;

            case CEvent.EvUserManager_Users_LoadAll:
                pPrjUsers = (CFile)event.GetArgumentAt(0);
                break;

            case CEvent.EvUserManager_Users_LoadLogged:
                pLoggedUsers = (Vector)event.GetArgumentAt(0);
                break;

            case CEvent.EvUserManager_ProcessServer_GetSocket:
                {
                    String  host = (String)  event.GetArgumentAt(0);
                    Integer port = (Integer) event.GetArgumentAt(1);

                }
                break;

            default:
                break;
        } // end switch

    }


    protected boolean OnEvent_UserConnectionOpen( CEvent event )
    {
        Integer i = (Integer)event.GetArgumentAt(0);

        // TODO: Add User to Logged User List...
        if( pLoggedUsers != null )
        {
            pLoggedUsers.addElement( i );
            return true;
        }
        return false;
    }


    protected boolean OnEvent_UserConnectionClose( CEvent event )
    {
        Integer i = (Integer)event.GetArgumentAt(0);
        if( mUserId == i.intValue() )
        {
            // ?????
            mExit = true;
        }
        else
        {
            // TODO: Remove User to Logged User List...
            Enumeration e = pLoggedUsers.elements();
            while( e.hasMoreElements() )
            {
                Integer in = (Integer) e.nextElement();
                if( in.equals( i ) )
                {
                    pLoggedUsers.removeElement( in );
                    return true;
                }
            }
        }
        return false;
    }






/*
    public synchronized void Send( CEvent event )
        throws IOException
    {
        pConnector.Send( event );
    }
*/


    public void Exit()
    {
        //try{
            pConnector.Close();
        //} catch( IOException ioe ) {};
        stop();
    }

}

