/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * Classe GW_BW_Mediator
 * PACOTE BW
 * DATA DE CRIACAO 04 / 01 / 2001
 */
package BW;

import BW.Interface.UI_GUIElement;
import BW.kernel.*;

/**
 * Classe GW_BW_Mediator faz serve de intermediario
 * entre o groupware (GW) e o framework BW.
 * Deve ser especializada, na forma de um mediador
 * concreto, adaptado para o groupware
 * em questao.
 *
 * @author Manuele Kirsch Pinheiro
 * @version 1.0
 */
public class GW_BW_Mediator {

     ///////////////////////////////// ATRIBUTOS //////////////////////////////

     /** mediador concreto. Este mediador concreto deve ser um objeto de
      * uma subclasse desta classe, adaptada para a situacao de uso real
      */
     static GW_BW_Mediator concreteMediator=null;


     ///////////////////////////////// METODOS ESTATICOS /////////////////////

     /** seta o objeto que funcionara de mediador concreto
      *  @see #concreteMediator
      */
     public static void SetConcreteMediator (GW_BW_Mediator concrete) {
     		System.out.println ("Setting concrete mediator: "+concrete);
     		concreteMediator = concrete;	
     }


     /** resgata o objeto que funciona como mediador concreto
      *  @return BW_Mediator	mediador concreto atualmente em uso
      *				null caso o mediador nao tenha sido setado
      *  @see #concreteMediator
      */
     public static GW_BW_Mediator GetConcreteMediator () {
     		return concreteMediator;
     }


     /** envia o sinal de inicio de processamento para o mediator,
      * avisando-o do inicio das atividades
      * @return boolean	true se foi possivel inicializar o mediador concreto;
      *			false se nao foi possivel
      */
     public static boolean Start () {
     		boolean ret = false;
     		try {
     		  	ret = concreteMediator._start();
     		} catch (NullPointerException npe) {
     			ret = false;
     		}
     		return ret;
     }

     /** envia o sinal de inicio de processamento para o mediator,
      * avisando-o do inicio das atividades
      * @param 	String host	servidor
      * @return boolean	true se foi possivel inicializar o mediador concreto;
      *			false se nao foi possivel
      */
     public static boolean Start (String host) {
     		boolean ret = false;
     		try {
     		  	ret = concreteMediator._start(host);
     		} catch (NullPointerException npe) {
     			ret = false;
     		}
     		return ret;
     }


     /** envia o sinal de fim do processamento para o mediator,
      * avisando-o do termino das atividades
      * @return boolean	true se foi possivel finalizar o mediador concreto;
      *			false se nao foi possivel
      */
     public static boolean Stop () {
     		boolean ret = false;
     		try {
     		  	ret = concreteMediator._stop();
     		} catch (NullPointerException npe) {
     			ret = false;
     		}
     		return ret;
     }



     /** assinala o inicio da ocorrencia de um evento.
      * Assinalado o inicio de um evento, este devera ficar
      * em uma fila ou pilha (de acordo com a implementacao)
      * no mediador concreto, aguardando ate que seja assinalado
      * seu termino, e sua duracao possa ser setada corretamente.
      *
      * @param 	BW_Event evt	evento que se inicia
      * @param	int typeId	tipo do evento
      * @return boolean	true	se tudo ocorreu como de esperado
      *			false	caso contrario
      * @see	#EndEvent
      */
     public static boolean BeginEvent (int typeId, BW_Event evt) {
     		boolean ret = false;
     		try {
     		 	ret = concreteMediator._beginEvent (typeId,evt);
     		} catch (NullPointerException npe) {
     			ret = false;
     		}
                return ret;
     }


      /** assinala o inicio da ocorrencia de um evento.
      * Assinalado o inicio de um evento, este devera ficar
      * em uma fila ou pilha (de acordo com a implementacao)
      * no mediador concreto, aguardando ate que seja assinalado
      * seu termino, e sua duracao possa ser setada corretamente.
      *
      * @param	int typeId	tipo do evento
      * @return boolean	true	se tudo ocorreu como de esperado
      *			false	caso contrario
      * @see 	#EndEvent
      */
     public static boolean BeginEvent (int typeId) {
     		boolean ret = false;
     		try {
     		 	ret = concreteMediator._beginEvent (typeId);
     		} catch (NullPointerException npe) {
     			ret = false;
     		}
     		return ret;
     }


     /** assinala o termino da ocorrencia de um evento.
      * O evento devera ser retirado da fila ou pilha (de acordo
      * com a implementacao), onde estava sendo mantido,
      * no mediador concreto, e setada sua duracao correta.
      *
      * @param 	BW_Event evt	evento que terminou
      * @param	int typeId	tipo do evento
      * @return boolean	true	se tudo ocorreu como de esperado
      *			false	caso contrario
      * @see	#BeginEvent
      */
     public static boolean EndEvent (int typeId, BW_Event evt) {
     		boolean ret = false;
     		try {
     		 	ret = concreteMediator._endEvent (typeId,evt);
     		} catch (NullPointerException npe) {
     			ret = false;
     		}
                return ret;
     }



     /** assinala o termino da ocorrencia de um evento.
      * O evento seguinte do tipo indicado devera ser
      * retirado da fila ou pilha (de acordo com a
      * implementacao), onde estava sendo mantido no mediador
      * concreto, e setada sua duracao correta.
      *
      * @param	int typeId	tipo do evento
      * @return boolean	true	se tudo ocorreu como de esperado
      *			false	caso contrario
      * @see	#BeginEvent
      */
     public static boolean EndEvent (int typeId) {
     		boolean ret = false;
     		try {
     		 	ret = concreteMediator._endEvent (typeId);
     		} catch (NullPointerException npe) {
     			ret = false;
     		}
                return ret;
     }

     /** assinala o cancelamento da ocorrencia de um evento.
      * O evento seguinte do tipo indicado devera ser
      * retirado da fila ou pilha (de acordo com a
      * implementacao), onde estava sendo mantido no mediador
      * concreto, e descartado.
      *
      * @param	int typeId	tipo do evento
      * @return boolean	true	se tudo ocorreu como de esperado
      *			false	caso contrario
      * @see	#BeginEvent
      * @see	#EndEvent
      */
     public static boolean CancelEvent (int typeId) {
     		boolean ret = false;
     		try {
     		 	ret = concreteMediator._cancelEvent (typeId);
     		} catch (NullPointerException npe) {
     			ret = false;
     		}
                return ret;
     }


     /** seta o elemento de interface principal
      * @param UI_Element main	elemento principal (gerente de interface)
      * @return	boolean	true se tudo ocorreu bem, false caso contrario
      */
     public static boolean SetMainUIElement (UI_GUIElement main) {
     		boolean ret = false;
     		try {
     		 	ret = concreteMediator._setMainUIElement (main);
     		} catch (NullPointerException npe) {
     			ret = false;
     		}
                return ret;

     }

     /** retorna o elemento de interface principal, se disponivel
      * @return	UI_Element	elemento principal (gerente de interface)
      *				null caso este nao tenha sido setado 	
      */
     public static UI_GUIElement GetMainUIElement () {
     		UI_GUIElement ret = null;
     		try {
     		 	ret = concreteMediator._getMainUIElement ();
     		} catch (NullPointerException npe) {
     			ret = null;
     		}
                return ret;

     }

     /** assinala o ingresso de um usuario no sistema
      * @param 	BW_Member member	usuario ingresso no sistema
      * @return	BW_Member		informacoes atualizadas do membro
      */
     public static BW_Member UserLogin (BW_Member member) {
     		BW_Member ret = null;
     		try {
     		 	ret = concreteMediator._userLogin (member);
     		} catch (NullPointerException npe) {
     		
     		}
     		return ret;
     }


     /** assinala a saida de um usuario no sistema
      * @param 	BW_Member member	usuario saido do sistema
      */
     public static void UserLogout (BW_Member member) {
     		try {
     		 	concreteMediator._userLogout (member);
     		} catch (NullPointerException npe) {
     		
     		}
     }

     /** Retorna no typeid do grupo default utilizado
      * @return	int 	identificador de tipo do grupo utilizado
      */
     public static int GetGroupTypeId () {
     		int ret = typeIdDefault.UNSET;
     		try {
     		 	ret = concreteMediator._getGroupTypeId();
     		} catch (NullPointerException npe) {
     		
     		}
     		return ret;     	
     }

     /** Retorna no objid do grupo default utilizado
      * @return	int 	identificador do grupo utilizado
      */
     public static int GetGroupObjId () {
     		int ret = typeIdDefault.UNSET;
     		try {
     		 	ret = concreteMediator._getGroupObjId();
     		} catch (NullPointerException npe) {
     		
     		}
     		return ret;     	
     }



     ///////////////////////////////// METODOS DE INSTANCIA //////////////////

     public boolean _start () {
     	//do nothing....
        //este metodo eh para ser especializado
        return false;
     }

     public boolean _start (String host) {
     	//do nothing....
        //este metodo eh para ser especializado
        return false;
     }

     public boolean _stop () {
     	//do nothing....
        //este metodo eh para ser especializado
        return false;
     }

     public boolean _beginEvent (int typeId, BW_Event evt) {
     	//do nothing....
        //este metodo eh para ser especializado
        return false;
     }

     public boolean _beginEvent (int typeId) {
     	//do nothing....
        //este metodo eh para ser especializado
        return false;
     }

     public boolean _endEvent (int typeId, BW_Event evt){
     	//do nothing....
        //este metodo eh para ser especializado
        return false;
     }

     public boolean _endEvent (int typeId){
     	//do nothing....
        //este metodo eh para ser especializado
        return false;
     }

     public boolean _cancelEvent (int typeId){
     	//do nothing....
        //este metodo eh para ser especializado
        return false;
     }

     public boolean _setMainUIElement (UI_GUIElement main){
     	//do nothing....
        //este metodo eh para ser especializado
        return false;
     }

     public UI_GUIElement _getMainUIElement (){
     	//do nothing....
        //este metodo eh para ser especializado
       return null;
     }

     public BW_Member _userLogin (BW_Member member) {
     	//do nothing....
        //este metodo eh para ser especializado
        return null;
     }

     public void _userLogout (BW_Member member) {
     	//do nothing....
        //este metodo eh para ser especializado
     }

     public int _getGroupTypeId () {
     	//do nothing....
        //este metodo eh para ser especializado
        return typeIdDefault.UNSET;     	
     }

     public int _getGroupObjId () {
     	//do nothing....
        //este metodo eh para ser especializado
        return typeIdDefault.UNSET;     	
     }


}


