/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * Classe UML_BW_Mediator
 * PACOTE BW.UML
 * DATA DE CRIACAO 04 / 01 / 2001
 */
package BW.UML;


//REVER os imports!!
import BW.UML.ObjId.*;
import BW.Interface.*;
import BW.control.*;
import BW.storage.*;
import BW.kernel.*;
import BW.*;

import COPSE.CEvent;

import java.util.*;
import java.net.*;

/**
 * Classe UML_BW_Mediator faz serve de intermediario
 * entre o groupware CUTE (construido com o framework
 * COPSE) e o framework para awareness BW.
 * COPSE e CUTE sao de autoria de Marcio Dias, pela
 * COPPE/UFRJ, 1998.
 * @author Manuele Kirsch Pinheiro
 * @version 1.0
 */
public class UML_BW_Mediator extends GW_BW_Mediator {


     ////////////////////////////// CONSTANTES UTILIZADAS //////////////////////////

     /* ***** TYPEIDS para os eventos ***** */

     	/** typeId identificador do evento "USERSESSION".
     	 * Este evento indica a ocorrencia de uma sessao de usuario no sistema.
     	 */
	public static final int USERSESSION = CEvent.EvCOPSE_Connection_Open;
	
	/** typeId indentificador do evento "SAVEDIAGRAM"
	 * Este evento indica a ocorrencia de uma requisicao de save para o
	 * diagrama ativo
	 */
	public static final int SAVEDIAGRAM = CEvent.EvCOPSE_FileController_Save;
	
	/** typeId indentificador do evento "NEWCLASS"
	 * Este evento indica a criacao de uma nova classe no
	 * diagrama ativo
	 */
	public static final int NEWCLASS = CEvent.EvCOPSE_FileController_Element_New;

	/** typeId indentificador do evento "CLASSEDITED"
	 * Este evento indica o processo de edicao/alteracao de uma classe no
	 * diagrama ativo
	 */
	public static final int CLASSEDITED = CEvent.EvCOPSE_FileController_Element_Update;
	
	/** typeId indentificador do evento "CLASSREMOVED"
	 * Este evento indica a remocao de uma classe do
	 * diagrama ativo
	 */
	public static final int CLASSREMOVED = CEvent.EvCOPSE_FileController_Element_Delete;
	
	/** typeId indentificador do evento "NEWRELATION"
	 * Este evento indica a criacao de um novo relacionamento entre classes no
	 * diagrama ativo
	 */
	public static final int NEWRELATION = CEvent.EvCOPSE_FileController_Element_New + 1000;

	/** typeId indentificador do evento "RELATIONEDITED"
	 * Este evento indica o processo de edicao/alteracao de um relacionamento
	 * entre classes do diagrama ativo
	 */
	public static final int RELATIONEDITED = CEvent.EvCOPSE_FileController_Element_Update + 1000;
	
	/** typeId indentificador do evento "RELATIONREMOVED"
	 * Este evento indica a remocao de um relacionamento entre classes do
	 * diagrama ativo
	 */
	public static final int RELATIONREMOVED = CEvent.EvCOPSE_FileController_Element_Delete + 1000;

	/** typeId indentificador do evento "BROADCASTMESSAGE"
	 * Este evento indica o envio de uma mensagem para todo o grupo (broadcast)
	 */
	public static final int BROADCASTMESSAGE = CEvent.EvCollaboration_Communication_Call_Message;
	
	
     /* ************* typeid e objid para o papel default - CUTE adota apenas um papel unico ******* */

        /** typeid default para o papel adotado pelo CUTE */
        public static final int PAPERTYPEID = typeIdDefault.BW_PAPER;

        /** objid default para o papel default adotado pelo CUTE */
        public static final int PAPEROBJID = typeIdDefault.BW_PAPER;


    /* ************* typeid e objid para o grupo default - CUTE adota apenas um grupo unico ******* */

        /** typeid default para o grupo adotado pelo CUTE */
        public static final int GROUPTYPEID = typeIdDefault.BW_GROUP;

        /** objid default para o grupo default adotado pelo CUTE */
        public static final int GROUPOBJID = typeIdDefault.BW_GROUP;


    /* ************** outros ************ */

        /** numero maximo de instancia na fila de eventos concluidos */
	public static int MAX = 2;
	
	/** contador para controle de salvamento  - de tempos em tempos salva todo o conteudo */
	public static int SAVEALL = 0;
	
	/**referencia a uma instancia de si mesmo */	
	public static UML_BW_Mediator myself;
	
	/** identificador do ultimo objeto gerado */
        public static int lastObjId = typeIdDefault.OBJID;


	
     ///////////////////////////////// ATRIBUTOS //////////////////////////////


		
     /** Hashtable contendo as pilhas dos eventos em aberto - uma para cada tipo de evento */
      Hashtable openEvents;

     /** fila com os eventos ja concluidos */
      Vector events;

     // /** identificador do ultimo objeto gerado */
     //int lastObjId;

     /** ST_Implementor especifico usado para esta aplicacao */
     ST_DatabaseImplementor implementor;

     /** Principal elemento de interface com o usuario */
     AwarenessMainWindow mainInterface;
     //UI_GUIElement mainInterface;

     /** Canal de comunicacao com a camada de interface */
     UI_Interface Interface;

     /** Canal de comunicacao com a camada de controle */
     FC_CL_Monitor monitor;

     /** TimeZone default */
     TimeZone tmzD;

     /** Locale Default */
     Locale lclD;

     /** Papel default utilizado pelo CUTE. Por nao ser integrado ao
      * ambiente COPSE (mas somente construido com o framework homonimo),
      * o CUTE nao possui a nocao de papeis e grupos, operando com um unico
      * valor default.
      */
     public BW_Paper paperD;

     /** Grupo default utilizado pelo CUTE. Por nao ser integrado ao
      * ambiente COPSE (mas somente construido com o framework homonimo),
      * o CUTE nao possui a nocao de papeis e grupos, operando com um unico
      * valor default.
      */
     public BW_Group groupD;

     /** Relacao de logins dos usuarios conhecidos, com seus respectivos
      * UIDs. Formato da Hash: (String login, Integer UID)
      */
     Hashtable members;

     /** Nome do servidor utilizado para buscar os identificadores de objeto */
     String host;

     /** Porto utilizada pelo servidor de identificadores de objeto */
     int port;


     ////////////////////////////////// CONSTRUTOR ////////////////////////////

/* -------------------------------------------------------------------------------- */

     /** construtor default */
     public UML_BW_Mediator () {
    	this.openEvents = new Hashtable();
    	this.initStacks();
    	this.events = new Vector();
    	//lastObjId = typeIdDefault.OBJID;
    	
    	implementor = null;
    	//mainInterface = null;
    	mainInterface = new AwarenessMainWindow();
    	
    	tmzD = TimeZone.getTimeZone ("BET");
    	lclD = new Locale ("pt","BR");
    	TimeZone.setDefault (tmzD);
    	Locale.setDefault (lclD);
    	
    	paperD = null;
    	groupD = null;
    	members = new Hashtable();
    	
    	host = "localhost";
    	port = 5433;
    	
    	UML_BW_Mediator.myself = this;
     }

/* -------------------------------------------------------------------------------- */

     /** finaliza as atividades do mediador
     protected void finalize () throws Throwable {
     	this._stop();
     	super.finalize();
     }
     */		

     /** termina a execucao do mediador e de todo o framework BW */
     public void terminate () {
     	try {
     		//this.finalize();	
     		this._stop();	
     	} catch (Throwable t) {
     	}
     }

/* -------------------------------------------------------------------------------- */


     ///////////////////////////////// METODOS ////////////////////////////////


/* -------------------------------------------------------------------------------- */

     /** inicializa as pilhas de eventos em aberto */
     void initStacks() {
          openEvents.put (mountKey (UML_BW_Mediator.USERSESSION),new Stack());
          openEvents.put (mountKey (UML_BW_Mediator.SAVEDIAGRAM),new Stack());
          openEvents.put (mountKey (UML_BW_Mediator.NEWCLASS),new Stack());
          openEvents.put (mountKey (UML_BW_Mediator.CLASSEDITED),new Stack());
          openEvents.put (mountKey (UML_BW_Mediator.CLASSREMOVED),new Stack());
          openEvents.put (mountKey (UML_BW_Mediator.NEWRELATION),new Stack());
          openEvents.put (mountKey (UML_BW_Mediator.RELATIONEDITED),new Stack());
          openEvents.put (mountKey (UML_BW_Mediator.RELATIONREMOVED),new Stack());
          openEvents.put (mountKey (UML_BW_Mediator.BROADCASTMESSAGE),new Stack());
     }

/* -------------------------------------------------------------------------------- */

     /** monta a chave utilizada na Hash de pilhas em aberto */
     Object mountKey (int typeId) {
     	return new Integer (typeId);
     }

/* -------------------------------------------------------------------------------- */

     /** monta a chave utilizada na Hash de membros conhecidos */
     Object mountKey (String login) {
     	return new String (login);
     }


/* -------------------------------------------------------------------------------- */

     /** busca o ultimo evento do tipo indicado que foi colocado na pilha de eventos incompletos */
      BW_Event pop (int typeId) {
		Object key  = this.mountKey (typeId);
	   	Stack s = (Stack) openEvents.get (key);
	   	//System.out.println ("[DEBUG] UML_BW_MEDIATOR.POP: pilhas: "+openEvents);
	   	return (BW_Event) s.pop();
	}
		 
/* -------------------------------------------------------------------------------- */

      /** coloca um evento de um tipo determinado na fila de eventos em aberto */
       void push (int type, BW_Event evt) {
		Object key = this.mountKey (type);     	
		Stack s = (Stack) openEvents.get (key);	   			
        	s.push (evt);		 	
        	//System.out.println ("[DEBUG] UML_BW_MEDIATOR.PUSH: pilha: "+openEvents);
	}

/* -------------------------------------------------------------------------------- */

     /** retorna um vetor com os prototipos de todos tipos de eventos observados
      *  @return	Vector	vetor com instancias prototipos de BW_Event
      */
     public static Vector getPrototypes () {
		
	Vector prot = new Vector();
     	
     	BW_Event evt = new BW_Event (UML_BW_Mediator.USERSESSION,UML_BW_Mediator.USERSESSION);
        evt.setName ("User Session");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.SAVEDIAGRAM,UML_BW_Mediator.SAVEDIAGRAM);
	evt.setName ("Save Diagram");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.BROADCASTMESSAGE,UML_BW_Mediator.BROADCASTMESSAGE);
	evt.setName ("Broadcast Message");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.NEWCLASS,UML_BW_Mediator.NEWCLASS);
	evt.setName ("New Class");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.CLASSEDITED,UML_BW_Mediator.CLASSEDITED);
	evt.setName ("Class Edited");
     	prot.addElement (evt);     	
     	evt = new BW_Event (UML_BW_Mediator.CLASSREMOVED,UML_BW_Mediator.CLASSREMOVED);
	evt.setName ("Class Removed");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.NEWRELATION,UML_BW_Mediator.NEWRELATION);
	evt.setName ("New Relation");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.RELATIONEDITED,UML_BW_Mediator.RELATIONEDITED);
	evt.setName ("Relation Edited");
     	prot.addElement (evt);     	
     	evt = new BW_Event (UML_BW_Mediator.RELATIONREMOVED,UML_BW_Mediator.RELATIONREMOVED);
	evt.setName ("Relation Removed");
     	prot.addElement (evt);

     	return prot;

     }

/* -------------------------------------------------------------------------------- */

     /** cria um vetor com eventos prototipos de todos os 9 tipos de eventos utilizados */
     Vector makePrototypes () {

     	Vector prot = new Vector();
     	
     	BW_Event evt = new BW_Event (UML_BW_Mediator.USERSESSION,UML_BW_Mediator.USERSESSION);
        evt.setName ("User Session");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.SAVEDIAGRAM,UML_BW_Mediator.SAVEDIAGRAM);
	evt.setName ("Save Diagram");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.BROADCASTMESSAGE,UML_BW_Mediator.BROADCASTMESSAGE);
	evt.setName ("Broadcast Message");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.NEWCLASS,UML_BW_Mediator.NEWCLASS);
	evt.setName ("New Class");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.CLASSEDITED,UML_BW_Mediator.CLASSEDITED);
	evt.setName ("Class Edited");
     	prot.addElement (evt);     	
     	evt = new BW_Event (UML_BW_Mediator.CLASSREMOVED,UML_BW_Mediator.CLASSREMOVED);
	evt.setName ("Class Removed");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.NEWRELATION,UML_BW_Mediator.NEWRELATION);
	evt.setName ("New Relation");
     	prot.addElement (evt);
     	evt = new BW_Event (UML_BW_Mediator.RELATIONEDITED,UML_BW_Mediator.RELATIONEDITED);
	evt.setName ("Relation Edited");
     	prot.addElement (evt);     	
     	evt = new BW_Event (UML_BW_Mediator.RELATIONREMOVED,UML_BW_Mediator.RELATIONREMOVED);
	evt.setName ("Relation Removed");
     	prot.addElement (evt);

     	return prot;
     	     	
     }

/* -------------------------------------------------------------------------------- */

     /** preenche os campos ainda em branco do evento recem gerado */
     void fillEvent (int type, BW_Event evt) {        

        if (evt.getResponsible()==null) {
        	BW_Member respon = Interface.getMember();
     	    	if (respon != null) {     	    		
     	    		evt.setResponsible (respon);
     	    		//System.out.println ("[DEBUG]UML_BW_MEDIATOR.FILLEVENT: responsible: "+respon.getUID()+" - "+respon.getLogin());
     	    	}        	
     	    		
     	}
     	
        if (evt.getInterval()==null)
        	evt.setInterval (new BW_TimeInterval (0,0,tmzD,lclD));
        	
     	switch (type) {
     		case USERSESSION:
     		        evt.setName ("User Session");
     		        if (evt.getDescription().equals(""))
     		        	evt.setDescription ("Sessao de um usuario");     		        	     		
     			break;	
     			
     		case SAVEDIAGRAM:
     		        evt.setName ("Save Diagram");
     		        if (evt.getDescription().equals(""))
     		        	evt.setDescription ("Salvamento de diagrama");     		        	     		     		
     			break;	
     			
     		
     		case BROADCASTMESSAGE:
     		        evt.setName ("Broadcast Message");
     		        if (evt.getDescription().equals(""))
     		        	evt.setDescription ("Mensagem de broadcast para o grupo");     		        	     		
     		     		
     			break;	
     			     		
     		case NEWCLASS:
     		        evt.setName ("New Class");
     		        if (evt.getDescription().equals(""))
     		        	evt.setDescription ("Nova classe introduzida no diagrama");     		        	     		
     		
     			break;	
     			
     		case CLASSEDITED:
     		        evt.setName ("Class Edited");
     		        if (evt.getDescription().equals(""))
     		        	evt.setDescription ("Classe do diagrama alterada");     		        	     		
     		
     			break;	
     			
     		case CLASSREMOVED:
     		        evt.setName ("Class Removed");
     		        if (evt.getDescription().equals(""))
     		        	evt.setDescription ("Classe do diagrama removida");     		        	     		
     		
     			break;	
     			
     		case NEWRELATION:
     		        evt.setName ("New Relation");
     		        if (evt.getDescription().equals(""))
     		        	evt.setDescription ("Novo relacionamento entre classes introduzido no diagrama");     		        	     		
     		
     			break;	
     			
     		case RELATIONEDITED:
     		        evt.setName ("Relation Edited");
     		        if (evt.getDescription().equals(""))
     		        	evt.setDescription ("Relacionamento entre classes alterado");     		        	     		
     		
     			break;	
     			
     		case RELATIONREMOVED:
     		        evt.setName ("Relation Removed");
     		        if (evt.getDescription().equals(""))
     		        	evt.setDescription ("Relacionamento entre classes removido");     		        	     		
     		
     			break;	
     			
     	}

     }


/* -------------------------------------------------------------------------------- */

     /** retorna o proximo identificador de objeto a ser usado */
     public int getNextObjId () {
	  //return UML_BW_Mediator.lastObjId++;	
	  clientID cid = new clientID (host,port);
	  int id = cid.getID().intValue();
	  //atualiza se o valor retornado for maior
	  if (id>UML_BW_Mediator.lastObjId)
	  	UML_BW_Mediator.lastObjId = id;
	  return id;
     }	

/* -------------------------------------------------------------------------------- */

     /** coloca o evento ja concluido na fila de eventos prontos */
     synchronized void finished (BW_Event evt) {
     	  events.addElement (evt);
     	  System.out.println ("[DBG]UML_BW_MEDIATOR.FINISHED: finished: "+evt.getObjId());
     	
     	  //se ja chegou ao limite, manda salvar
     	  int number = events.size();
     	  if (number >= UML_BW_Mediator.MAX) {
     	  	monitor.eventOccurence (events);
     	  	events.removeAllElements();
     	  	UML_BW_Mediator.SAVEALL++;
     	  }
     	
     	  //de tempos em tempos, manda salvar tudo
     	  if (UML_BW_Mediator.SAVEALL >= 2*UML_BW_Mediator.MAX) {
     	  	monitor.saveAll();
     	  	UML_BW_Mediator.SAVEALL=0;
     	  }	
     	
     }
     
/* -------------------------------------------------------------------------------- */

    /** Le as informacoes de grupo e papel defautlt do sistema.
     *  Tambem seta a listagem de membros conhecidos do grupo
     * @see #paperD
     * @see #groupD
     * @see #members
     */
    public void readDefaults () {

    	try {
    	   this.groupD = this.Interface.getGroup();
    	   this.paperD = (BW_Paper) this.groupD.getPapers().firstElement();

    	   //se certos atributos do grupo nao estiverem setados
    	   if (groupD.getDescription().equals(""))
    	   	paperD.setDescription ("Usurio default do COPSE/CUTE");
    	   if (groupD.getName().equals(""))
    	   	groupD.setName ("COPSE/CUTE");
    	
    	   //papel nao definido
           if (paperD==null) {
                paperD = new BW_Paper (UML_BW_Mediator.PAPERTYPEID,UML_BW_Mediator.PAPEROBJID);
                paperD.setLevel (BW_Paper.MAXIMUM);
                paperD.setName ("member");
                paperD.setDescription ("Usuario default do COPSE/CUTE");
                BW_Profile profD = UML_BW_Mediator.makeProfile();
                profD.setTypeId (typeIdDefault.BW_PAPERPROFILE);
                profD.setObjId (this.getNextObjId());
                paperD.setProfile (profD);
                this.monitor.addPaper (paperD);
    	   }
    	
    	   Hashtable mbs = this.groupD.getMembers();
    	   Enumeration en = mbs.keys();
    	   while (en.hasMoreElements()) {
    	        Integer uid = (Integer)en.nextElement();
    	   	BW_Member m = (BW_Member) mbs.get (uid);
    	   	this.members.put (this.mountKey(m.getLogin()),uid);
    	   }
    	
    	   //System.out.println ("[DBG]UML_BW_Mediator.readDefaults: readed: "+mbs+" \n--- "+groupD);    	
    	} catch (NullPointerException npe) {
           //grupo nao definido - cria um novo
           if (groupD==null) {
           	groupD = new BW_Group (UML_BW_Mediator.GROUPTYPEID,UML_BW_Mediator.GROUPOBJID);
           	groupD.setName ("COPSE/CUTE");
           	groupD.setDescription ("Grupo de usuarios default do COPSE/CUTE");
           	this.monitor.setGroup(groupD);
           }           	
    	}
    	
    }


/* -------------------------------------------------------------------------------- */

  /** cria um profile default, com typeid e objid = typeIdDefault.BW_PROFILE,
   *  e com um conjunto padrao de eventos ja registrados.
   * @return BW_Profile	profile exemplo criado
   */
  public static BW_Profile makeProfile () {
  	//BW_Profile profD = new BW_Profile (typeIdDefault.BW_PROFILE,typeIdDefault.BW_PROFILE);
  	BW_Profile profD = new BW_Profile (typeIdDefault.BW_PROFILE,UML_BW_Mediator.GetNextObjId());
  	
  	//intervalo "monstro" para pegar todos os eventos
  	//de 1999/jan/1 0:0:0 - 2999/dez/31 23:59:59
  	Calendar c1 = Calendar.getInstance (TimeZone.getDefault(),Locale.getDefault());  	
  	c1.set ( 1999 , 0 , 1 , 0 , 0, 0 );
  	Calendar c2 = Calendar.getInstance (TimeZone.getDefault(),Locale.getDefault());  	
  	c2.set ( 2010 , 11 , 31 , 23 , 59, 59 );
  	BW_TimeInterval tiD = new BW_TimeInterval (0,0,c1,c2);
  	tiD.setTimeZone (TimeZone.getDefault());
  	tiD.setLocale (Locale.getDefault());
  	
  	profD.setInterval (tiD);
  	BW_Event evt;
  	//BW_Event evt = new BW_Event (UML_BW_Mediator.SAVEDIAGRAM,typeIdDefault.UNSET);
  	//profD.insertEvent (evt);
  	evt = new BW_Event (UML_BW_Mediator.NEWCLASS,typeIdDefault.UNSET);
  	profD.insertEvent (evt);
  	evt = new BW_Event (UML_BW_Mediator.CLASSEDITED,typeIdDefault.UNSET);
  	profD.insertEvent (evt);
  	evt = new BW_Event (UML_BW_Mediator.CLASSREMOVED,typeIdDefault.UNSET);
  	profD.insertEvent (evt);
  	
  	return profD;
  }

/* -------------------------------------------------------------------------------- */


     /////////////////////////////// METODOS REDEFINIDOS //////////////////////


/* -------------------------------------------------------------------------------- */

     /** envia o sinal de inicio de processamento para o mediator,
      * avisando-o do inicio das atividades. Chamado normalmente pelo metodo
      * estatico "Start" da classe GW_BW_Mediator
      * @see 	GW_BW_Mediator#Start
      * @return boolean	true se foi possivel inicializar o mediador concreto;
      *			false se nao foi possivel
      */
     public boolean _start () {
        return this._start("localhost");
     }

/* -------------------------------------------------------------------------------- */

     /** envia o sinal de inicio de processamento para o mediator,
      * avisando-o do inicio das atividades. Chamado normalmente pelo metodo
      * estatico "Start" da classe GW_BW_Mediator
      * @see 	GW_BW_Mediator#Start
      * @param	String host	servidor utilizado
      * @return boolean	true se foi possivel inicializar o mediador concreto;
      *			false se nao foi possivel
      */
     public boolean _start (String server) {
          boolean ret = false;

          try {
              //inicializa o implementor
              implementor = new ST_DatabaseImplementor (server);
              //System.out.println ("[DBG]UML_BW_Mediator._start: starting implementor");

              //inicializa o lastObjId
              ST_Setup setup = implementor.read (new ST_Setup());
              if (setup!=null) {
	              UML_BW_Mediator.lastObjId = setup.getLastObjId();
	              this.host = server;
	              this.port = setup.getPort();	
	      }
	      else {
	              this.host = server;
	              this.port = 5433;		
	      }
	
	
              //inicializa camada storage
              Vector v1 = ST_StartUp.Start(implementor);
              //System.out.println ("[DBG]UML_BW_Mediator._start: starting storage");

              //inicializa camada control
              Vector v2 = CL_StartUp.Start (v1);
              this.monitor = CL_StartUp.getMonitorFacade();
              //System.out.println ("[DBG]UML_BW_Mediator._start: starting control");

              //seta o controle dos Ids dos eventos para ca
              this.monitor.setControlIds(false);

              //inicializa camada interface
              this.Interface = UI_StartUp.Start(v2);
              //System.out.println ("[DBG]UML_BW_Mediator._start: starting Interface");

              //inicializa mainInterface
              //this.mainInterface = new AwarenessMainWindow(this.Interface);
              this.mainInterface.setUI_Interface (this.Interface);
              //System.out.println ("[DBG]UML_BW_Mediator._start: starting main user interface");

              //seta interface principal
              Interface.addGUIElement (mainInterface,this.makePrototypes());

              //le o grupo, o papel e os usuarios do grupo
              this.readDefaults();
              //System.out.println ("[DBG]UML_BW_Mediator._start: reading defaults");

              UML_BW_Mediator.myself = this;

              ret = true;

          } catch (Exception e) {
              ret = false;
          }

          return ret;
     }

/* -------------------------------------------------------------------------------- */

     /** envia o sinal de fim do processamento para o mediator,
      * avisando-o do termino das atividades. Causa o fechamento das
      * comunicacoes com o servidor.
      * Normalmente chamado pelo metodo estatico "Stop" da classe
      * GW_BW_Mediator.
      * @see 	GW_BW_Mediator#Stop
      * @return boolean	true se foi possivel finalizar o mediador concreto;
      *			false se nao foi possivel
      */
     public boolean _stop () {
     	   boolean ret = false;
     	
     	  try {
     	  	//salva os eventos ainda na fila
     	  	this.monitor.eventOccurence (events);
     	  	this.events.removeAllElements();
     	  	
     	  	//informa o terminate a interface     	  	
     	  	this.Interface.terminate();
     	  	ret = true;
     	  } catch (Exception e) {
     	  	ret = false;
     	  }
     	
     	  /*
     	  //chama o stop do implementor para fechar o acesso ao banco
	  if (implementor != null)
     	        ret = implementor.close();
	  else  ret = true;
          */
     	  return ret;
     }

/* -------------------------------------------------------------------------------- */


     /** assinala o inicio da ocorrencia de um evento.
      * Assinalado o inicio de um evento, este devera ficar
      * em uma pilha especifica para o seu tipo, aguardando
      * ate que seja assinalado o seu termino. Normalmente
      * chamado pelo metodo "BeginEvent" da classe mae.
      *
      * @see	GW_BW_Mediator#BeginEvent
      * @param	BW_Event evt	evento que se inicia
      * @param	int typeId	tipo do evento
      * @return boolean	true	se tudo ocorreu como de esperado
      *			false	caso contrario
      *
      */
     public boolean _beginEvent (int typeId, BW_Event evt) {
        boolean ret = false;

        try {
     		//identifica o tipo do evento
     		//System.out.println ("[DBG]UML_BW_MEDIATOR._BEGINEVENT: analizing "+evt);	     	
     		int type = evt.getTypeId();     		
     	
     		//verifica se o intervalo do evento esta setado - FILLEVENT FAZ ISSO
     		//BW_TimeInterval interv = evt.getInterval();
     		//if (interv==null) {
     		//	interv = new BW_TimeInterval (tmzD,lclD);
     		//	evt.setInterval (interv);
     		//}
     		
     		//preenche o que nao estiver ainda preenchido no evento
     		evt.setObjId (this.getNextObjId());
     		this.fillEvent (type,evt);
		//System.out.println ("[DEBUG] UML_BW_MEDIATOR._BEGINEVENT: filled "+evt);	     	
		
		
     		//coloca o evento ainda incompleto na pilha certa
        	this.push (type,evt);
        	
        	ret = true;
        	
        } catch (Exception e) {
                ret = false;
                e.printStackTrace();
        }	

        return ret;
     }

/* -------------------------------------------------------------------------------- */


     /** assinala o inicio da ocorrencia de um evento.
      * Assinalado o inicio de um evento, este devera ficar
      * em uma pilha especifica para o seu tipo, aguardando
      * ate que seja assinalado o seu termino. Normalmente
      * chamado pelo metodo "BeginEvent" da classe mae.
      *
      * @see	GW_BW_Mediator#BeginEvent
      * @param	int typeId	tipo do evento
      * @return boolean	true	se tudo ocorreu como de esperado
      *			false	caso contrario
      *
      */
     public boolean _beginEvent (int typeId) {
     	   BW_Event evt = new BW_Event(typeId,UML_BW_Mediator.lastObjId);
     	   return this._beginEvent (typeId,evt);	
     }


/* -------------------------------------------------------------------------------- */

      /** assinala o termino da ocorrencia de um evento.
      * Assinalado o termino de um evento, este devera ser
      * retirado da pilha especifica para o seu tipo, e
      * sera calculada a sua duracao total. Normalmente
      * chamado pelo metodo "BeginEvent" da classe mae.
      *
      * @see	GW_BW_Mediator#EndEvent
      * @param 	BW_Event evt	evento que termina
      * @param	int typeId	tipo do evento
      * @return boolean	true	se tudo ocorreu como de esperado
      *			false	caso contrario
      *
      */
     public boolean _endEvent (int typeId, BW_Event evt) {
	  boolean ret = false;     	
     	
     	  try {
     	  	//System.out.println ("[DEBUG] UML_BW_MEDIATOR._ENDEVENT: processing "+evt);

		//busca o evento na pilha
		BW_Event e = this.pop (typeId);
		//System.out.println ("[DEBUG] UML_BW_MEDIATOR._ENDEVENT: poping out "+e);
   	  	
   	  	//seta o objid do evento e preenche o que nao esta preenchido
		evt.setObjId (e.getObjId());
		this.fillEvent (typeId,evt);
     	  	
     	  	//pega o intervalo do evento - FILL EVENT PREECHE O INTERVALO
     	  	BW_TimeInterval i1 = evt.getInterval();
     	  	//if (i1==null)
     	  	//	i1 = new BW_TimeInterval (tmzD,lclD);
     	  		
         	//pega o tempo final    	         	
         	Calendar c = Calendar.getInstance (evt.getInterval().getTimeZone(),evt.getInterval().getLocale());
         													
		//seta o tempo do evento (inicial e final)		
		BW_TimeInterval i2 = e.getInterval();
					
		//i1.setBegin (i2.getBegin());
		//i1.setEnd (c);
		i1.setInterval (i2.getBegin(),c);
		
		evt.setInterval (i1);
		//System.out.println ("[DEBUG] UML_BW_MEDIATOR._ENDEVENT: with interval "+evt);			
															
		//coloca na fila de eventos prontos
		this.finished (evt);
					
		ret = true;
					
	   } catch (Exception ex) {
	   	System.out.println ("[DBG]UML_BW_MEDIATOR._ENDEVENT: EXCEPTION");
	   	ex.printStackTrace ();
		ret = false;	
	   }							
				
	   return ret;

     }


/* -------------------------------------------------------------------------------- */

      /** assinala o termino da ocorrencia de um evento.
      * Assinalado o termino de um evento, este devera ser
      * retirado da pilha especifica para o seu tipo, e
      * sera calculada a sua duracao total. Normalmente
      * chamado pelo metodo "BeginEvent" da classe mae.
      *
      * @see	GW_BW_Mediator#EndEvent
      * @param	int typeId	tipo do evento
      * @return boolean	true	se tudo ocorreu como de esperado
      *			false	caso contrario
      *
      */
     public boolean _endEvent (int typeId){
	  boolean ret = false;     	
     	
     	  try {
		//busca o evento na pilha
		BW_Event evt = this.pop (typeId);
		//System.out.println ("[DEBUG] UML_BW_MEDIATOR._ENDEVENT: popped "+evt);			
				
		//pega o tempo final    	
         	Calendar c = Calendar.getInstance (evt.getInterval().getTimeZone(),evt.getInterval().getLocale());

		//seta o tempo do evento final
		BW_TimeInterval i1 = evt.getInterval();
		//i1.setEnd (c);	
		i1.setInterval (i1.getBegin(),c);
		evt.setInterval (i1);
		//System.out.println ("[DEBUG] UML_BW_MEDIATOR._ENDEVENT: with interval "+evt);			
					
		//preenche o que nao esta preenchido
		this.fillEvent (typeId,evt);
		//System.out.println ("[DEBUG] UML_BW_MEDIATOR._ENDEVENT: filled "+evt);			
										
		//coloca na fila de eventos prontos
		this.finished (evt);
					
		ret = true;
					
	   } catch (Exception ex) {
		ret = false;	
	   }							
			
	   return ret;
     }

/* -------------------------------------------------------------------------------- */

     /** assinala o cancelamento da ocorrencia de um evento.
      * O proximo evento na pilha de eventos do tipo indicado
      * devera ser retirado desta e descartado. Normalmente
      * chamado pelo metodo "CancelEvent" da classe mae.
      *
      * @param	int typeId	tipo do evento
      * @return boolean	true	se tudo ocorreu como de esperado
      *			false	caso contrario
      * @see	GW_BW_Mediator#CancelEvent
      */
      public boolean _cancelEvent (int typeId){
	  boolean ret = false;
					
	  try {					
		 //tira o evento da fila
		 BW_Event evt = this.pop (typeId);
							 
		 if (evt!=null) ret = true;
		 		
	  } catch (Exception e) {
		 ret = false;		
	  }
				  
	  return ret;						
     }
     
/* -------------------------------------------------------------------------------- */

     /** seta o elemento de interface principal. Normalmente
      * chamado pelo metodo "SetMainUIElement" da classe mae.
      * @see	GW_BW_Mediator#SetMainUIElement
      * @param UI_Element main	elemento principal (gerente de interface)
      * @return	boolean	true se tudo ocorreu bem, false caso contrario
      */
     public boolean _setMainUIElement (UI_GUIElement main){
         mainInterface = (AwarenessMainWindow) main;     	 	
     	 return true;	
     }

/* -------------------------------------------------------------------------------- */

     /** retorna o elemento de interface principal, se disponivel
      * @return	UI_Element	elemento principal (gerente de interface)
      *				null caso este nao tenha sido setado 	
      */
     public UI_GUIElement _getMainUIElement (){
     	  UI_GUIElement ret = null;
     	  ret = (UI_GUIElement) mainInterface;
     	  //System.out.println ("reading main window "+ret);
     	  return ret;
     }
/* -------------------------------------------------------------------------------- */

     /** Assinala a entrada de um usuario no sistema
      *  @param	BW_Member member	usuario ingresso, com o login setado corretamente
      *  @return BW_Member		membro com dados atualizados
      */
     public BW_Member _userLogin (BW_Member memb) {
        BW_Member mb = null;
        try {
          //busca o uid do usuario
          Integer uid = (Integer) members.get (this.mountKey (memb.getLogin()));
          mb = groupD.getAMember (uid.intValue());
          //System.out.println ("member readed: "+mb);

          if (!mb.setActivePaper (paperD)) {

          	BW_Profile profD = UML_BW_Mediator.makeProfile();          	
          	mb.insertPaper (paperD,profD);
          	mb.setActivePaper (paperD);
          }

	  this.setProfile(mb);
	  	
	  this.Interface.setMemberInfo(mb);	  	
          this.Interface.setMember (mb);

          this.Interface.Notify();

        } catch (NullPointerException npe) {
           //membro novo
           if (mb==null) {
           	mb = new BW_Member (memb.getLogin(),this.getNextObjId());
           	mb.setTypeId (typeIdDefault.BW_MEMBER);
           	mb.setName (memb.getLogin());
           	try {
           	    mb.setMachine (InetAddress.getLocalHost().getHostName());
           	} catch (Exception e) {}
           	BW_AwarenessProfile awp = mb.getAwarenessProfile ();
           	awp.setObjId (this.getNextObjId());
           	mb.setAwarenessProfile (awp);
           	
           	this.monitor.addMember(mb);
           	
           	this.members.put (this.mountKey(mb.getLogin()),new Integer(mb.getObjId()));
           	
           	this._userLogin (mb);
           }

        }

        return mb;
     }

/* -------------------------------------------------------------------------------- */

     /** Assinala a saida de um usuario no sistema
      *  @param	BW_Member member	usuario, com o login setado corretamente
      */
     public void _userLogout (BW_Member memb) {
        try {
     	    //System.out.println ("[DBG]UML_BW_MEDIATOR._USERLOGOUT: logging out "+members);
     	    Integer uid = (Integer) this.members.get (this.mountKey(memb.getLogin()));
     	    BW_Member realmember = this.groupD.getAMember (uid.intValue());
     	    //System.out.println ("[DBG]UML_BW_MEDIATOR._USERLOGOUT: logging out "+realmember);
            this.Interface.memberLogout(realmember);
        } catch (Exception e) {
            System.out.println ("[DBG]UML_BW_MEDIATOR._USERLOGOUT: Exception ");
            e.printStackTrace();
        }
     }

/* -------------------------------------------------------------------------------- */

    /**   altera o profile pessoal do membro para deste o ultimo login ate hoje */
     public void setProfile (BW_Member mb) {
          Calendar hoje = Calendar.getInstance (tmzD,lclD);
          BW_Profile ProfPers = mb.getAwarenessProfile().getPersonalProfile(0);

          if (ProfPers==null) {
           	ProfPers = UML_BW_Mediator.makeProfile();
          }

          BW_TimeInterval PersInt = ProfPers.getInterval();
          BW_TimeInterval LastLog = mb.getLastLogin();

          if (LastLog!=null)
	        PersInt.setInterval (LastLog.getBegin(),hoje);
	  else
	  	PersInt.setInterval (PersInt.getBegin(),hoje);

	  	          	
	  ProfPers.setInterval (PersInt);
	  mb.removePersonalProfile(ProfPers);
	  mb.insertPersonalProfile(ProfPers);	

	  //setando o mesmo intervalo para o awareness profile
	  BW_AwarenessProfile awareness_profile = mb.getAwarenessProfile();
	  awareness_profile.setInterval (PersInt);
	  mb.setAwarenessProfile (awareness_profile);
	
	  //System.out.println ("[DBG] UML_BW_Mediator.setProfile: member"+mb);
    }

/* -------------------------------------------------------------------------------- */


    /** retorna o typeid do grupo default utilizado pelo CUTE.
     * @see #GROUPTYPEID
     * @see #groupD
     */
     public int _getGroupTypeId () {
        return UML_BW_Mediator.GROUPTYPEID;
     }

/* -------------------------------------------------------------------------------- */

    /** retorna o objid do grupo default utilizado pelo CUTE.
     * @see #GROUPOBJID
     * @see #groupD
     */
     public int _getGroupObjId () {
        return UML_BW_Mediator.GROUPOBJID;
     }


/* -------------------------------------------------------------------------------- */

   /** retorna o host servidor em uso
    * @return	String	nome do servidor
    */
   public String _getHost() {
   	return this.host;   	
   }

/* -------------------------------------------------------------------------------- */

    /** retorna a porta para o servidor de identificador de objetos
     * @return	int	porta utilizada
     */
    public int _getPort () {
    	return this.port;
    }

/* -------------------------------------------------------------------------------- */

    /** retorna o proximo objId */
     public static int GetNextObjId () {
     	  UML_BW_Mediator lixo = (UML_BW_Mediator) GW_BW_Mediator.GetConcreteMediator();
     	  String shost; int sport;
     	  if (lixo!=null) {
     	  	shost = lixo._getHost();
     	  	sport = lixo._getPort();
     	  }
     	  else {
     	  	System.out.println ("Concrete NULL");
     	  	shost = "localhost";
     	  	sport = 5433;     	  	
     	  }
     	
 	  clientID cid = new clientID (shost,sport);
	  int id = cid.getID().intValue();
	  //atualiza se o valor retornado for maior
	  if (id>lastObjId)
	  	lastObjId = id;

          //System.out.println ("[DBG]UML_BW_Mediator.GetNextObjId: next id: "+id+"("+lastObjId+")");
     	  return id;
     }

/* -------------------------------------------------------------------------------- */

    /** retorna o papel default */
    public static BW_Paper GetPaperDefault() {
    	return myself.paperD;
    }
}
