/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE CL_AWARENESS
 * PACOTE BW.CONTROL
 * DATA DE CRIACAO 11 / 01 / 2001
 */

package BW.control;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Calendar;
import java.util.Vector;

import BW.storage.FC_ST_Context;
import BW.typeIdDefault;
import BW.control.*;
import BW.kernel.*;

/**
 * classe CL_Awareness e responsavel pela contextualizacao
 * dentro do BW
 * @autor Manuele K P
 * @version 1.0
 */
public class CL_Awareness {

    ///////////////////////////////// ATRIBUTOS //////////////
	
	/** lista dos participantes ativos no grupo, no formato (uid,member) */
	Hashtable actives;
	
	/** controle do horario de entrada no sistema de cada usuario ativo,
	 * no formato (uid,Calendar)
	 */
	Hashtable login_time;
	
	/** conexao com o gerente de monitoramento */
	CL_Monitor monitor;

	/** conexao com a camada de armazenamento */
	FC_ST_Context storage;

	/** conexao com a camada de interface */
	FC_CL_Context ui;
	
    //////////////////////////////// CONSTRUTOR //////////////
    public CL_Awareness () {
       actives = new Hashtable();
       login_time = new Hashtable();
       monitor = null;
       storage = null;
       ui = null;
    }


    //////////////////////////////// METODOS //////////////////


    /////////////////// sobre os membros
    	
	/** avisa sobre o ingresso de um usuario
	 * especifico no sistema
	 * @param	int uid	identificador do usuario
  	 * @return 	BW_Member objeto sobre o usuario ou
	 *		null se o membro nao pertencer ao grupo
	 */
	public BW_Member memberLogin(int uid) {
	    BW_Member mb = null;	
	    try {	
		//busca no grupo
		BW_Group group = monitor.getGroup();		
        	mb = group.getAMember(uid);
        	
		//inclui na lista de ativos
		actives.put (new Integer(mb.getObjId()),mb);
		login_time.put ( new Integer(mb.getObjId()), Calendar.getInstance());
		
	    } catch (NullPointerException npe) {
             	mb = null;
	    }
	    return mb;
	}


	/** avisa o monitor que um usuario deixou o sistema
	 * @param 	BW_Member memb	membro q deixou o sistema
	 */
	public void memberLogout (BW_Member memb) {
	    try {
		actives.remove (new Integer (memb.getObjId()));
		//System.out.println ("[DBG] CL_Awarereness.memberLogout: SETTING LASTLOGIN");
		Calendar log_inicio = (Calendar) login_time.remove (new Integer (memb.getObjId()));		
		Calendar log_fim = Calendar.getInstance();
		BW_TimeInterval lastlogin = memb.getLastLogin();
		if (lastlogin!=null) {
			lastlogin.setInterval(log_inicio,log_fim);
		} else {
			lastlogin = new BW_TimeInterval (log_inicio,log_fim);
		}
		memb.setLastLogin (lastlogin);
		
		this.setMemberInfo (memb);
		System.out.println ("[DBG]CL_Awarereness.memberLogout: MEMBER LOGOUT - LASTLOGIN SET");
	    } catch (Exception e) {
	     	System.out.println ("[DBG]CL_Awarereness.memberLogout: IMPOSSIBLE TO LOGOUT");
	    }	
		
	}
	
       /** Retorna as informacoes sobre um membro qualquer do gruop
        * @param BW_Member memb	objeto BW_Member com o uid do membro que se deseja conhecer
        * @return BW_Member	informacoes completa sobre o membro
        */
       public BW_Member getMemberInfo (BW_Member memb) {       		
                BW_Member ret = null;
                ret = monitor.getGroup().getAMember(memb.getObjId());
                return ret;
       }

       /** Altera as informacoes sobre um membro do grupo
        * @param BW_Member memb	membro que se esta alterando
        */
       public void setMemberInfo (BW_Member memb) {
       		monitor.getGroup().setAMember(memb.getObjId(),memb);
       		//System.out.println ("[DBG] CL_Awarereness.setMemberInfo: Member SET");
       }

       /** Retorna o objeto representativo do grupo
        * @return 	BW_Group	gruop ativo no momento
        */
       public BW_Group getGroup () {
        	BW_Group ret = monitor.getGroup();
        	return ret;
       }


     /////////////////////////// eventos

     /** Retorna os eventos de interesse de um usuario
      * @param	BW_Member memb	membro do grupo que se deseja contextualizar
      * @return	Vector	vetor de BW_Events ocorridos de interesso do membro
      */
       public Vector getEvents (BW_Member memb) {
       		Vector ret = null;
       		try {
       			//REFAZER!!!!!!!!!!!!!!!!!
       			//Vector v = this.processProfile(memb);
       			//ret = storage.loadEvents ((Vector)v.firstElement(), (BW_TimeInterval)v.lastElement());
       			BW_Profile profFinal = this.processProfile(memb);
       			System.out.println ("[DBG]CL_Awareness.getEvents: final profile: "+profFinal);
       			ret = this.storage.readEvents (profFinal);
       			
       			//para cada evento, associa-o ao seu responsavel ja carregado na memiria
       			BW_Group group = this.getGroup();
       			for (int i=0; i<ret.size(); i++) {
       				BW_Event evento = (BW_Event) ret.elementAt(i);
       				BW_Member respon = evento.getResponsible();
       				if (respon!=null) {
       					BW_Member real = group.getAMember (respon.getObjId());
       					if (real!=null) evento.setResponsible (real);
       				}
       			}
       			
       		} catch (Exception e) {
       			ret = new Vector();
       		}	
       		
       		return ret;
       }


       //////////////////////////  SET'S e GET'S

       /** seta a ligacao com o gerente de monitoramento
        * @param 	CL_Monitor	gerente de monitoramento
        */		
       public void setCLMonitor (CL_Monitor mon) {
       		this.monitor = mon;
       }

       /** seta a conexao com a camada de armazenamento
        * @param	FC_ST_Context cont	fachada para camada de armazenamento
        */
       public void setFCSTContext (FC_ST_Context cont) {
       		this.storage = cont;
       }

	/** seta a conexao com a camada de interface
	 * @param	FC_CL_Context cont	fachada para a camada de interface
	 */
	public void setFCCLContext (FC_CL_Context cont) {
		this.ui = cont;
	}
	

       /** retorna o gerente de monitoramento
        * @return	CL_Monitor	gerente de monitoramento
        */		
       public  CL_Monitor getCLMonitor () {
       		return this.monitor;
       }

       /** retorna a conexao com a camada de armazenamento
        * @return	FC_ST_Context	fachada para funcoes de contextualizacao
        *				da camada de armazenamento
        */
       public FC_ST_Context getFCSTContext () {
       		return this.storage;
       }

	/** retorna a conexao com a camada de interface
	 * @return	FC_CL_Context	fachada para funcoes de contextualizacao
	 *				da camada de controle
	 */
	public FC_CL_Context getFCCLContext () {
		return this.ui;
	}
	
	
	/** retorna uma hash no formato (uid, member) com todos
	 * os membros ativos no momento
	 */
	public Hashtable getActiveUsers () {
		return this.actives;
	}
	
	
       ////////////////////////// METODOS ESPECIAIS


       /////////// FALTA!!!!!!!! //////////////

       /** Realiza o processo de filtragem dos eventos: com base nos profiles
        * do usuario, ela monta um profile novo, chamado Profile Final, que
	* descreve todos os eventos que interessam ao usuario e o intervalo
	* de interesse.
	*/
       public BW_Profile processProfile  (BW_Member memb) {
		BW_Profile profFinal = new BW_Profile (typeIdDefault.BW_PROFILE,typeIdDefault.UNSET);
		
		//pega todos os profiles envolvidos
		
		BW_AwarenessProfile user_pers;
		BW_Profile user_perspaper;
		BW_Profile paper_profile;
		
		user_pers = memb.getAwarenessProfile();
		
		BW_Paper act_paper = memb.getActivePaper();
		if (act_paper != null) {
			act_paper = memb.getPaper (act_paper.getName());
			user_perspaper = memb.getPersonalPaperProfile(act_paper);		
			paper_profile = act_paper.getProfile();
		}
		else { //sem papel definido
			user_perspaper = new BW_Profile (typeIdDefault.BW_PERSONALPAPERPROFILE,typeIdDefault.UNSET);
			paper_profile = new BW_Profile (typeIdDefault.BW_PAPERPROFILE,typeIdDefault.UNSET);			
		}
								
		//monta o conjunto de eventos de interesse
		Hashtable evts_pers = user_pers.getAllPersonalEvents();
		Hashtable evts_perspap = user_perspaper.getEvents();
		Hashtable evts_pap = paper_profile.getEvents();
		
		Enumeration keys = evts_perspap.keys();
		while (keys.hasMoreElements()) {
			Object tkey = keys.nextElement();
			evts_pers.put (tkey,evts_perspap.get(tkey));
		}
		
		keys = evts_pap.keys();
		while (keys.hasMoreElements()) {
			Object tkey = keys.nextElement();
			evts_pers.put (tkey,evts_pap.get(tkey));
		}
		
		profFinal.setEvents (evts_pers);
		
		//faz o merge de todos os intervalos
                BW_TimeInterval int_pers = user_pers.getGlobalInterval();
                BW_TimeInterval int_perspap = user_perspaper.getInterval();
                //BW_TimeInterval int_paper = paper_profile.getInterval();

		BW_TimeInterval intFinal = int_pers.merge (int_perspap);
		//intFinal = intFinal.merge (int_paper);
		

                /* **	*******************************************************	** *
                 * **	optou por desconsiderar o intervalo de tempo indicado	** *
                 * **	no profile do papel, pois este, alem de ser geral para	** *
                 * **	todos os membros que desempenham este papel, tem sua	** *
                 * **	maior importancia nos eventos nele assinalados, que	** *
                 * **	representam os tipos de eventos importantes para o 	** *
                 * **	bom desempenho do papel. 				** *
                 * **	******************************************************	** */
	
		
		profFinal.setInterval (intFinal);
		
		//retorna o profile Final
		return profFinal;
		
       }



       /** notifica a camada de interface quanto a presenca de novos eventos,
        * no caso da camada de armazenamento ter alertado esta quanto a isso.
        */
       public void Notify () {
           try {
       	    ui.Notify();	
       	   } catch (NullPointerException npe) {}
       	
       }

	/** Finaliza as operacoes */
	public void terminate () {
	       System.out.println ("[DBG]CL_Awareness.terminate: finalizing activities");
	       monitor.terminate () ;
	       storage.terminate();
	}
}
