/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE BW_Event
 * PACOTE BW.KERNEL
 * DATA DE CRIACAO 15 / 07 / 2000
 */

package BW.kernel;

import java.util.Hashtable;

import BW.typeIdDefault;

/**
 * Classe BW_Event: descreve um evento simples dentro do framework BW
 * @author Manuele Kirsch Pinheiro
 * @version 1.2
 */
public class BW_Event extends BW_Element {

     ///////////////////////////////// ATRIBUTOS //////////////

    /** nome do evento */
    String name;

    /** descricao do evento */
    String description;

    /** detalhes sobre o evento - opcional*/
    String details;

    /** membro do grupo responsavel pelo evento */
    BW_Member responsible;

    /** intervalo de ocorrencia do evento */
    BW_TimeInterval interval;


    //////////////////////////////// CONSTRUTORES ////////////

    /** construtor - somente indica o identificador do tipo do
     * evento e o identificador do objeto evento
     * @param 	int typeId	id do tipo de evento
     * @param 	int objId	id do evento proprieamente dito
     */
    public BW_Event (int typeId, int objId) {
	super (typeId, objId);

	this.description = new String ();
	this.details = new String();
    	this.name = new String();
    	
	//CUIDADO!! Tratar NullPointerException
	//o resposavel e o intervalo vao ser null
	//enquanto nao forem setados
	this.responsible = null;	
	this.interval = null;
    }



    /** construtor - indica o identificador do tipo do
     * evento e do proprio objeto evento, alem do membro
     * responsavel pelo evento
     * @param 	int typeId	id do tipo de evento
     * @param 	int objId	id do evento proprieamente dito
     * @param 	BW_Member resp	responsavel pelo evento
     */
    public BW_Event (int typeId, int objId, BW_Member resp) {
	super (typeId, objId);

	this.description = new String ();
	this.details = new String();
    	this.name = new String();	
        this.responsible = (BW_Member)resp.clone();

	//CUIDADO!! Tratar NullPointerException
	//o intervalo vao ser null enquanto nao for setado	
	this.interval = null;
    }

    /** construtor simples */
    public BW_Event () {
      	super (typeIdDefault.BW_EVENT,(typeIdDefault.BW_EVENT+typeIdDefault.OBJID)+1);
	this.description = new String ();
	this.details = new String();
    	this.name = new String();	

	//CUIDADO!! Tratar NullPointerException
	//o resposavel e o intervalo vao ser null
	//enquanto nao forem setados
	this.responsible = null;	
	this.interval = null;
    }


	/////////////////////////////// METODOS //////////////////

        /**
         * metodo setName seta o responsavel pelo evento
         * @param 	String Name	nome do evento
         */
         public void setName (String Name) {
         	this.name = new String (Name);
         	//System.out.println ("[DEBUG] - name: "+this.name);
         }
	
	/**
         * metodo setResponsible seta o responsavel pelo evento
	 * @param	BW_Member resp	responsavel
	 */
	public void setResponsible (BW_Member resp) {
		this.responsible = (BW_Member) resp.clone();
	}

	/**
	 * metodo setDescrioption seta a descricao do evento
	 * @param 	String descrip	descricao do evento
	 */
	public void setDescription (String descrip) {
         	this.description = new String (descrip);
	}


	/**
	 * metodo setDetail altera a descricao dos detalhes
	 * sobre o evento
	 * @param	String detail	detalhes sobre o evento
	 */
	public void setDetails (String details) {
         	this.details = new String (details);
	}

	/**
	 * metodo setInterval seta o intervalo de duracao evento
	 * @param   BW_TimeInterval interv	intervalo do evento
	 */
	public void setInterval (BW_TimeInterval interv) {
            this.interval = new BW_TimeInterval (interv);	
	}

		
	/**
         * metodo getName retorna o nome do evento
         * @return	String 	nome do evento
         */
         public String getName () {
         	return new String (this.name);
         }
	
	
	/**
	 * metodo getResponsible retorna o responsavel pelo evento
	 * @return 	BW_Member	responsavel pelo evento
	 */
	public BW_Member getResponsible () {		
        	if (responsible != null)
        		return (BW_Member)responsible.clone();
        	else 	return null;
	}	

	/**
	 * metodo getInterval retorna o intervalo de tempo do evento
	 * @return 	BW_TimeInterval 	intervalo do evento
	 */
	public BW_TimeInterval getInterval () {
		if (interval!=null)
        		return (BW_TimeInterval)interval.clone();
        	else
        		return null;	
	}	

	/**
	 * metodo getDescription retorna a descricao do evento
	 * @return 	String	descricao
	 */
	public String getDescription () {
        	return new String (description);
	}	

	/**
	 * metodo getDetails retorna a informacao sobre
         * detalhes do evento
	 * @return 	String	detalhes
	 */
	public String getDetails () {
        	return new String (details);
	}	

   	/** metodo getFields retorna um Hashtable com os
	 * pares (atributo,valor) do objeto
    	 * @return Hashtable  - atributo,valor
    	 */
	public Hashtable getFields () {
		Hashtable ht = super.getFields ();
		//System.out.println ("[DEBUG] name: "+this.name);
		ht.put ("name",this.getName());
		
		//System.out.println ("[DEBUG] responsible");
		if (this.getResponsible() != null)
			ht.put ("responsible",this.getResponsible());
			
		//System.out.println ("[DEBUG] interval: "+this.interval);
		if (this.getInterval() != null)
	 		ht.put ("interval",this.getInterval());
 		//System.out.println ("[DEBUG] description: "+this.description);
		ht.put ("description",this.getDescription());
		//System.out.println ("[DEBUG] details: "+this.details);
		ht.put ("details",this.getDetails());
		return (ht);
	}

	
        /** metodo setFields seta os atributos do evento
         * com base em uma Hashtable, como a retornada
         * pelo metodo getFields
         * @param Hashtable fields	campos para o objeto
         * @return boolean	true se conseguiu setas os campos
         *			false caso contrario
         * @see	#getFields
         */
        public boolean setFields (Hashtable fields) {
		Hashtable bkp = super.getFields();
        	boolean ret = super.setFields (fields);

        	if (ret && fields.containsKey("name") && fields.containsKey("responsible")
        			&& fields.containsKey("interval") && fields.containsKey("details")
        			&& fields.containsKey("description") ) {
        		this.setName ((String) fields.get ("name"));
        		this.setResponsible ((BW_Member) fields.get("responsible"));
        		this.setInterval ((BW_TimeInterval) fields.get("interval"));
        		this.setDetails ((String) fields.get ("details"));
        		this.setDescription ((String) fields.get ("description"));
        		ret = true;		
        	}
        	else {
			ret = false;
			super.setFields (bkp);
			}
        	return ret;
        }

        public Object clone ()	{
        	BW_Event copy = new BW_Event (this.getTypeId(),this.getObjId());
        	copy.setName (this.getName());
        	copy.setResponsible (this.getResponsible());
        	copy.setInterval (this.getInterval());
        	copy.setDescription (this.getDescription());
        	copy.setDetails (this.getDetails());
                return (Object) copy;
        }
	
}

