 /**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * Classe BW_Paper
 * PACOTE BW.KERNEL
 * DATA DE CRIACAO 11 / 11 / 2000
 */

package BW.kernel;

import java.util.Hashtable;

import BW.typeIdDefault;

/**
 * Classe BW_Paper: descreve a nocao de papel simples
 * dentro do framework BW
 * @author Manuele Kirsch Pinheiro
 * @version 1.2
 */
public class BW_Paper extends BW_Element {

     ///////////////////////////////// ATRIBUTOS //////////////

     /** nivel hierarquico do papel. Quanto mais baixo o numero,
      * mais alta a posicao hierarquica.
      */
      int level;

      /** nome do papel */
      String name;

      /** descricao do papel */
      String description;

      /** profile do papel*/
      BW_Profile profile;

      /** maior nivel hierarquico */
      public static int MAXIMUM = 0;

      /** menor nivel hierarquico */
      public static int MINIMUM = 32000;


      //////////////////////////////// CONSTRUTORES //////////

      /** construtor
       * @param	int typeId	identificador de tipo do papel
       * @param	int objId	identificador do objeto
       */
      public BW_Paper (int typeId, int objId) {
      		super (typeId,objId);
      		this.level = BW_Paper.MAXIMUM;
      		this.name = new String ();
      		this.description = new String();
      		this.profile = new BW_Profile (typeIdDefault.BW_PAPERPROFILE,typeIdDefault.BW_PAPERPROFILE+typeIdDefault.OBJID);
      }

      /** construtor
       * @param	int level	nivel hierarquico do papel
       * @see	#level
       * @param	String name	nome do papel
       */
      public BW_Paper (int level, String name) {
      		super (typeIdDefault.BW_PAPER,typeIdDefault.BW_PAPER+typeIdDefault.OBJID);
      		this.level = level;
      		this.name = new String (name);
      		this.description = new String();		
      		this.profile = new BW_Profile (typeIdDefault.BW_PAPERPROFILE,typeIdDefault.BW_PAPERPROFILE+typeIdDefault.OBJID);
      }

      public BW_Paper () {
      		super (typeIdDefault.BW_PAPER,typeIdDefault.BW_PAPER+typeIdDefault.OBJID);
      		this.level = BW_Paper.MAXIMUM;
      		this.name = new String ();
      		this.description = new String();		
      		this.profile = new BW_Profile (typeIdDefault.BW_PAPERPROFILE,typeIdDefault.BW_PAPERPROFILE+typeIdDefault.OBJID);
      }


      //////////////////////////////// METODOS ///////////////	

      //////////// sets

      /** metodo setLevel ajusta o nivel hierarquico do papel.
       * @param int level	novo nivel hierarquico do papel
       * @see	#level
       */
       public void setLevel (int Level) {
       		this.level = Level;
       }

      /** metodo setName altera o nome dado ao papel
       * @param String newname	novo nome para o papel
       */
       public void setName (String newname) {
        	this.name = new String (newname);
       }

      /** metodo setDescription seta uma descricao para o papel
       * @param String descrip	descricao para o papel
       */
       public void setDescription (String descrip) {
        	this.description = new String (descrip);
       }

       /** metodo setMaximum altera o valor do mais alto nivel
        * hierarquico do papel
        * @param int newmax	novo valor de MAXIMUM
        * @see #MAXIMUM
        */
        public void setMAXIMUM (int newmax) {
        	BW_Paper.MAXIMUM = newmax;

        }

       /** metodo setMinimum altera o valor do menor nivel
        * hierarquico do papel
        * @param int newmin	novo valor de MINIMUM
        * @see #MINIMUM
        */
        public void setMINIMUM (int newmin) {
        	BW_Paper.MINIMUM = newmin;

        }

       /** metodo setProfile seta o profile do papel
        * @param BW_Profile newprofile	novo profile para o papel
        */
        public void setProfile (BW_Profile newprofile) {
        	this.profile = (BW_Profile) newprofile.clone();
        }

       //////////// gets

      /**
       * metodo getLevel retorna o nivel hierarquico do papel
       * @see	#level
       * @return	int 	nivel do papel
       */
      public int getLevel () {
      		return this.level;
      }

      /**
       * metodo getName retorna o nome dado ao papel
       * @return 	String	nome do papel
       */
      public String getName () {
      		return new String (this.name);
      }


      /**
       * metodo getDescription retorna o nome dado ao papel
       * @return 	String	nome do papel
       */
      public String getDescription () {
      		return new String (this.description);
      }

      /**
       * metodo getProfile retorna o profile do papel
       * @return	BW_Profile	profile do papel
       */
      public BW_Profile getProfile () {
      		return (BW_Profile) this.profile.clone();
      }

      /**
       * metodo getMAXIMUM retorna o valor do maior nivel hieraquico
       * @return int maior nivel hierarquico do papel
       * @see #MAXIMUM
       */
      public int getMAXIMUM () {
      		return BW_Paper.MAXIMUM;
      }

      /**
       * metodo getMINIMUM retorna o valor do menor nivel hieraquico
       * @return int menor nivel hierarquico do papel
       * @see #MINIMUM
       */
      public int getMINIMUM () {
      		return BW_Paper.MINIMUM;
      }


   	/** metodo getFields retorna um Hashtable com os
	 * pares (atributo,valor) do objeto
    	 * @return Hashtable  - atributo,valor
    	 */
	public Hashtable getFields () {
		Hashtable ht = super.getFields();
		ht.put ("level",new Integer (this.getLevel()));
		ht.put ("name",this.getName());
		ht.put ("description",this.getDescription());
		ht.put ("profile",this.getProfile());
		return ht;			
	}

	
	/** metodo setFields seta os atributos do cobjeto
	 * com base em uma Hashtable no mesmo formato retornado
	 * pelo metodo getFields. Retorna true se tudo correu
	 * bem, e false caso contrario
	 * @param	Hashtable fields	atributos a serem preenchidos
	 * @return	boolean	true - se foi possivel ajustar os atribuitos
	 *			false - caso contrario
	 * @see	#getFields
	 */
	 public boolean setFields (Hashtable fields) {
		Hashtable bkp = super.getFields();
	 	boolean ret = super.setFields(fields);
	 	//System.out.println ("[DEBUG] - SUPER.GETFIELDS "+ret);
	 	try {
	 		if (ret && fields.containsKey ("level")  && fields.containsKey("name")
	 			&& fields.containsKey ("description") && fields.containsKey("profile") ) {
	 			
	 			this.setLevel ( ( (Integer)fields.get("level") ).intValue() );
	 			this.setName ( (String) fields.get ("name") );
	 			this.setDescription ( (String) fields.get ("description") );
	 			this.setProfile ( (BW_Profile) fields.get ("profile") );
	 			ret = true;	 	
	 		}
			else  {
				ret = false;
				super.setFields (bkp);
			}

	 	} catch  (NullPointerException npe){
	 		 ret = false;
			 super.setFields (bkp);
	 	}		
	 	
	 	return ret;
	 } 	

	
	
	public Object clone () {
		BW_Paper copy = new BW_Paper();
		copy.setLevel (this.getLevel());
		copy.setName (this.getName());
		copy.setDescription (this.getDescription());
		copy.setProfile (this.getProfile());
		return (Object) copy;
	}
	
}

