/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE ST_SETUP
 * PACOTE BW.STORAGE
 * DATA DE CRIACAO 10 / 01 / 2001
 */

package BW.storage;

import BW.kernel.*;
import BW.typeIdDefault;


/*
import java.util.Vector;
import java.util.Hashtable;
*/

/**
 * classe ST_Setup traz informacoes de setup
 * para o framework BW
 * @autor Manuele K P
 * @version 1.0
 */
public class ST_Setup {

    /** tempo de vida das informacoes armazenadas, dado em dias */
    int timelife;	

    /** servidor default */
    String server;

    /** se esta instancia pertence a um servidor ou nao */
    boolean isServer;


    /** ultimo identificador de objeto armazenado */
    int lastObjId;

    /** porta utilizada pelo servidor de objid.
     * Este seria um servidor para fornecimento centralizado de identificadores de objeto.
     */
    int port;

    /** valor default para o atributo lifeTime, dado em dias */
    public static final int MAX = 120;

    /** valor default para a porta
     * @see #port
     */
    public static final int MAXPORT = 5433;


    /** construtor default - como servidor local */
    public ST_Setup () {
       this.timelife = ST_Setup.MAX;
       this.server = "localhost";
       this.isServer = true;
       this.port =  ST_Setup.MAXPORT;
       this.lastObjId = typeIdDefault.OBJID;
    }


    /** seta o lifeTime
     * @see #timelife
     * @param int life	tempo de vida das informacoes
     */
    public void setTimeLife (int life) {
    	this.timelife = life;    	
    }

    /** seta o servidor utilizado
     * @param 	String server	servidor utilizado
     */
    public void setServer (String server) {
    	this.server = server;
    }

    /** seta se a maquina local possui um servidor
     * @param boolean isServer	true se e o servidor;
     				false, caso contrario
     */
    public void setIsServer (boolean isServer){
    	this.isServer = isServer;
    }

    /** altera o valor do ultimo identificador de objeto
     * @param	int lastid	novo valor para lastObjId
     * @see #lastObjId
     */
    public void setLastObjId (int lastid) {
    	this.lastObjId = lastid;
    }

    /** seta o valor para a porta
     * @param	int port	novo valor para a porta
     * @see #port
     */
    public void setPort (int port) {
    	this.port = port;
    }


    /** resgata o lifeTime
     * @see #timelife
     * @return int 	tempo de vida das informacoes
     */
    public int getTimeLife () {
    	return this.timelife;    	
    }

    /** retorna o  nome do servidor utilizado
     * @return 	String servidor utilizado
     */
    public String getServer () {
    	return this.server;
    }

    /** retorna se a maquina local possui um servidor
     * @return boolean 	true se e o servidor;
     			false, caso contrario
     */
    public boolean getIsServer (){
    	return this.isServer;
    }

    /** retorna o valor do ultimo identificador de objeto
     * @return	int 	valor do lastObjId
     * @see #lastObjId
     */
    public int getLastObjId () {
    	return this.lastObjId;
    }

    /** retorna o valor da porta utilizado
     * @return	int 	valor de port
     * @see #port
     */
    public int getPort () {
    	return this.port;
    }


    public String toString () {
    	StringBuffer buf = new StringBuffer();
    	buf.append ("[ timelife=");	buf.append (this.timelife);
    	buf.append (", server=");	buf.append (this.server);
    	buf.append (", isserver=");	buf.append (this.isServer);
    	buf.append (", lastobjid=");	buf.append (this.lastObjId);
    	buf.append (", port=");	buf.append (this.port);
    	buf.append ("]");
        return buf.toString();
    }
}