/**
 * UFRGS - II - PPGC - MESTRADO EM CIENCIA DA COMPUTACAO
 * AUTORA MANUELE KIRSCH PINHEIRO
 * CLASSE ST_STRATEGIST
 * PACOTE BW.STORAGE
 * DATA DE CRIACAO 19 / 07 / 2000
 */

package BW.storage;

import BW.kernel.*;
import BW.storage.*;

import java.util.Vector;
import java.util.Hashtable;


/**
 * classe ST_Implementor descreve as implementacoes do
 * armazenamento dos dados em disco no BW.
 * Trata-se de uma abstracao da implementacao real, a
 * qual e descrita pela classe ST_Implementor, seguindo
 * o modelo definido no design pattern Brigde.
 * @see BW.storage.ST_Implementor
 * @autor Manuele K P
 * @version 1.0
 */
public class ST_Strategist {

	///////////////////////////////////////
	////// FALTA MUITA COISA AINDA  //////
	//////////////////////////////////////

	////////////////////////// ARGUMENTOS ////////////

	/** implementacao da base de dados utilizada */
	ST_Implementor impl;
	
	/** vector com outras possiveis implementacoes, em ordem de importancia */
	Vector impl_alter;
	
	/** indice da implementacao da base de dados ativa no momento */
	int impl_index;

	/** controlador da camada de armazenamento */
	ST_Controler control;
	
	/** setup, com informacoes pertinentes do sistema */
	ST_Setup setup;
	
	/** fachada para a camada de conttrole */
	FC_ST_Context facade;

	
	/////////////////////////// CONSTRUTOR ////////////
	
	public ST_Strategist () {
        	this.impl = null;
        	this.control = null;
        	this.setup = null;
        	this.facade = null;
        	this.impl_alter = new Vector();
        	this.impl_index = 0;
	}

	/** finaliza as atividades na base de dados
	
	protected void finalize () {
	       try {
	         if (impl.getStatus())
         	 	impl.close();
               } catch (Exception e) {

               }
	}
	*/
	
	/** termina suas atividades */
	public void terminate () {
		System.out.println ("[DBG]ST_Strategist.terminate: finalizing activities");
		try {
			//this.finalize();
	         	if (impl.getStatus())
         	 		impl.close();
			
		//} catch (Throwable t) {
		} catch (Exception e) {
			System.out.println ("problemas...");
		}	
	}	
	////////////////////////// METODOS EM ABERTO ////////////////
		
 	/** altera a estrategia de implementacao, buscando a proxima
	 * implementacao a ser utilizada
	 */
	public void next() {
         	//////AINDA FALTAM OUTRAS OPCOES
         	//////if (impl==null)
		//////	return (ST_Implementor) new ST_LogWriter();
		/////else return impl;		
		try {
			ST_Implementor newimpl = (ST_Implementor) impl_alter.elementAt(++impl_index);
			if (newimpl!=null)
				this.impl = newimpl;
		} catch (IndexOutOfBoundsException ie) {
			if (this.impl_index<0) {
				this.impl_index=0;
				this.next();
			}	
		}
	}

	
	/** reseta a implementacao utilizada para a primeira da lista
	 * de implementacoes possiveis
	 * @see #alter_impl
	 */
	public void reset () {
		try {
			ST_Implementor newimpl = (ST_Implementor) impl_alter.firstElement();
			if (newimpl!=null)
				this.impl = newimpl;
			this.impl_index = 0;
		} catch (IndexOutOfBoundsException ie) {
			this.impl_index = 0;
		}
	}
	
	
	/** Acrescenta uma implementacao ao fim da fila das implementacoes possiveis
	 * @param ST_Implementor newimpl	novo implementacao possivel
	 */
	public void addImplementor (ST_Implementor newimpl) {
		impl_alter.addElement(newimpl);
	}
	
	
	/** Notifica possiveis alteracoes feitas na base de dados
	 * por outras instancias */
	public void Notify () {
		facade.Notify();
	}

	
	
	/////////////////////////////// METODOS ////////////

	/** Grava um conjunto de elementos na base de dados.
	 * Todos estes elementos devem ser de uma mesma classe,
	 * derivada (subclasse) de BW_Element.
	 * @param	Vector what	dados a serem gravados.
	 * @return	boolean		status da operacao.
	 *				true se ocorreu tudo bem,
	 *				false caso contrario
	 */
	public boolean save (Vector what) {
		return impl.write (what);
	}
		
	/** grava um objeto isolado na base de dados
	 * @return	boolean	status da gravacao
	 */
	public boolean save (BW_Element obj) {
		return impl.write (obj);	
	}
	
	/** Executa um statemant SQL junto a base de dados.
	 * Atencao! Nenhume verificacao quanto a consistencia
	 * da consulta junto a base e realizada.
	 * @param	String sql	consulta SQL
	 */
	public boolean save (String sql) {
		return this.impl.write (sql);	
	}
	
 	/** solicita o armazenamento das informacoes do objeto
	 *  setup na base de dados.
	 */	
	 public boolean saveSetup () {
	 	return this.impl.write (this.setup);
	 }


        /** Executa uma QUERY sql, tipicamente um SELECT,
         *  sobre a base de dados.
         * Nenhuma verificacao quanto a consistencia da
         * consulta e fita. Logo, pode retornar null.
	 * @param 	String sql	sql descrevendo o que deve ser lido/consultado
	 * @return	Vector		elementos (tipicamente BW_Element) lidos
	 */
	public Vector read (String sql) {
		return this.impl.read (sql);
	}	
	
	/** faz a leitura e carga do objeto Setup.
	 * @return	ST_Setup	setup com os dados atualizados
	 */
	public ST_Setup readSetup () {
		ST_Setup ret = this.impl.read(this.setup);
		//se a leitura nao retornar null, carrega o novo setup
		if (ret != null)
			this.setup = ret;
		return ret;
	}
	
	
	/** Le um conjunto de elementos de um determinado tipo
	 * (obrigatoriamente, uma subclasse de BW_Element) relacionados
	 * a um determinado intervalo.
	 * O significado semantico deste intervalo pode variar de acordo
	 * com o tipo de elemento sendo carragado. Por exemplo, ao se fazer
	 * a carga de um membro por este metodo, o intervalo vai significar
	 * o ultimo periodo de login dos participantes.
	 * @param	BW_Element what		tipo de dado a ser lido
	 * @param	BW_TimeInterval when	intervalo relacionado
	 */
	 public Vector read (BW_Element what, BW_TimeInterval when) {
	        return this.impl.read (what,when);
	 }
	
	 	
	 /** Faz a carga de todos os elementos de um determinado tipo (subclasse
	  * de BW_Element).
	  * @param	BW_Element what		tipo de dado a ser lido
	  * @return	Vector	todos os dados daquele tipo lidos
	  */
	 public Vector read (BW_Element what) {
	        //System.out.println ("[DBG] ST_Strategist.read(BW_Element): reading...");
	 	return this.impl.read (what);
	 }
	
	
	 /** Faz a carga de uma unica instancia de um tipo de elemento
	  * @param 	BW_Element what	tipo de dado a ser lido
	  * @param	int objId	identificador do objeto a ser resgatado
	  * @return	BW_Element	objeto lido
	  */
	 public BW_Element read (BW_Element what, int objId) {
	 	return this.impl.read (what,objId);
	 }

	

	 		
	/** re-abre a base de dados
	 * @return	boolean	status da operacao
	 */
	public boolean reOpen () {
	       boolean ret = false;
	       try {
	         if (impl != null && impl.getStatus()) {
         	 	impl.close();
         	 	impl.open();
         	 	ret = true;
         	 } else ret = false;
               } catch (Exception e) {
			ret = false;
               }         	
               return ret;
	}

	
        /** fecha e conclui as operacoes sobre a base de dados
	 * @return 	boolean		true  se foi fechado com exito
	 *				false caso contrario
	 */
	public boolean close () {
	        boolean ret = false;
		if (this.impl != null && this.impl.getStatus())
			ret = this.impl.close();			
		return ret;	
	}	

 	/** retorna o status da base de dados, se esta aberta ou nao
	 *  operacao feita sobre ela
	 * @return	boolean		true - se a base esta ok
	 *				false caso contrario
	 */
	public boolean getStatus () {
		return this.impl.getStatus();
	}
	
	
	////////////////////////// SETS E GETS
		
	/** seta o controle do pacote Storage
	 * @param ST_Controler cl	controlador do pacote Storage
	 * @see ST_Controler
	 */
	public void setControler (ST_Controler cl) {
		this.control = cl;
	}
	
	/** seta a implementacao real a ser utilizada, colocando-a no
	 * inicio da fila de possibilidades, e resetando a contagem de
	 * posicoes na fila.
	 * @see #impl_alter
	 * @see #next
	 * @param ST_Implementor ipl	objeto real para manipulacao da base de dados
	 * @see ST_Implementor
	 */
	public void setImplementor (ST_Implementor ipl) {
	        if ( this.impl != null )
	        	if (this.impl.getStatus())
	        		this.impl.close();
		
		this.impl = ipl;
		if (!this.impl.getStatus())
			this.impl.open();
		
		this.impl_alter.insertElementAt (impl,0);
		this.impl_index = 0;
	}
	
	
	/** seta o objeto com o setup inicial com informacoes sobre o sistema
	 * @param ST_Setup set	objeto setup a ser usado
	 */
	public void setSetup (ST_Setup set) {
		this.setup = set;
	}
	
	/** seta a comunicacao com a camada de controle, para contextualizacao
	 * @param FC_ST_Context fcd	fachada para acesso aos demais pacotes
 	 */
	public void setFCContext (FC_ST_Context fcd) {
		this.facade = fcd;
	}
	
	
	/** retorna o controle do pacote Storage
	 * @return ST_Controler 	controlador do pacote Storage
	 * @see ST_Controler
	 */
	public ST_Controler getControler () {
		return this.control;
	}
	
	/**retorna a implementacao real utilizada no momento
	 * @return ST_Implementor	objeto real para manipulacao da base de dados
	 * @see ST_Implementor
	 */
	public ST_Implementor getImplementor () {
		return this.impl;
	}
	
	/** retorna o objeto com o setup inicial com informacoes sobre o sistema
	 * @return ST_Setup 	objeto setup a ser usado
	 */
	public ST_Setup getSetup () {
		return this.setup;
	}
	
	/** retorna a comunicacao com a camada de controle, para contextualizacao
	 * @return FC_ST_Context 	fachada para acesso aos demais pacotes
 	*/
	public FC_ST_Context getFCContext () {
		return this.facade;
	}
	
}