/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * package: COPSE
 * Classe: CEventQueueConnector
\******************************************************************************/

package COPSE;

import java.io.*;
//import java.net.*;
import java.util.*;


public class CEventQueueConnector
    implements CEventConnector
{
    private int         mServerPort;
    private int         mConnectionId;

    //private CConnectionHandler pUserConnection;
    private CEventConnector pConnector;

    // Event Queue
    private Vector      pQueue;

    public CEventQueueConnector ( int serverPort, int connectionId, CEventConnector connector ) //CConnectionHandler userConnection )
    {
        mServerPort = serverPort;
        mConnectionId = connectionId;

        //pUserConnection = userConnection;
        pConnector = connector;

        pQueue = new Vector();
    }

    public int  GetConnectionId()                   { return mConnectionId; }
    public void SetConnectionId( int connId )       { mConnectionId = connId; }

    public int  GetServerPort()                     { return mServerPort; }
    public void SetServerPort( int port )           { mServerPort = port; }


    public CEvent Receive()
        throws IOException
    {
        synchronized( pQueue )
        {
            while( pQueue.isEmpty() )
            {
                try {
                    pQueue.wait();
                } catch (InterruptedException ie )
                { throw new IOException ( "[DEBUG] CEventQueueConnector - Interrupted Exception" ); }
            }

            Object obj = pQueue.firstElement();
            pQueue.removeElementAt(0);

            if( obj instanceof CEvent )
                return((CEvent) obj);

            if( obj == this )
                throw new IOException ( "[DEBUG] CEventQueueConnector - Connector was closed ..." );

            throw new IOException( "Data error on communication ..." );
        }
    }

    public void Send( CEvent event )
        throws IOException
    {
        System.err.println( "[DEBUG] CEventQueueConnector - Sending event ["+event.GetType()+"|port:"+mServerPort+"|conn:"+mConnectionId+"]" );
        event.SetSocketPort( mServerPort );
        event.SetConnectionId( mConnectionId );
        //pUserConnection.Send( event );
        try {
            pConnector.Send( event );
        } catch( IOException e )
        {
            e.printStackTrace( System.err );
            throw e;
        }
    }

    public void AddEvent( CEvent event )
    {
        synchronized( pQueue )
        {
            if( pQueue.isEmpty() )
                pQueue.notifyAll();

            pQueue.addElement( event );
        }
    }

    public void Close()
    {
        synchronized( pQueue )
        {
            if( pQueue.isEmpty() )
            {
                pQueue.removeAllElements();
                pQueue.addElement( this );
                pQueue.notifyAll();
            }
            else
                pQueue.addElement( this );
        }
    }
}

