/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: COPSE.CLIENT.GUI
 * Classe:  CDlgCommunicationChat
\******************************************************************************/

package COPSE.CLIENT.GUI;


import  java.awt.*;
import  java.awt.event.*;
import  java.util.*;

import  COPSE.*;
import  COPSE.CLIENT.*;


/*************************************************************************
 *  Class CDlgCommunicationChat
 *************************************************************************/
public class    CDlgCommunicationChat
    extends     Dialog
    implements  ActionListener, WindowListener
{
    protected   CGUIManager             pGUI;               // ???
    protected   CGUIChatControl         pControl;
    protected   CClientSessionManager   pSessionManager;


    protected   TextArea        pChatMessagesText;
    protected   TextArea        pMessageText;

    protected   Choice          pTargetUserChoice;
    protected   Button          btnSend;



	public CDlgCommunicationChat(CGUIManager gui, boolean modal, CGUIChatControl control, CClientSessionManager sessionManager)
	{
	    super(gui.GetMainWindow(), modal);
	    pGUI = gui;
	    pControl = control;
	    pSessionManager = sessionManager;

		setLayout(new BorderLayout(1,1));
		setSize( 350, 300 );
		//addNotify();

		setFont(new Font("Helvetica", Font.PLAIN, 12));


        Label label = new Label( "Chat History:" );
        add( "North", label );

        pChatMessagesText = new TextArea(10, 70);
        pChatMessagesText.setEditable( false );
        add( "Center", pChatMessagesText );



        Panel p = new Panel();
        p.setLayout(new BorderLayout());
        p.setSize( 330, 160 );

		label = new Label("Your Message:");
		p.add("North", label);

		pMessageText = new TextArea(3, 80);
		//pMessageText.setBounds( 10,31,315,60);
		p.add("Center", pMessageText);


        Panel p2 = new Panel();
        p2.setLayout( null );
        p2.setSize( 330, 55 );
		label = new Label("Send Message To:");
		label.setBounds(1,1,130,20);
		p2.add(label);

		pTargetUserChoice = new Choice();
		pTargetUserChoice.setBounds( 1, 22, 200, 30 );
		p2.add(pTargetUserChoice);

		UpdateUserList();

        p.add( "South", p2 );

		btnSend = new Button("Send");
		btnSend.setSize( 80,30);
		btnSend.addActionListener( this );

		p.add("East", btnSend );


        add( "South", p );


        addWindowListener( this );
        setTitle( "Chat" );
        setResizable( true );
    }

    public void UpdateUserList()
    {
        CToolSession toolSession = pSessionManager.GetClientToolSession();
        CUserSession clientSession = pSessionManager.GetClientSession();

        pTargetUserChoice.removeAll();

        pTargetUserChoice.add( "ALL" );
        Enumeration e = toolSession.GetUserSessions();
        while( e.hasMoreElements() )
        {
            CUserSession user = (CUserSession) e.nextElement();
            if( user != clientSession )
                pTargetUserChoice.add( user.GetUserName() );
        }
    }



    public synchronized void setVisible( boolean visible )
    {
        if( visible )
        {
        	Rectangle bounds = getParent().getBounds();
        	Rectangle abounds = getBounds();

        	setLocation(bounds.x + (bounds.width - abounds.width)/ 2,
        	     bounds.y + (bounds.height - abounds.height)/2);
        }

    	super.setVisible( visible );
    }

    // WindowEvent Listener methods
    public void windowOpened(WindowEvent e)         {}
    public void windowClosed(WindowEvent e)         {}
    public void windowClosing(WindowEvent e)        { setVisible(false); }
    public void windowIconified(WindowEvent e)      { setVisible(false); }
    public void windowDeiconified(WindowEvent e)    {}
    public void windowActivated(WindowEvent e)      {}
    public void windowDeactivated(WindowEvent e)    {}

    // ActionEvent Listener method
    public void actionPerformed( ActionEvent event )
    {
        Object source = event.getSource();

        if( source == btnSend )
            btnSend_Clicked();
    }

    protected void btnSend_Clicked()
    {
        // System.err.println( "Sending Chat Message..." );

        String message = pMessageText.getText();
        if( message.equals("") )
            return;
        pMessageText.setText("");

        int userId = -1;
        String selectedName = pTargetUserChoice.getSelectedItem();
        if( selectedName.equals( "ALL" ) )
            userId = 0;
        else
        {
            CToolSession tool = pSessionManager.GetClientToolSession();
            CUserSession user = tool.FindUserSession( selectedName );

            if( user != null )
                userId = user.GetUserId();
        }

        if( userId == -1 )
            return; // Show Message Error

        CUserSession clientSession = pSessionManager.GetClientSession();

        try{
            CEvent event = new CEvent( CEvent.EvCollaboration_Communication_Chat_Message );
            event.AddArgument( new Integer( clientSession.GetUserId() ) );
            event.AddArgument( new Integer( userId ) );
            event.AddArgument( message );
            pGUI.Send( event );
        } catch( Exception e ) {}
    }

    public void AddMessage( String message )
    {
        pChatMessagesText.append( message );
    }

}