/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: COPSE.CLIENT
 * Classe:  CGUIChatControl
\******************************************************************************/


package COPSE.CLIENT.GUI;


import  java.awt.*;
import  java.awt.event.*;

import  COPSE.*;
import  COPSE.CLIENT.*;


public class    CGUIChatControl
    extends     CGUIControl
    implements  ItemListener, WindowListener
{
    protected   CGUICollaborationManager    pCollabManager;


    protected   CheckboxMenuItem            mnChat;

    protected   CDlgCommunicationChat       pChatDlg = null;


    public CGUIChatControl ( CGUICollaborationManager collabManager, Menu menu )
    {
        super( collabManager.GetGUIManager() );
        pCollabManager = collabManager;

        mnChat = new CheckboxMenuItem( "Chat" );
        mnChat.addItemListener( this );
        mnChat.setEnabled( IsEnabled() );
        menu.add( mnChat );
    }


    protected void ChangeMenuSettings()
    {
        CClientSessionManager session = pCollabManager.GetSessionManager();
        if( session == null )
        {
            mnChat.setEnabled( false );
            if( pChatDlg != null )
            {
                pChatDlg.setVisible( false );
                pChatDlg = null;
            }

            SetEnabled( false );
        }
        else
        {
            if( IsEnabled() )   return;

            SetEnabled( true );
            mnChat.setEnabled( mIsEnabled );
            pChatDlg = CreateChatDialog();
        }
    }


    protected CDlgCommunicationChat CreateChatDialog()
    {
        CDlgCommunicationChat chat = new CDlgCommunicationChat( pGUI, false, this, pCollabManager.GetSessionManager() );
        chat.addWindowListener( this );
        return chat;
    }

    public void Activate()
    {
        ChangeMenuSettings();
    }

    public void Update()
    {
        if( pChatDlg != null )
            pChatDlg.UpdateUserList();

        ChangeMenuSettings();
    }


    //* ItemListener
    public void itemStateChanged(ItemEvent event)
    {
        if( event.getSource() == mnChat )
        {
            if( pChatDlg == null )
                pChatDlg = CreateChatDialog();

            pChatDlg.setVisible( mnChat.getState() );
        }
    }


    //* WindowListener
    public void windowOpened(WindowEvent e)         {}
    public void windowClosed(WindowEvent e)         {}

    public void windowClosing(WindowEvent e)
        {
            //pChatDlg.setVisible( false );
            mnChat.setState(false);
        }

    public void windowIconified(WindowEvent e)
        {
            //pChatDlg.setVisible( false );
            mnChat.setState(false);
        }

    public void windowDeiconified(WindowEvent e)    {}
    public void windowActivated(WindowEvent e)      {}
    public void windowDeactivated(WindowEvent e)    {}




    //* Event Handler
    protected   void    ProcessEvent( CEvent event )
    {
        int type = event.GetType();

        switch( type )
        {
            case CEvent.EvCollaboration_Communication_Chat_Message:
                {
                    Integer fromId = (Integer) event.GetArgumentAt(0);
                    Integer toId   = (Integer) event.GetArgumentAt(1);
                    String message = (String) event.GetArgumentAt(2);

                    AddMessage( fromId.intValue(), toId.intValue(), message );
                }
                break;

            default:
                super.ProcessEvent(event);
        }
    }






    //
    protected void AddMessage( int from, int to, String message )
    {
        if( pChatDlg == null )  return;

        CClientSessionManager sessionManager = pCollabManager.GetSessionManager();
        CToolSession tool = sessionManager.GetClientToolSession();
        CUserSession user = tool.FindUserSession( from );

        String fromName = "id=" + Integer.toString(from);

        if( user != null )
            fromName = user.GetUserName();


        // Add message to Chat Window...
        String text;
        if( to != 0 )
             text = "#" + fromName + "#: " + message + "\n";
        else text = "<" + fromName + ">: " + message + "\n";

        pChatDlg.AddMessage( text );
    }

}


