/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: COPSE.CLIENT
 * Classe:  CGUIFileControl
\******************************************************************************/

package COPSE.CLIENT.GUI;

import  java.awt.*;
import  java.awt.event.*;

import  COPSE.*;
import  COPSE.CLIENT.*;

public class    CGUIFileControl
    extends     CGUIControl
    implements  ActionListener
{
    protected   CGUIToolManager pTool;

    protected   Menu            pFileMenu;
    protected   MenuItem        mnFileOpen;
    protected   MenuItem        mnFileSave;
    protected   MenuItem        mnFileClose;
    protected   MenuItem        mnFileExit;

    protected   CDlgFileOpen    pFileOpenDlg = null;


    public CGUIFileControl ( CGUIToolManager tool)
    {
        super( tool.GetGUIManager() );
        mIsEnabled = true;
        pTool = tool;
    }


    public void AddMenu ( MenuBar menuBar )
    {
        pFileMenu   = new Menu( "File" );

        mnFileOpen  = new MenuItem( "Open..." );
        mnFileSave  = new MenuItem( "Save" );
        mnFileClose = new MenuItem( "Close" );
        mnFileExit  = new MenuItem( "Exit" );

        mnFileOpen.addActionListener( this );
        mnFileSave.addActionListener( this );
        mnFileClose.addActionListener( this );
        mnFileExit.addActionListener( this );

        pFileMenu.add( mnFileOpen );
        pFileMenu.add( mnFileSave );
        pFileMenu.add( mnFileClose );
        pFileMenu.addSeparator();
        pFileMenu.add( mnFileExit );

        menuBar.add( pFileMenu );

        ChangeMenuSettings();
    }


    protected void ChangeMenuSettings()
    {
        boolean isClientOpen = pTool.IsClientOpen();

        mnFileOpen.setEnabled( !isClientOpen );
        mnFileSave.setEnabled( isClientOpen );
        mnFileClose.setEnabled( isClientOpen );
    }

    // Override SetEnabled not to start (Thread)...
    public void SetEnabled( boolean enable )
    {
        mIsEnabled = enable;
    }


    // ActionEvent Listener method
    public void actionPerformed( ActionEvent event )
    {
        Object source = event.getSource();

        if( source == mnFileOpen )      { _FileOpen(); }
        else
        if( source == mnFileSave )      { _FileSave(); }
        else
        if( source == mnFileClose )     { _FileClose( false ); }
        else
        if( source == mnFileExit )      { _FileClose( true ); }

        /*
        else
            super.actionPerformed( event );
        */
    }






    protected void _FileOpen()
    {
        if( pFileOpenDlg == null )
            pFileOpenDlg = new CDlgFileOpen( pGUI.GetMainWindow(), false, this );

        pFileOpenDlg.setVisible(true);
    }

    public void DlgFileOpenOk()
    {
        // Open Client ...
        System.err.println( "Open Client..." );

        try {
            pTool.OpenClient( pFileOpenDlg.mHost,
                              Integer.parseInt(pFileOpenDlg.mPort),
                              pFileOpenDlg.mUser );
            ChangeMenuSettings();
        } catch(Exception e)
        {   pTool.ProcessException( e );  }
    }



    protected void _FileSave()
    {
        pTool.Save();
    }



    public void _FileClose( boolean exit )
    {
        // before, open a confirmation dialog...
        /*
        if( pFileCloseDlg == null )
            pFileCloseDlg = new CDlgFileClose( pGUI.GetMainWindow(), false, this );

        pFileCloseDlg.SetExit( exit );
        pFileCloseDlg.setVisible( true );
        */
        DlgFileCloseOk( exit );
    }


    public void DlgFileCloseOk( boolean exit )
    {
        pTool.CloseClient( exit );
    }

}


