/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * package: COPSE.SERVER
 * Classe: CServer
\******************************************************************************/

package COPSE.SERVER;

import java.io.*;
import java.net.*;
import java.util.*;


import COPSE.*;
import COPSE.CONTROL.*;
import FILE.*;


/*

class CServerApp
{
    CServer pServer;

    public static void main(String[] args)
    {
        try {
            pServer = new CServer(args);
            pServer.start();
            pServer.join();
        } catch (Exception e)
        {
            e.printStackTrace(System.err);
            System.exit(1);
        }
    }
}

*/


public class CServer
    extends Thread implements Runnable 
{
    // Socket to accept new user connections
    private       ServerSocket            pServerSocket;
    protected     String                  mServerSocketHost;
    protected     int                     mServerSocketPort;


    // Manager of the Session (joining and departing policies)
    //BUGFIX: private CServerSessionManager     pSessionManager;
    protected CServerSessionManager     pSessionManager;


    // Manager of the Collaboration
    protected CServerCollaborationManager pCollabManager;


    // Process and send the events to the Server Components
    protected CServerEventHandler       pEventHandler;


    /** The "Document" controled by this server */
    protected CFile                     pFile;
    protected CFileController           pFileController; // (Floor) Lock Control  ???


    // Exit command was requested ?
    protected boolean                     mExit;






    public CServer( int port, String fileName )
        throws Exception
    {
        System.err.println( "[DEBUG] CServer <init>: port = ["+port+"], filename = ["+fileName+"]" );


        System.err.println( "[DEBUG] CServer: Session manager ..." );
        pSessionManager = new CServerSessionManager( this );

        System.err.println( "[DEBUG] CServer: Event handler ..." );
        InitEventHandler();

        System.err.println( "[DEBUG] CServer: File manager ..." );
        InitFileManagement( fileName );

        System.err.println( "[DEBUG] CServer: Collaboration manager ..." );
        InitCollaborationManager();

        // Creating Socket to accept User Connections
        CreateServerSocket( port );

        mExit = false;
    }

    protected   void    CreateServerSocket( int port )
        throws Exception
    {
        System.err.println( "[DEBUG] CServer: Server Socket ..." );
        mServerSocketPort = port;
        pServerSocket = new ServerSocket( port );

        // Temporary
        //mServerSocketHost = "localhost";
        mServerSocketHost = pServerSocket.getInetAddress().getHostName();
        System.err.println ( " Host: "+mServerSocketHost);
    }



    public int                      GetServerSocketPort()   { return mServerSocketPort; }
    public String                   GetServerSocketHost()   { return mServerSocketHost; }


    public CServerCollaborationManager  GetCollaborationManager() { return pCollabManager; }

    public CServerSessionManager    GetSessionManager()     { return pSessionManager; }

    public CFileController          GetFileController()     { return pFileController; }

    public CFile                    GetFile()               { return pFile; }



    protected void  InitCollaborationManager()
    {
        pCollabManager = new CServerCollaborationManager( this );
        pCollabManager.start();
    }


    /**  Init the file management.
     *   By default, it looks for the filename in args[1].
     *   If there isn't any argument, it creates a new (empty) file that will be shared by all clients. [A default FileController
     *   is also created. It's based on element locking policy.]<br>
     *   This method should be override to load the correct file and to create the appropriate FileController.
     */
    protected void InitFileManagement( String fileName )
        throws IOException
    {
        if( fileName == null )
        {
            // Create an empty file
            pFile = new CFile( "untitled", "." );
        }
        else
            try {
                // Load the requested file
                pFile = CFile.Load( fileName, "." );
            } catch (ClassNotFoundException e)
            {
                throw new IOException( new String("FileClass not found: " + e.toString()) );
            };


        // Create the default FileController
        pFileController = CreateFileController();
    }



    public void SetFile( CFile file )
    {
        pFile = file;
        pFileController = CreateFileController();
    }



    /** Init the Event Handler
     */
    protected void InitEventHandler()
        throws Exception
    {
        // Create the default event dispatcher
        System.err.println( "[DEBUG] CServer - CreatingServerEventHandler ..." );
        pEventHandler = CreateServerEventHandler();
        pEventHandler.start();
    }

    protected CServerEventHandler CreateServerEventHandler()
    {
        return new CServerEventHandler( this );
    }


    protected CFileController CreateFileController()
    {
        // Create the default FileController
        return new CFileController( pFile );
    }


    public void run()
    {
        System.out.println( "[DEBUG] CServer: running... (accepting new connections)" );
        // Accepting new connections from User Clients
        Socket slaveSocket = null;

        try {
            pServerSocket.setSoTimeout( 1 );
        } catch( SocketException e ){System.out.println("aqui deu pau");}

        mExit = false;
        while( !mExit )
        {
            slaveSocket = null;
            // Wait for next "open user connection" request
            try {
                slaveSocket = pServerSocket.accept();
            }
            catch( java.io.IOException e2 ) {}
            catch( Exception ex ) {}
            finally
            {
                if( slaveSocket != null )
                {
                    // Try to open connection after verifying the user login
                    System.err.println( "[DEBUG]: CServer - Connecting User ..." );
                    _ConnectUser( slaveSocket );
                }
            }
        }
    }




    protected CUserConnectionHandler CreateUserConnectionHandler( int userId, int toolId, String name,
        CSocketStreamConnector connector )
    {
        int connId = pSessionManager.CreateNewConnectionId();

        CServerUserSession userSession = new CServerUserSession( userId, connId );
        userSession.SetUserName( name );

        CUserConnectionHandler pConnectionHandler = new CUserConnectionHandler( connector, pEventHandler, userSession );

        CServerToolSession toolSession = (CServerToolSession) pSessionManager.FindToolSession( toolId );
        if( toolSession == null )
        {
            toolSession = new CServerToolSession( toolId );
            pSessionManager.AddToolSession( toolSession );
        }

        toolSession.AddUserSession( userSession );


        pConnectionHandler.start();
        return pConnectionHandler;
    }


    private void _ConnectUser( Socket socket )
    {
        CSocketStreamConnector pSocketConnector;

        boolean     ok = false;

        Integer     userId = new Integer(0);
        Integer     toolId = new Integer(0);

        try {
            System.err.println( "[DEBUG] CServer - Creating SocketConnector..." );
            pSocketConnector = CSocketStreamConnector.New( socket );

            System.err.println( "[DEBUG] CServer - Loading Connection Command ..." );
            CEvent event = pSocketConnector.Receive();

            if( event.GetType() == CEvent.EvCOPSE_Connection_Open )
            {
                // Get User Identification
                userId = (Integer) event.GetArgumentAt(0);
                toolId = (Integer) event.GetArgumentAt(1);
                String name = (String) event.GetArgumentAt(2);

                // Create the Conection Handler:
                // This includes the user to the SessionManager
                CUserConnectionHandler pConnection = CreateUserConnectionHandler( userId.intValue(), toolId.intValue(), name, pSocketConnector );


                event = new CEvent( CEvent.EvCOPSE_Connection_Open );
                event.AddArgument( new Integer(pConnection.GetConnectionId()) );
                event.AddArgument( userId );
                event.AddArgument( toolId );
                event.AddArgument( name );

                // Notify all users about this new connection
                pSessionManager.NotifyAll( event );

                // that's all
                return;
            }
            else
            {
                event = new CEvent( CEvent.EvCOPSE_Connection_Error );
                event.AddArgument( new String("Invalid command. Open Connection command expected!") );
                pSocketConnector.Send( event );
            }

        } catch ( Exception e )
        {}
    }

/*
    private int _CreateNewConnectionId()
    {
        mLastConnId++;
        return mLastConnId;
    }
*/
/*
    private void _CloseAllConnections()
    {
        CEvent event = new CEvent( CEvent.EvUserConnection_Exit );

        Enumeration e = pUserConnections.elements();
        while( e.hasMoreElements() )
        {
            CUserServer userServer = (CUserServer) e.nextElement();
            userServer.Send( event );
            userServer.Exit();
        }

        e = pUserConnections.elements();
        while( e.hasMoreElements() )
        {
            CUserServer userServer = (CUserServer) e.nextElement();
            try {
                userServer.join( 100 );
            } catch( InterruptedException ie ){};
            if( userServer.isAlive() )
                userServer.stop();
        }

        pUserConnections.removeAllElements();
    }
*/

/*

    public final synchronized void RemoveConnection( CUserServer server )
    {
        if( Thread.currentThread() == server )
        {
            pUserConnections.removeElement(server);

            CEvent event = new CEvent( CEvent.EvUserConnection_Close );
            event.AddArgument( new Integer(server.GetConnId()) );
            NotifyAll( event );
        }
        else
            System.err.println( "ERROR: Server - Remove User Connection ...");
    }


    public final synchronized void Notify ( int connId, CEvent event )
    {
        Enumeration e = pUserConnections.elements();
        while( e.hasMoreElements() )
        {
            CUserServer userServer = (CUserServer) e.nextElement();
            if( userServer.GetConnId() == connId )
            {
                userServer.Send( event );
                return;
            }
        }
    }

    public final synchronized void NotifyAll ( CEvent event )
    {
        System.err.println( "DEBUG: Server - Notifying Users About Event '"+event.GetType()+"'");
        System.err.println( "DEBUG: Server - There is(are) "+ pUserConnections.size() +" CUserServer(s)");

        Enumeration e = pUserConnections.elements();
        while( e.hasMoreElements() )
        {
            CUserServer userServer = (CUserServer) e.nextElement();
            userServer.Send( event );
        }
    }

    */
/*
    private void _Exit()
    {
        mExit = true;

        synchronized( this )
        {
            System.err.println( "DEBUG: closing all connections...");
            _CloseAllConnections();

            System.err.println( "DEBUG: closing ServerSocket...");
            try {
                pServerSocket.close();
            } catch( IOException e ) {};
        }

    }
*/

    public void Exit()
    {

        CloseFile();

        pCollabManager.Exit();

        // Send exit event to all clients...
        try {
            CEvent event = new CEvent( CEvent.EvCOPSE_Server_Exit );
            pSessionManager.NotifyAll( event );
        } catch ( Exception e ){}

        // Stop all client connection handlers
        pSessionManager.CloseAllUserConnectionHandlers();

        // Stop the CServer thread...
        this.stop();

        // Close the server socket
        try {
            pServerSocket.close();
        } catch( Exception ioe ) {}

        // Close the server Event Handler
        pEventHandler.stop();
    }

    protected void CloseFile()
    {
        try{
            pFile.Save();
        } catch (IOException e) {}
    }


}

