/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: FILE
 * Classe:  CFileSystemMgr
\******************************************************************************/

package FILE;

import java.io.*;
import java.util.*;


public class CFileSystemMgr
{
    //** Attributes
    private   static String        sSystemFile = "filesystem.index";
    private   static String        sSystemPath = ".";

    private   CFile         pSystemFile;
    private   CFolderEntry  pRootFolder;



    //** Class Services
    public static CFileSystemMgr CreateFileSystem (String systemFilePath)
    {
        if( (systemFilePath != null) && (systemFilePath != "") )
            sSystemPath = systemFilePath;
        return new CFileSystemMgr();
    }

    public static CFileSystemMgr OpenFileSystem (String systemFilePath)
        throws IOException
    {
        if( (systemFilePath != null) && (systemFilePath != "") )
            sSystemPath = systemFilePath;

        try
        {
            CFile file = CFile.Load(sSystemFile, sSystemPath);
            CFileSystemMgr system = new CFileSystemMgr( file );

            return system;
        } catch ( ClassNotFoundException ex )
        {
            throw new IOException( "System File is corrupted!" );
        }

    }


    //** Private Constructors
    private CFileSystemMgr()
    {
        pSystemFile = new CFile( sSystemFile, sSystemPath );
        pRootFolder = new CFolderEntry();
        pRootFolder.SetName( sSystemPath );

        pSystemFile.AddElement( pRootFolder );

        try {
            pSystemFile.Save();
        } catch( IOException e ) {}
    }

    private CFileSystemMgr( CFile file )
        throws ClassNotFoundException
    {
        pSystemFile = file;
        pRootFolder = null;

        Enumeration en = file.GetElements();
        while( en.hasMoreElements() )
        {
            CElement elem = (CElement) en.nextElement();

            if( elem instanceof CFolderEntry )
            {
                CFolderEntry folder = (CFolderEntry) elem;
                if( folder.GetParentFolder() == null )
                {
                    pRootFolder = folder;
                    pRootFolder.SetName( sSystemPath );
                    break;
                }
            }
        }

        if( pRootFolder == null )
            throw new ClassNotFoundException();
    }



    //** Methods
    public CFolderEntry     GetRootFolder()         { return pRootFolder; }


    public CFolderEntry     CreateFolderEntry   (String name, CFolderEntry parentFolder)
        throws IOException
    {
        File file;
        if( parentFolder == null )
            file = new File( pRootFolder.GetPath(), name );
        else
            file = new File( parentFolder.GetPath(), name );

        if( !file.exists() )
            file.mkdirs();
        else
        if( !file.isDirectory() )
            throw new IOException( "This path exists and is not a directory!" );

        CFolderEntry folder = new CFolderEntry();
        folder.SetName( name );
        if( parentFolder == null )
        {
            pRootFolder.AddEntry( folder );
            folder.SetParentFolder( pRootFolder );
        }
        else
        {
            parentFolder.AddEntry( folder );
            folder.SetParentFolder( parentFolder );
        }

        pSystemFile.AddElement( folder );
        pSystemFile.Save();

        return folder;
    }





    public CDocEntry        GetDocEntry         (int docId)
    {
        CElement elem = pSystemFile.GetElement( docId );
        if(elem instanceof CDocEntry)
            return (CDocEntry)elem;
        else
            return null;
    }

    public CDocEntry        CreateDocEntry     (String name, CFolderEntry parentFolder)
        throws IOException
    {
        String path;
        if( parentFolder == null )
            path = pRootFolder.GetPath();
        else
            path = parentFolder.GetPath();

        File file = new File(path, name);

        if( !file.exists() )
        {
            System.out.println( "Creating Document..." );
            CDocument doc = new CDocument( name, path );
            doc.Save();
        }
        else
        if( file.isFile() )
            throw new IOException( "This path exists and is not a file!" );


        CDocEntry doc = new CDocEntry();
        doc.SetName( name );
        if( parentFolder == null )
        {
            pRootFolder.AddEntry( doc );
            doc.SetParentFolder( pRootFolder );
        }
        else
        {
            parentFolder.AddEntry( doc );
            doc.SetParentFolder( parentFolder );
        }

        pSystemFile.AddElement( doc );
        pSystemFile.Save();

        return doc;
    }


    public CDocument GetDocument( CDocEntry docEntry )
        throws IOException
    {
        String path = docEntry.GetParentPath();
        try {
            CDocument doc = (CDocument) CFile.Load( docEntry.GetName(), path );
            return doc;
        }
        catch (ClassNotFoundException e1)
        {
            throw new IOException( "Document is corrupted" );
        }
        catch (ClassCastException e2 )
        {
            throw new IOException( "Document is corrupted" );
        }
    }



    //** Methods to handle Attributes
//    public void     SetSystemFile( String name )       { mSystemFile = name; }
//    public String   GetSystemFile()                    { return mSystemFile; }

//    public void     SetSystemPath( String path )       { mSystemPath = path; }

    public static String   GetSystemPath()                    { return sSystemPath; }




    //** DEBUG
    public void DEBUG_write( PrintStream pStr )
    {
        pStr.println( "DEBUG: CFileSystemMgr");
        pStr.println( "       SystemFile:      " + sSystemFile );
        pStr.println( "       SystemPath:      " + sSystemPath );
        pStr.println( );

        pRootFolder.DEBUG_write(pStr);

        pStr.println( "\nFile:");

        pSystemFile.DEBUG_write(pStr);
    }

}
