/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: FILE.OBJ
 * Classe:  CObjClass
\******************************************************************************/

package FILE.OBJ;


import  java.awt.Rectangle;
import  java.util.Vector;
import  java.util.Enumeration;
import  java.io.PrintStream;

import  FILE.*;


/*************************************************************************
 *  Class CObjClass
 *************************************************************************/
public class    CObjClass
    extends     CElement
{
    // Aggregation
    protected   Vector      pAttributes;
    protected   Vector      pMethods;

    // Association
    protected   Vector      pRelations;

    // attributes
    protected   String      mName;
    protected   boolean     mAbstract;
    protected   String      mStereotype;
    protected   String      mParameters;

    protected   Rectangle   pRect;


    // Constructor
    public CObjClass ()
    {
        super();

        pAttributes = new Vector();
        pMethods = new Vector();

        // Association
        pRelations = new Vector();

        // attributes
        mName = "New Class";
        mAbstract = false;
        mStereotype = "";
        mParameters = "";

        pRect = new Rectangle( 100, 80 );
    }

    public CObjClass (String name)
    {
        this();
        mName = name;
    }


    // Attributes
    public String   GetName ()                  { return mName; }
    public void     SetName (String name)       { mName = name; }

    public boolean  IsAbstract ()               { return mAbstract; }
    public void     SetAbstract ()              { mAbstract = true; }
    public void     SetAbstract (boolean abs)   { mAbstract = abs; }

    public String   GetStereotype ()            { return mStereotype; }
    public void     SetStereotype (String ste)  { mStereotype = ste; }

    public String   GetParameters ()            { return mParameters; }
    public void     SetParameters (String pars) { mParameters = pars; }

    public Rectangle GetRegion ()               { return pRect; }
    public void     SetRegion (Rectangle rect)  { pRect = rect; }



    // General
    private CElement _FindElement( Enumeration e, int id )
    {
        while( e.hasMoreElements() )
        {
            CElement elem = (CElement) e.nextElement();
            if( elem.GetId() == id )
                return elem;
        }
        return null;
    }

    private CObjFeature _FindElement( Enumeration e, String name )
    {
        while( e.hasMoreElements() )
        {
            CObjFeature elem = (CObjFeature) e.nextElement();
            if( name.equals( elem.GetName() ) )
                return elem;
        }
        return null;
    }


    // Aggregation: Attributes
    public CObjAttribute FindAttribute (int id)
    {   return (CObjAttribute) _FindElement( GetAttributes(), id ); }

    public CObjAttribute FindAttribute (String name)
    {   return (CObjAttribute) _FindElement( GetAttributes(), name ); }

    public Enumeration  GetAttributes ()
    {   return pAttributes.elements();  }


    public boolean AddAttribute ( CObjAttribute attrib )
    {
        if( FindAttribute(attrib.GetName()) == null )
        {
            pAttributes.addElement( attrib );
            return true;
        }
        return false;
    }

    public void RemoveAttribute ( CObjAttribute attrib )
    {
        pAttributes.removeElement( attrib );
    }



    // Aggregation: Methods
    public CObjMethod FindMethod (int id)
    {   return (CObjMethod) _FindElement( GetMethods(), id ); }

    public CObjMethod FindMethod (String name)
    {   return (CObjMethod) _FindElement( GetMethods(), name ); }

    public Enumeration  GetMethods ()
    {   return pMethods.elements();  }


    public boolean AddMethod ( CObjMethod method )
    {
        if( FindMethod(method.GetName()) == null )
        {
            pMethods.addElement( method );
            return true;
        }
        return false;
    }

    public void RemoveMethod ( CObjMethod method )
    {
        pMethods.removeElement( method );
    }


    // Aggregation: Relations
    protected CObjRelation FindRelation (int id)
    {   return (CObjRelation) _FindElement( GetRelations(), id ); }

    public Enumeration  GetRelations ()
    {   return pRelations.elements();  }

    public void AddRelation ( CObjRelation relation )
    {
        if( FindRelation(relation.GetId()) == null )
            pRelations.addElement( relation );
    }

    public void RemoveRelation ( CObjRelation relation )
    {
        pRelations.removeElement( relation );
    }




    public CElement Clone()
    {
        CObjClass aclass = new CObjClass();
        aclass.CopyFrom( this );
        return aclass;
    }

    public CObjClass CloneWithoutRelations()
    {
        CObjClass aclass = new CObjClass();
        aclass.CopyFromWithoutRelations( this );
        return aclass;
    }

    public void CopyInfoFrom( CObjClass aclass )
    {
        mName = aclass.GetName();
        mAbstract = aclass.IsAbstract();
        mStereotype = aclass.GetStereotype();
        mParameters = aclass.GetParameters();
    }

    public void CopyFrom( CElement elem )
    {
        if( !(elem instanceof CObjClass) )
            return;

        super.CopyFrom( elem );

        CObjClass aclass = (CObjClass) elem;
        CopyInfoFrom( aclass );

        pAttributes.removeAllElements();
        Enumeration e = aclass.GetAttributes();
        while( e.hasMoreElements() )
            AddAttribute( (CObjAttribute) ((CObjAttribute)e.nextElement()).Clone() );

        pMethods.removeAllElements();
        e = aclass.GetMethods();
        while( e.hasMoreElements() )
            AddMethod( (CObjMethod) ((CObjMethod)e.nextElement()).Clone() );

        pRelations.removeAllElements();
        e = aclass.GetRelations();
        while( e.hasMoreElements() )
            AddRelation( (CObjRelation) e.nextElement() );
    }



    public void CopyFromWithoutRelations( CObjClass aclass )
    {
        super.CopyFrom( aclass );

        CopyInfoFrom( aclass );

        pAttributes.removeAllElements();
        Enumeration e = aclass.GetAttributes();
        while( e.hasMoreElements() )
            AddAttribute( (CObjAttribute) ((CObjAttribute)e.nextElement()).Clone() );

        pMethods.removeAllElements();
        e = aclass.GetMethods();
        while( e.hasMoreElements() )
            AddMethod( (CObjMethod) ((CObjMethod)e.nextElement()).Clone() );
    }







    public void DEBUG_write( PrintStream pStr )
    {
        pStr.print( "Class "+mName+" (id:"+GetId()+") " );
        if( mAbstract )
            pStr.print( "Abstract" );
        pStr.println("");

        pStr.println( "    Region: ["+pRect.x+","+pRect.y+","+pRect.width+","+pRect.height+"]" );

        if( !mStereotype.equals("") )
            pStr.println( "   Stereotype: " + mStereotype );

        if( !mParameters.equals("") )
            pStr.println( "   Parameters: " + mParameters );

        Enumeration e;

        pStr.println( "   Attributes:" );
        e = GetAttributes();
        while( e.hasMoreElements() )
        {
            CObjAttribute attrib = (CObjAttribute) e.nextElement();
            pStr.print( "    - " );
            attrib.DEBUG_write( pStr );
        }

        pStr.println( "   Methods:" );
        e = GetMethods();
        while( e.hasMoreElements() )
        {
            CObjMethod method = (CObjMethod) e.nextElement();
            pStr.print( "    - " );
            method.DEBUG_write( pStr );
        }

    }

}
