/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: FILE.OBJ
 * Classe:  CObjRelation
\******************************************************************************/

package FILE.OBJ;


import  java.util.*;
import  java.awt.Point;
import  java.io.PrintStream;

import  FILE.*;

/*************************************************************************
 *  Class CObjRelation
 *************************************************************************/
public class    CObjRelation
    extends     CElement
{
    // Relation(ship)  types:
    private static final int RelBegin                 = 0;
    public  static final int Association              = 0;
    public  static final int Inheritance              = 1;
    public  static final int AggregationByRef         = 2;
    public  static final int AggregationByValue       = 3;
    public  static final int Instanciation            = 4;
    private static final int RelEnd                   = 4;


    // attributes
    protected   String      mName;
    protected   int         mType;

    protected   Vector      pExtremities;
    //protected   CRelationExtremity   pExtr1, pExtr2;

    // Internal points of the relationship "line"
    protected   Vector      pIntPoints;


    // Constructor
    public CObjRelation ()
    {
        super();

        // attributes
        mName = "";
        mType = Association;

        pExtremities = new Vector();
        pIntPoints = new Vector();
    }


    public CObjRelation (int type)
    {
        this();
        mType = type;
    }


    // Attributes
    public String   GetName ()                  { return mName; }
    public void     SetName (String name)       { mName = name; }

    public int      GetType ()                  { return mType; }
    public void     SetType (int type)          { mType = type; }



    public Enumeration  GetExtremities()
    {   return pExtremities.elements(); }

    public CObjRelationExtremity GetExtremity (int i)
    {   return (CObjRelationExtremity) pExtremities.elementAt(i); }

    public void AddExtremity( CObjRelationExtremity extr )
    {   pExtremities.addElement( extr ); }

    public void RemoveExtremity( CObjRelationExtremity extr )
    {
        pExtremities.removeElement( extr );
        extr.SetClass( null );
    }



    public Enumeration GetInternalPoints ()
    {   return pIntPoints.elements();  }

    public void     AddInternalPoint (Point pt)
    {   pIntPoints.addElement( pt );  }

    public void     RemoveInternalPoint (Point pt)
    {   pIntPoints.removeElement( pt );  }

    public void     SetInternalPoints(Enumeration e)
    {
        int i = -1;

        while( e.hasMoreElements() )
        {
            Point pOld;
            Point pNew = (Point) e.nextElement();

            i++;
            try{
                pOld = (Point) pIntPoints.elementAt(i);
            } catch (Exception exc)
            {
                pIntPoints.addElement( pNew );
                continue;
            }

            pOld.x = pNew.x;
            pOld.y = pNew.y;
        }

        i++;
        while( i < pIntPoints.size() )
        {
            pIntPoints.removeElement( pIntPoints.lastElement() );
        }


        /*
        pIntPoints.removeAllElements();

        while( e.hasMoreElements() )
        {
            Point p = (Point) e.nextElement();
            pIntPoints.addElement( p );
        }
        */
    }

    public Vector   GetPoints()
    {
        CObjRelationExtremity extr;

        Vector points = new Vector();

        // Get First Point
        extr = (CObjRelationExtremity) pExtremities.firstElement();
        points.addElement( extr.GetPoint() );

        // Get Internal Points...
        Enumeration e = GetInternalPoints();
        while( e.hasMoreElements() )
            points.addElement( e.nextElement() );

        // GetLast Point
        extr = (CObjRelationExtremity) pExtremities.lastElement();
        points.addElement( extr.GetPoint() );

        return points;
    }



    public CElement Clone()
    {
        CObjRelation rel = new CObjRelation();
        rel.CopyFrom( this );
        return rel;
    }

    public void CopyFrom( CElement elem )
    {
        super.CopyFrom( elem );

        CObjRelation rel = (CObjRelation) elem;
        SetName( rel.GetName() );
        SetType( rel.GetType() );

        pExtremities.removeAllElements();
        Enumeration e = rel.GetExtremities();
        while( e.hasMoreElements() )
        {
            CObjRelationExtremity extr = (CObjRelationExtremity) e.nextElement();
            CObjRelationExtremity newExtr = new CObjRelationExtremity( this );
            newExtr.CopyFrom( extr );
        }
    }


    public void UpdateFrom( CElement elem )
    {
        //super.CopyFrom( rel );
        CObjRelation rel = (CObjRelation) elem;
        SetName( rel.GetName() );
        SetType( rel.GetType() );

        Enumeration e1 = pExtremities.elements();
        Enumeration e2 = rel.GetExtremities();
        while( e2.hasMoreElements() )
        {
            CObjRelationExtremity extr1 = (CObjRelationExtremity) e1.nextElement();
            CObjRelationExtremity extr2 = (CObjRelationExtremity) e2.nextElement();
            extr1.CopyInfoFrom( extr2 );
        }
    }

    public void DEBUG_write( PrintStream pStr )
    {
        pStr.println( "Relation   "+mName+" (id:"+GetId()+") - Type: " + mType );
        Enumeration e = pExtremities.elements();
        while( e.hasMoreElements() )
        {
            CObjRelationExtremity extr = (CObjRelationExtremity) e.nextElement();
            pStr.print( "   " );
            extr.DEBUG_write( pStr );
        }
    }


}
