/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: GRAPH
 * Classe:  CGraphics
\******************************************************************************/

package GRAPH;

import java.awt.*;
import java.awt.image.*;
import java.util.*;


public class CGraphics
{
    protected Graphics      pG;

    protected CScale        pScale;

    protected boolean       mDashed;

    protected Font          pRealFont;
    protected Font          pScaledFont;

    private static int      dash    = 5;
    private static double   Pi      = 3.14;




    public CGraphics    ( Graphics graf )
    {
        this( graf, null );
    }

    public CGraphics    ( Graphics graf, CScale scale )
    {
        pG = graf;
        pScale = scale;
        mDashed = false;

        pRealFont = pG.getFont();

        if( pScale != null )
        {
            pScaledFont = new Font( pRealFont.getName(), pRealFont.getStyle(), RTS_X( pRealFont.getSize() ) );
            pG.setFont( pScaledFont );
        }
        else
            pScaledFont = null;
    }


    public Graphics     GetGraphics ()  {  return pG;  }



    public void SetScale ( CScale scale )
    {
        pScale = scale;
    }

    public CScale   GetScale ()
    {
        return pScale;
    }

    public void     SetDashedPen()
    {   mDashed = true; }

    public void     SetSolidPen()
    {   mDashed = false; }

    public boolean  IsDashedPen()
    {   return mDashed; }




    // RTS: Real To Scaled view
    private   int     RTS_X( int x )
    {
        if( pScale == null )    return x;
        return (int)(pScale.xScale * x);
    }

    private   int     RTS_Y( int y )
    {
        if( pScale == null )    return y;
        return (int)(pScale.yScale * y);
    }

    private   Point         RTS_Point( Point p )
    {   return new Point( RTS_X(p.x), RTS_Y(p.y) ); }

    public    Rectangle     RTS_Rect( Rectangle r )
    {   return new Rectangle( RTS_X(r.x), RTS_Y(r.y), RTS_X(r.width), RTS_Y(r.height) ); }


    private   CPolyline     RTS_Poly( CPolyline p1 )
    {
        CPolyline p2 = new CPolyline();

        Enumeration e = p1.GetPoints();
        while( e.hasMoreElements() )
            p2.AddPoint( RTS_Point( (Point) e.nextElement() ) );

        return p2;
    }



    // STR: Scaled view to Real
    private   int     STR_X( int x )
    {
        if( pScale == null )    return x;
        return (int)(x / pScale.xScale );
    }

    private   int     STR_Y( int y )
    {
        if( pScale == null )    return y;
        return (int)(y / pScale.yScale);
    }

    private   Point         STR_Point( Point p )
    {   return new Point( STR_X(p.x), STR_Y(p.y) ); }

    private   Rectangle     STR_Rect( Rectangle r )
    {   return new Rectangle( STR_X(r.x), STR_Y(r.y), STR_X(r.width), STR_Y(r.height) ); }


    private   CPolyline     STR_Poly( CPolyline p1 )
    {
        CPolyline p2 = new CPolyline();

        Enumeration e = p1.GetPoints();
        while( e.hasMoreElements() )
            p2.AddPoint( STR_Point( (Point) e.nextElement() ) );

        return p2;
    }



    // It assumes the values are already scaled
    private void  _drawDashedLine( int sx1, int sy1, int sx2, int sy2 )
    {
        Point p1 = new Point( sx1, sy1 );
        Point p2 = new Point( sx2, sy2 );

        if( p1.x > p2.x )
        {
            Point p = p2;
            p2 = p1;
            p1 = p;
        }

        float dx, dy, d;
        dx = p2.x - p1.x;
        dy = p2.y - p1.y;
        d = (float)Math.sqrt( dx*dx + dy*dy );
        dx = dash * dx / d;
        dy = dash * dy / d;

        int x0, y0, x1, y1, i = 0;
        int n = (int) (d / dash);

        x0 = p1.x + (int)(i * dx);

        do{
            y0 = p1.y + (int)(i * dy);
            i++;

            if( i != n-1 )
            {
                x1 = p1.x + (int)(i * dx);
                y1 = p1.y + (int)(i * dy);
            }
            else
            {
                x1 = p2.x;
                y1 = p2.y;
            }

            i++;

            pG.drawLine( x0, y0, x1, y1 );

            x0 = p1.x + (int)(i * dx);

        } while ( i < n );
    }



    private void _drawDashedArc( int x, int y, int width, int height, int startAngle, int arcAngle )
    {
        int radius = (int)((width + height) / 4);
        int numSteps = (int) ((arcAngle*Pi*radius / 180) / dash);
        int angle = (int)(arcAngle / numSteps);

        int a1, a2;

        for( int i=0; i<numSteps; i+=2 )
        {
            a1 = startAngle + (int)(arcAngle*i/ numSteps);
            a2 = a1 + angle;
            pG.drawArc( x, y, width, height, a1, a2 );
        }
    }



    // Some not scaled methods (just to improve performance)


    public void RealDrawLine(int x1, int y1, int x2, int y2)
    {
        if( !mDashed )
            pG.drawLine( x1, y1, x2, y2 );
        else
            _drawDashedLine( x1, y1, x2, y2 );
    }

    public void RealFillRect(int x, int y, int width, int height )
    {   pG.fillRect( x, y, width, height ); }


    // ** dashed able
    public void RealDrawRect(int x, int y, int width, int height )
    {
        if(!mDashed)
            pG.drawRect(  x, y, width, height );
        else
        {
            int x1, y1, x2, y2;
            x1 = x;          y1 = y;
            x2 = x+width;    y2 = y+height;

            _drawDashedLine( x1, y1, x2, y1 );
            _drawDashedLine( x1, y1, x1, y2 );
            _drawDashedLine( x2, y2, x2, y1 );
            _drawDashedLine( x2, y2, x1, y2 );
        }
    }


    public void RealDrawPolyline( CPolyline poly )
    {
        CScale scale = pScale;
        pScale = null;

        drawPolyline( poly );

        pScale = scale;
    }





    // Graphics methods...




    public CGraphics create()
    {   return new CGraphics( pG.create(), pScale );    }

    public CGraphics create(int x, int y, int width, int height)
    {   return new CGraphics( pG.create( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height) ), pScale );    }

    public void translate(int x, int y)
    {   pG.translate( RTS_X(x), RTS_Y(y) );    }



    public Color getColor()
    {   return pG.getColor();    }

    public void setColor(Color c)
    {   pG.setColor(c); }

    public void setPaintMode()
    {   pG.setPaintMode();  }

    public void setXORMode()
    {   pG.setXORMode(Color.white); }

    public void setXORMode(Color c1)
    {   pG.setXORMode(c1); }



    public Font getFont()
    {   return pRealFont; }

    public void setFont(Font font)
    {
        pRealFont = font;
        pScaledFont = new Font( font.getName(), font.getStyle(), RTS_X(font.getSize()) );
        pG.setFont( pScaledFont );
    }

    public FontMetrics getFontMetrics()
    {   return pG.getFontMetrics(pRealFont); }

    public FontMetrics getFontMetrics(Font f)
    {
        Font scaled = new Font( f.getName(), f.getStyle(), RTS_X(f.getSize()) );
        return pG.getFontMetrics(scaled);
    }


    public Rectangle getClipBounds()
    {   return STR_Rect( pG.getClipBounds() ); }

    public void clipRect(int x, int y, int width, int height)
    {   pG.clipRect( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height) ); }

    public void setClip(int x, int y, int width, int height)
    {   pG.setClip( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height) ); }

    public Shape getClip()
    {   return STR_Rect( pG.getClip().getBounds() );   }

    public void setClip(Shape clip)
    {   pG.setClip( RTS_Rect( clip.getBounds() ) );  }

    public void copyArea(int x, int y, int width, int height, int dx, int dy)
    {   pG.copyArea( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height), RTS_X(dx), RTS_Y(dy) ); }


    // ** dashed able
    public void drawLine(int x1, int y1, int x2, int y2)
    {
        if( !mDashed )
            pG.drawLine( RTS_X(x1), RTS_Y(y1), RTS_X(x2), RTS_Y(y2) );
        else
            _drawDashedLine( RTS_X(x1), RTS_Y(y1), RTS_X(x2), RTS_Y(y2) );
    }

    public void fillRect(int x, int y, int width, int height )
    {   pG.fillRect( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height) ); }


    // ** dashed able
    public void drawRect(int x, int y, int width, int height )
    {
        if(!mDashed)
            pG.drawRect( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height) );
        else
        {
            int x1, y1, x2, y2;
            x1 = RTS_X(x);          y1 = RTS_Y(y);
            x2 = RTS_X(x+width);    y2 = RTS_Y(y+height);

            _drawDashedLine( x1, y1, x2, y1 );
            _drawDashedLine( x1, y1, x1, y2 );
            _drawDashedLine( x2, y2, x2, y1 );
            _drawDashedLine( x2, y2, x1, y2 );
        }
    }

    public void clearRect(int x, int y, int width, int height )
    {   pG.clearRect( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height) ); }


    public void drawRoundRect(int x, int y, int width, int height, int arcWidth, int arcHeight)
    {   pG.drawRoundRect( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height), RTS_X(arcWidth), RTS_Y(arcHeight) ); }

    public void fillRoundRect(int x, int y, int width, int height, int arcWidth, int arcHeight)
    {   pG.fillRoundRect( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height), RTS_X(arcWidth), RTS_Y(arcHeight) ); }


    public void draw3DRect(int x, int y, int width, int height, boolean raised )
    {   pG.draw3DRect( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height), raised ); }

    public void fill3DRect(int x, int y, int width, int height, boolean raised )
    {   pG.fill3DRect( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height), raised ); }


    // ** dashed able
    public void drawOval(int x, int y, int width, int height )
    {
        if( !mDashed )
            pG.drawOval( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height) );
        else
            _drawDashedArc( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height), 0, 359 );
    }

    public void fillOval(int x, int y, int width, int height )
    {   pG.fillOval( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height) ); }


    // ** dashed able
    public void drawArc(int x, int y, int width, int height, int startAngle, int arcAngle)
    {
        if( !mDashed )
            pG.drawArc( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height), RTS_X(startAngle), RTS_Y(arcAngle) );
        else
            _drawDashedArc( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height), startAngle, arcAngle );
    }

    public void fillArc(int x, int y, int width, int height, int startAngle, int arcAngle)
    {   pG.fillArc( RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height), RTS_X(startAngle), RTS_Y(arcAngle) ); }


    // ** Dashed able
    public void drawPolyline( CPolyline polyline )
    {
        CPolyline ps = RTS_Poly( polyline );

        if( !mDashed )
            pG.drawPolyline( ps.GetXPoints(), ps.GetYPoints(), ps.GetTotalPoints() );
        else
        {
            Point p1, p2;

            if( ps.GetTotalPoints() < 2 )
                return;

            Enumeration e = ps.GetPoints();
            p1 = (Point) e.nextElement();
            while( e.hasMoreElements() )
            {
                p2 = (Point) e.nextElement();
                _drawDashedLine( p1.x, p1.y, p2.x, p2.y );
                p1 = p2;
            }
        }
    }


    // ** dashed able
    public void drawPolygon( CPolyline polyline )
    {
        CPolyline ps = RTS_Poly( polyline );

        if( !mDashed )
            pG.drawPolygon( ps.GetPolygon() );
        else
        {
            Point p0, p1, p2;

            if( ps.GetTotalPoints() < 2 )
                return;

            Enumeration e = ps.GetPoints();
            p0 = p1 = (Point) e.nextElement();
            while( e.hasMoreElements() )
            {
                p2 = (Point) e.nextElement();
                _drawDashedLine( p1.x, p1.y, p2.x, p2.y );
                p1 = p2;
            }

            _drawDashedLine( p1.x, p1.y, p0.x, p0.y );
        }

    }


    public void fillPolygon( CPolyline polyline )
    {
        CPolyline ps = RTS_Poly( polyline );
        pG.fillPolygon( ps.GetPolygon() );
    }

    public void drawString(String str, int x, int y)
    {   pG.drawString( str, RTS_X(x), RTS_Y(y) );  }



    public boolean drawImage(Image img, int x, int y, ImageObserver observer)
    {   return pG.drawImage(img, RTS_X(x), RTS_Y(y), RTS_X(img.getWidth(observer)), RTS_Y(img.getHeight(observer)), observer );  }

    public boolean drawImage(Image img, int x, int y, int width, int height, ImageObserver observer)
    {   return pG.drawImage(img, RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height), observer ); }

    public boolean drawImage(Image img, int x, int y, Color bgcolor, ImageObserver observer)
    {   return pG.drawImage(img, RTS_X(x), RTS_Y(y), RTS_X(img.getWidth(observer)), RTS_Y(img.getHeight(observer)), bgcolor, observer );  }

    public boolean drawImage(Image img, int x, int y, int width, int height, Color bgcolor, ImageObserver observer)
    {   return pG.drawImage(img, RTS_X(x), RTS_Y(y), RTS_X(width), RTS_Y(height), bgcolor, observer ); }


    public boolean drawImage(Image img, int dx1, int dy1, int dx2, int dy2,
        int sx1, int sy1, int sx2, int sy2, ImageObserver observer)
    {
        return pG.drawImage( img, RTS_X(dx1), RTS_Y(dy1), RTS_X(dx2), RTS_Y(dy2),
            RTS_X(sx1), RTS_Y(sy1), RTS_X(sx2), RTS_Y(sy2), observer);
    }

    public boolean drawImage(Image img, int dx1, int dy1, int dx2, int dy2,
        int sx1, int sy1, int sx2, int sy2, Color bgcolor, ImageObserver observer)
    {
        return pG.drawImage( img, RTS_X(dx1), RTS_Y(dy1), RTS_X(dx2), RTS_Y(dy2),
            RTS_X(sx1), RTS_Y(sy1), RTS_X(sx2), RTS_Y(sy2), bgcolor, observer);
    }

    public void dispose()
    {
        pG.dispose();
    }

    public void finalize()
        throws Throwable
    {
        pG.finalize();
        super.finalize();
    }

    public String toString()
    {   return new String( "CGraphics - " + pG.toString() ); }

}
