/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: GRAPH
 * Classe:  CPolyline
\******************************************************************************/

package GRAPH;

import java.io.*;
import java.awt.*;
import java.util.*;


public class CPolyline
    implements Serializable
{
    protected Vector    pPoints;
    protected Polygon   pPoly;
    protected Polygon   pBounds;

    public CPolyline()
    {   pPoints = new Vector();
        pPoly = null;
        pBounds = null;
    }

    public CPolyline( Polygon poly )
    {
        this();
        for( int i = 0; i<poly.npoints; i++ )
            AddPoint( new Point( poly.xpoints[i], poly.ypoints[i] ) );
    }

    public CPolyline( CPolyline poly )
    {
        this();
        Enumeration e = poly.GetPoints();
        while( e.hasMoreElements() )
        {
            Point p = (Point) e.nextElement();
            AddPoint( new Point(p) );
        }
    }

    /** Update - Notify the polyline any of its internal points has been moved */
    public void Update()
    {
        pPoly = null;
        pBounds = null;
    }

    public void AddPoint( Point p )
    {   pPoints.addElement(p);
        pPoly = null;
        pBounds = null;
    }

    public void RemovePoint( Point p )
    {   pPoints.removeElement( p );
        pPoly = null;
        pBounds = null;
    }


    public Point GetFirstPoint()
    {   return (Point) pPoints.firstElement(); }

    public Point GetLastPoint()
    {   return (Point) pPoints.lastElement(); }

    public void SetFirstPoint(Point p)
    {   pPoints.setElementAt(p, 0); }

    public void SetLastPoint(Point p)
    {   pPoints.setElementAt(p, pPoints.size()-1); }



    public Point GetPointAt( int i )
    {   return (Point) pPoints.elementAt(i); }


    public Enumeration GetPoints()
    {   return pPoints.elements(); }

    public int  GetTotalPoints()
    {   return pPoints.size(); }

    public int[]    GetXPoints()
    {
        int i = 0, n = GetTotalPoints();
        int [] points = new int[n];

        Enumeration e = GetPoints();
        while( e.hasMoreElements() )
        {
            points[i] = ((Point)e.nextElement()).x;
            i++;
        }

        return points;
    }

    public int[]    GetYPoints()
    {
        int i = 0, n = GetTotalPoints();
        int [] points = new int[n];

        Enumeration e = GetPoints();
        while( e.hasMoreElements() )
        {
            points[i] = ((Point)e.nextElement()).y;
            i++;
        }

        return points;
    }

    public Polygon GetPolygon()
    {
        if( pPoly == null )
            pPoly = new Polygon( GetXPoints(), GetYPoints(), GetTotalPoints() );
        return pPoly;
    }

    public boolean Contains( int x, int y )
    {   return Contains( new Point(x,y) );  }

    public boolean Contains( Point pt )
    {
        if( pBounds == null )
        {
            Point p = GetFirstPoint();
            pBounds = new Polygon();
            pBounds.addPoint( p.x+5, p.y+5 );
            _AddPoligonPoint( 1 );
            pBounds.addPoint( p.x-5, p.y-5 );
        }
        return pBounds.contains(pt.x,pt.y);
    }

    private void _AddPoligonPoint( int pos )
    {
        Point p = GetPointAt(pos);

        if( pos % 2 == 0) // Even
            pBounds.addPoint( p.x+5, p.y-5 );
        else // Odd
            pBounds.addPoint( p.x+5, p.y-5 );

        if( pos < (GetTotalPoints()-1) )
            _AddPoligonPoint( pos+1 );

        if( pos % 2 == 0) // Even
            pBounds.addPoint( p.x-5, p.y-5 );
        else // Odd
            pBounds.addPoint( p.x-5, p.y+5 );
    }


    public Rectangle GetBounds()
    {   return GetPolygon().getBounds(); }


    public void Translate( int dx, int dy )
    {
        Enumeration e = pPoints.elements();
        while( e.hasMoreElements() )
        {
            Point p = (Point) e.nextElement();
            p.translate( dx, dy );
        }

        Update();
    }
}