/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: GRAPH
 * Classe:  CSidePosition
\******************************************************************************/

package GRAPH;

import  java.awt.*;
import  java.io.*;


public class    CSidePosition
    implements  Serializable
{
    public static final int     TOP     = 0;
    public static final int     BOTTOM  = 1;
    public static final int     LEFT    = 2;
    public static final int     RIGHT   = 3;

    protected   int             mSide;
    protected   int             mPosition;  // Offset


    public CSidePosition( int side, int position )
    {
        mSide = side;
        mPosition = position;
    }

    public CSidePosition( CSidePosition sidePos )
    {   this( sidePos.GetSide(), sidePos.GetPosition() ); }


    public  int     GetSide()               { return mSide; }
    public  int     GetPosition()           { return mPosition; }

    public  void    SetSide     (int side)  { mSide = side; }
    public  void    SetPosition (int pos)   { mPosition = pos; }

//    public  void    SetSidePosition     ( int side, int pos )
    public  void    Set     ( int side, int pos )
    {
        mSide = side;
        mPosition = pos;
    }

    public  Point   GetPoint ( Rectangle rect )
    {
        int x, y;

        switch( mSide )
        {
            case TOP:
                x = rect.x + GetPosition();
                y = rect.y;
                break;
            case BOTTOM:
                x = rect.x + GetPosition();
                y = rect.y + rect.height;
                break;
            case LEFT:
                x = rect.x;
                y = rect.y + GetPosition();
                break;
            case RIGHT:
            default:
                x = rect.x + rect.width;
                y = rect.y + GetPosition();
        }

        return new Point(x,y);
    }


    public static  CSidePosition   CreateSidePosition  ( Rectangle rect, Point pA, Point pB )
    {
        Point pIn, pOut;

        if( rect.contains(pA) && (!rect.contains(pB)) )
        {   pIn = pA; pOut = pB; }
        else
        if( rect.contains(pB) && (!rect.contains(pA)) )
        {   pIn = pB; pOut = pA; }
        else
            return null;


        int position, side;

        if( pIn.x == pOut.x )
        {
            position = pIn.x - rect.x;

            if( pOut.y < rect.y )  // pOut is above
            {
                side = TOP;
                //pIn.y = rect.y;
            }
            else                   // pOut is below
            {
                side = BOTTOM;
                //pIn.y = rect.y + rect.height;
            }

            return new CSidePosition( side, position );
        }
        else
        if( pIn.y == pOut.y )
        {
            position = pIn.y - rect.y;

            if( pOut.x < rect.x )  // pOut at left
            {
                side = LEFT;
                //pIn.x = rect.x;
            }
            else                   // pOut at right
            {
                side = RIGHT;
                //pIn.x = rect.x + rect.width;
            }

            return new CSidePosition( side, position );
        }
        else
        {
            float a, b;
            int x, y;

            a = ((float)          ( pIn.y - pOut.y )          ) / (pIn.x - pOut.x);
            b = ((float) ( pIn.x * pOut.y - pOut.x * pIn.y )  ) / (pIn.x - pOut.x);


            Rectangle r = new Rectangle( pIn );
            r = r.union( new Rectangle( pOut ) );


            Point p0, p1, p2, p3;
            y = rect.y;                     p0 = new Point( (int)((y - b) / a), y );
            y = rect.y + rect.height;       p1 = new Point( (int)((y - b) / a), y );
            x = rect.x;                     p2 = new Point( x, (int)(a*x + b) );
            x = rect.x + rect.width;        p3 = new Point( x, (int)(a*x + b) );


            if( r.contains(p0) )
            {
                int dist0 = (pIn.x-p0.x)*(pIn.x-p0.x) + (pIn.y-p0.y)*(pIn.y-p0.y);
                int dist;

                if( r.contains(p2) )
                {
                    dist = (pIn.x-p2.x)*(pIn.x-p2.x) + (pIn.y-p2.y)*(pIn.y-p2.y);
                    if( dist0 < dist )
                        return new CSidePosition( TOP, p0.x - rect.x );
                    else
                        return new CSidePosition( LEFT, p2.y - rect.y );
                }
                else
                if( r.contains(p3) )
                {
                    dist = (pIn.x-p3.x)*(pIn.x-p3.x) + (pIn.y-p3.y)*(pIn.y-p3.y);
                    if( dist0 < dist )
                        return new CSidePosition( TOP, p0.x - rect.x );
                    else
                        return new CSidePosition( RIGHT, p3.y - rect.y );
                }
                else
                    return new CSidePosition( TOP, p0.x - rect.x );
            }
            else

            if( r.contains(p1) )
            {
                int dist1 = (pIn.x-p1.x)*(pIn.x-p1.x) + (pIn.y-p1.y)*(pIn.y-p1.y);
                int dist;

                if( r.contains(p2) )
                {
                    dist = (pIn.x-p2.x)*(pIn.x-p2.x) + (pIn.y-p2.y)*(pIn.y-p2.y);
                    if( dist1 < dist )
                        return new CSidePosition( BOTTOM, p1.x - rect.x );
                    else
                        return new CSidePosition( LEFT, p2.y - rect.y );
                }
                else
                if( r.contains(p3) )
                {
                    dist = (pIn.x-p3.x)*(pIn.x-p3.x) + (pIn.y-p3.y)*(pIn.y-p3.y);
                    if( dist1 < dist )
                        return new CSidePosition( BOTTOM, p1.x - rect.x );
                    else
                        return new CSidePosition( RIGHT, p3.y - rect.y );
                }
                else
                    return new CSidePosition( BOTTOM, p1.x - rect.x );
            }
            else

            if( r.contains(p2) )
                return new CSidePosition( LEFT, p2.y - rect.y );

            else
                return new CSidePosition( RIGHT, p3.y - rect.y );
        }

    }
}