/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: PROCESS
 * Classe:  CAbstractActivity
\******************************************************************************/

package PROCESS;

import java.io.*;
import java.util.*;

import FILE.CElement;

import PROCESS.MODEL.*;





public abstract class    CAbstractActivity
    extends     CElement
{
    protected static    String      sProcessName = "Process";

    protected CAbstractProcess      pProcess;
    protected CStep                 pStep;

    protected Vector                pSubActs;
    protected Vector                pRoles;

    protected int                   mToolId; // If this Activity is general or defined by a specific tool

    public  CAbstractActivity()
    {
        super();

        pProcess = null;
        pStep = null;
        pSubActs = null;
        pRoles = new Vector();
        mToolId = 0;
    }

    // Process
    public CAbstractProcess     GetProcess ()
    {
        return pProcess;
    }

    public void SetProcess (CAbstractProcess proc)
    {
        pProcess = proc;
        pProcess.AddActivity( this );
    }

    // Tool
    public boolean  IsDefinedByTool ()              { return mToolId != 0; }
    public int      GetToolId ()                    { return mToolId; }
    public void     SetToolId ( int id )
    {
        mToolId = id;
    }

    // Step
    public CStep  GetStep  ()                       { return pStep; }
    public void   SetStep  (CStep step)
    {
        pStep = step;

        // Setup activities
        CSubModel subModel = pStep.GetSubModel();
        SetSubActivities( subModel );
    }

    // SubActivities
    public Enumeration  GetSubActivities ()
    {
        if( pSubActs != null )
            return pSubActs.elements();
        else
            return (new Vector()).elements();
    }


    public void         SetSubActivities( CSubModel subModel )
    {
        if( subModel == null )
            return;

        if( pStep == subModel.GetOwnerStep() )
        {
            pSubActs = new Vector();
            Enumeration enum = subModel.GetModelElements();
            while( enum.hasMoreElements() )
            {
                CModelElement mElem = (CModelElement) enum.nextElement();
                if( mElem instanceof CStep )
                {
                    try{
                        CAbstractActivity act = (CAbstractActivity)this.getClass().newInstance();
                        act.SetProcess( pProcess );
                        pSubActs.addElement( act );
                        act.SetStep( (CStep)mElem );
                    }
                    catch(Exception e) {};
                }
            }

            if( pSubActs.size() == 0 )
                pSubActs = null;

        }

    }


    /**
     * Add a sub-activity to this activity.
     * Here, the sub-activity won't be added to the process.
     * If the subactivity is a new activity in the process, it should be added
     * explicitly by calling "process.AddElement( subActivity.);".
     */
    public void     AddActivity( CAbstractActivity subActivity )
    {
        pSubActs.addElement( subActivity );
    }


    // Roles
    public void         AddRole    ( CAbstractRole role )   { pRoles.addElement( role ); }
    public void         RemoveRole ( CAbstractRole role )   { pRoles.removeElement( role ); }
    public Enumeration  GetRoles   ()               { return pRoles.elements(); }


    // Name
    public String   GetName()
    {
        if( pStep == null )
            return sProcessName;
        else
            return pStep.GetName();
    }

    public void     SetName(String name)
    {
        if( pStep != null )
            pStep.SetName(name);
        else
            sProcessName = name;
    }


    //** DEBUG
    public void DEBUG_write( PrintStream pStr )
    {
        pStr.println( "DEBUG: CAbstractActivity" );
        pStr.println( "       Name:     " + GetName() );
        pStr.println( "       DefTool:  " + GetToolId() );

        super.DEBUG_write( pStr );

        if( pSubActs != null )
        {
            pStr.println( "--- SubActivities ---" );

            Enumeration en = GetSubActivities();
            while( en.hasMoreElements() )
            {
                CAbstractActivity act = (CActivity) en.nextElement();
                act.DEBUG_write( pStr );
            }

            pStr.println( "---------------------" );
        }

    }


}


