
/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: PROCESS
 * Classe:  CFlow
\******************************************************************************/

package PROCESS2;

import java.io.*;
import java.util.*;

import FILE.CElement;


public class CFlow
    extends CElement
{
    public static final int     SIMPLE      = -1;
    public static final int     AND_SPLIT   = 0;
    public static final int     AND_JOIN    = 1;
    public static final int     OR_SPLIT    = 2;
    public static final int     OR_JOIN     = 3;

    protected   int             mType;

    protected   Vector          pFromActivities;
    protected   Vector          pToActivities;

    protected   CAbstractActivity   pParentActivity;


    public CFlow( CAbstractActivity parent )
    {
        super();

        mType = SIMPLE;
        pParentActivity = parent;

        pFromActivities = new Vector( 1,1 );
        pToActivities   = new Vector( 1,1 );
    }



    protected   boolean     ValidateActivity( CAbstractActivity act )
    {
        if( act == null )   return false;
        return( act.GetParent() == pParentActivity );
    }



    public int  GetType()       { return mType; }

    public void SetType( int type )
    {
        if( type == mType ) return;

        CAbstractActivity a, b;
        a = b = null;

        if( pFromActivities.size() > 0 )
            a = (CAbstractActivity)pFromActivities.firstElement();

        if( pToActivities.size() > 0 )
            b = (CAbstractActivity)pToActivities.firstElement();

        switch( type )
        {
            case SIMPLE:
                SetFromActivity( a );
                SetToActivity( b );
                break;

            case AND_SPLIT:
            case OR_SPLIT:
                if( (mType == AND_SPLIT) || (mType == OR_SPLIT) )
                    break;
                SetFromActivity( a );
                break;

            case AND_JOIN:
            case OR_JOIN:
                if( (mType == AND_JOIN) || (mType == OR_JOIN) )
                    break;
                SetToActivity( b );
                break;

            default:
                return;

        }

        mType = type;
    }




    public boolean SetFromActivity ( CAbstractActivity from )
    {
        if( !ValidateActivity(from) )
            return  false;

        pFromActivities.removeAllElements();
        pFromActivities.addElement( from );
        return true;
    }

    public boolean SetToActivity   ( CAbstractActivity to )
    {
        if( !ValidateActivity(to) )
            return  false;

        pToActivities.removeAllElements();
        pToActivities.addElement( to );
        return true;
    }



    public boolean AddFromActivity ( CAbstractActivity from )
    {
        if( !ValidateActivity(from) )
            return  false;

        if( pFromActivities.size() != 0 )
            if( (mType != AND_JOIN) && (mType != OR_JOIN) )
                return false;

        pFromActivities.addElement( from );
        return true;
    }

    public boolean AddToActivity   ( CAbstractActivity to )
    {
        if( !ValidateActivity(to) )
            return  false;

        if( pToActivities.size() != 0 )
            if( (mType != AND_SPLIT) && (mType != OR_SPLIT) )
                return false;

        pToActivities.addElement( to );
        return true;
    }



    public CAbstractActivity    GetParent ()
    {   return pParentActivity; }

/*
    public void SetParentActivity (CAbstractActivity parent)
    {   pParentActivity = parent; }
*/



    public Enumeration     GetFromActivities()
    {
        return pFromActivities.elements();
    }

    public Enumeration     GetToActivities()
    {
        return pToActivities.elements();
    }


    //** DEBUG
    public void DEBUG_write( PrintStream pStr )
    {
        pStr.println( "DEBUG: CFlow" );
        pStr.println( "       Type:     " +
              ( (mType == SIMPLE) ? "SIMPLE" :
                (mType == AND_SPLIT) ? "AND_SPLIT" :
                (mType == AND_JOIN)  ? "AND_JOIN" :
                (mType == OR_SPLIT)  ? "OR_SPLIT" :
                (mType == OR_JOIN)   ? "OR_JOIN" : "<Unidentified>" )
            );

        super.DEBUG_write( pStr );

        pStr.println( "----- From Activities:" );

        Enumeration en = GetFromActivities();
        while( en.hasMoreElements() )
        {
            CAbstractActivity act = (CAbstractActivity) en.nextElement();
            pStr.println( "     - Activity: " + act.GetName() );
        }


        pStr.println( "\n----- To Activities:" );

        en = GetToActivities();
        while( en.hasMoreElements() )
        {
            CAbstractActivity act = (CAbstractActivity) en.nextElement();
            pStr.println( "     - Activity: " + act.GetName() );
        }


        pStr.println( "---------------------" );
    }

}