/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: TOOL.UML.CLIENT
 * Classe:  UMLGUIToolManager
\******************************************************************************/

package TOOL.UML.CLIENT;

import  java.awt.*;

import  COPSE.CEvent;
import  COPSE.CLIENT.*;
import  COPSE.CLIENT.GUI.*;
import  COPSE.CONTROL.*;
import  GRAPH.*;
import  TOOL.UML.*;

//temp - DEBUG
import FILE.*;
import FILE.OBJ.*;

public class    UMLGUIToolManager
    extends     CGUIToolManager
    //implements  WindowListener, CClientFileControllerListener
{
    protected   UMLClassControl     pClassControl;
    protected   UMLRelationControl  pRelationControl;

    protected   UMLCanvas           pUMLCanvas;
    protected   CUMLGraphElement    pSelectedElement;


    public String  GetTitle ()
    {   return "CUTE - Cooperative UML Technique Editor";  }

    public UMLGUIToolManager ( CGUIManager gui )
    {
        super( gui );
    }

    protected CClient   CreateClient( int userId, String userName, String host, int port )
        throws Exception
    {
        return new UMLClient( userId, userName, host, port );
    }


    public void AddMenu ( MenuBar menuBar )
    {
        super.AddMenu( menuBar );

        // Add specific tool menus and controls...
        pClassControl = new UMLClassControl(this);
        pClassControl.AddMenu( menuBar );

        pRelationControl = new UMLRelationControl(this);
        pRelationControl.AddMenu( menuBar );
    }



    public      UMLCanvas   GetCanvas()             { return pUMLCanvas; }



    public      void        ChangeMenuSettings()
    {
        super.ChangeMenuSettings();

        pClassControl.ChangeMenuSettings();
        pRelationControl.ChangeMenuSettings();
    }

    public      void        SetSelectedElement( CUMLGraphElement element )
    {
        if( (pSelectedElement!=null) && (pSelectedElement instanceof CUMLGraphClass) )
            pClassControl.SetSelectedElement( null );
/* */
        else
        if( pSelectedElement instanceof CUMLGraphRelation )
            pRelationControl.SetSelectedElement( null );
/* */

        pSelectedElement = element;

        if( (pSelectedElement!=null) && (pSelectedElement instanceof CUMLGraphClass) )
            pClassControl.SetSelectedElement( (CUMLGraphClass)pSelectedElement );
/* */
        else
        if( pSelectedElement instanceof CUMLGraphRelation )
            pRelationControl.SetSelectedElement( (CUMLGraphRelation)pSelectedElement );
/* */
    }

    protected   Panel       CreateClientPanel()
    {
        Panel p;
        p = new Panel();
        p.setLayout( new BorderLayout() );

        pUMLCanvas = new UMLCanvas(this);
        pUMLCanvas.setSize( 1000, 1000 );

        ScrollPane scrollPane = new ScrollPane();
        scrollPane.add( pUMLCanvas );

        p.add( "Center", scrollPane );
        return p;
    }


    public void CloseClient( boolean exit )
    {
        pClassControl.SetEnabled( false );
        pRelationControl.SetEnabled( false );
        pUMLCanvas.SetFileController( null );

        super.CloseClient( exit );

        if( pClient == null )
        {
        }
    }




    public void CreateNewClass( Rectangle rect )
    {
        UMLDlgClassNew dlg = new UMLDlgClassNew( this, rect );
        dlg.setVisible( true );
    }

    public void CreateNewRelation( CPolyline poly, CObjClass classes[] )
    {
        int classesId[] = new int[2];
        CSidePosition sides[] = new CSidePosition[2];
        Point first, last, aux;

        first = poly.GetFirstPoint();
        last = poly.GetLastPoint();

        poly.RemovePoint( first );
        poly.RemovePoint( last );


        classesId[0] = classes[0].GetId();
        {   try {
                aux = poly.GetFirstPoint();
            } catch (Exception e) { aux = last; }
        }
        sides[0] = CSidePosition.CreateSidePosition( classes[0].GetRegion(), first, aux );


        classesId[1] = classes[1].GetId();
        {   try {
                aux = poly.GetLastPoint();
            } catch (Exception e) { aux = first; }
        }
        sides[1] = CSidePosition.CreateSidePosition( classes[1].GetRegion(), last, aux );


        UMLDlgRelationNew dlg = new UMLDlgRelationNew( this, poly, classesId, sides );
        dlg.setVisible( true );
    }


    //* FileController Listener Methods
    public void  FileControllerLoaded()
    {
        super.FileControllerLoaded();

        System.err.println( "{DEBUG} UMLGUIToolManager - FileController Loaded..." );
        pUMLCanvas.SetFileController( pClient.GetFileController() );
        //pClientPanel.repaint(); // ?

        pClassControl.Activate();
        pRelationControl.Activate();
    }

    public void  ElementStateChanged (int elemId, CEvent event)
    {
        int type = event.GetType();

        //pClientPanel.repaint(); // ?
        switch( type )
        {
            case CEvent.EvCOPSE_FileController_Element_New:
                {
                    pUMLCanvas.AddElement( elemId );
                    int connId = ((Integer) event.GetArgumentAt(0) ).intValue();
                    if( pClient.GetSessionManager().GetClientSession().GetConnectionId() == connId )
                        pUMLCanvas.SelectElement( elemId );
                }
                break;

            case CEvent.EvCOPSE_FileController_Element_Delete:
                pUMLCanvas.RemoveElement( elemId );
                break;


            case CEvent.EvCOPSE_FileController_Element_Lock:
                {
                    if( (pSelectedElement != null) &&
                        (pSelectedElement.GetElementController().GetElement().GetId() == elemId) )
                    {

                        if( (pClassControl.GetSelectedElement() == pSelectedElement) && pClassControl.IsWaitingForLock() )
                        {
                            pClassControl.ReceiveLockConfirmation();
                        }

                        else
                        if( (pRelationControl.GetSelectedElement() == pSelectedElement) && pRelationControl.IsWaitingForLock() )
                        {
                            pRelationControl.ReceiveLockConfirmation();
                        }
                    }

                    pUMLCanvas.UpdateElement( elemId );
                }
                break;

            // TEMP - DEBUG
            case CEvent.EvCOPSE_FileController_Element_Update:
                {
                    CElement elem = pClient.GetFileController().GetFile().GetElement( elemId );
                    if( elem instanceof CObjRelation )
                    {
                        System.err.println( "[DEBUG] UMLGUIToolManager - Received Event Update Relation: ");
                        elem.DEBUG_write( System.err );
                    }
                }
                //break;

            default:
                pUMLCanvas.UpdateElement( elemId );
        }
        pUMLCanvas.repaint();
    }


}


