/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: TOOL.UML
 * Classe:  CUMLGraphClass
\******************************************************************************/

package TOOL.UML;

import java.awt.*;
import java.util.*;

import  FILE.*;
import  FILE.OBJ.*;

import  GRAPH.*;

import  COPSE.CONTROL.*;
import  COPSE.CUserSession;


/*************************************************************************
 *  Class CUMLGraphClass
 *************************************************************************/
public class CUMLGraphClass
    extends CUMLGraphElement
{
    // Data
    protected   CObjClass       pClass;


    // Presentation
    protected   Rectangle       pBase;

    protected   CGrpRectangle   pgRect;         // Class Region
    protected   CGrpLabel       pgName;         // Class Name

    protected   CGrpLabel       pgStereo;       // Class Stereotype

    protected   CGrpRectangle   pgParamRect;    // Class Parameters Rect
    protected   CGrpLabel       pgParamText;    // Class Parameters Text



    public CUMLGraphClass ( CVisualElementController elemController )
    {
        super( elemController );
        pClass = (CObjClass) elemController.GetElement();

        _CreateGraphicObjects();
    }

    private void _CreateGraphicObjects()
    {
        // Class Rectangle
        Rectangle rect = pClass.GetRegion();
        pBase = rect;

        pgRect = new CGrpRectangle( new Rectangle( 0, 0, rect.width, rect.height ) );

        // Class Name
        Font font;
        if( pClass.IsAbstract() )
            font = new Font( "Helvetica", Font.BOLD | Font.ITALIC, 12 );
        else
            font = new Font( "Helvetica", Font.BOLD, 12 );

        pgName = new CGrpLabel( new Point( rect.width/2, 10), pClass.GetName(),
                CGrpLabel.CENTER, font );

        String text;
        font = new Font( "Helvetica", Font.PLAIN, 10 );

        // Stereotype
        text = pClass.GetStereotype();
        if( !text.equals("") )
            pgStereo = new CGrpLabel( new Point( 2, 0 ), text, CGrpLabel.LEFT, font );
        else
            pgStereo = null;

        // Parameters
        text = pClass.GetParameters();
        if( !text.equals("") )
            pgParamText = new CGrpLabel( new Point( rect.width+5, -5 ), text, CGrpLabel.RIGHT, font );
        else
            pgParamText = null;

        pgParamRect = null;

    }


    public boolean     Contains (int x, int y)
    {
        return pBase.contains(x,y);
    }





    public void Draw( CGraphics graf )
    {
        // Contextos: g1 (sem clipping) e g2 (com clipping)
        CGraphics g1 = graf.create();
        g1.translate( pBase.x, pBase.y );
        CGraphics g2 = graf.create();
        g2.translate( pBase.x, pBase.y );
        g2.clipRect( 0, 0, pBase.width, pBase.height );

        if( pElemController.IsLocked() )
        {
        	// temporary
            int connId = pElemController.GetLockerConnectionId();
            //int userId =
        	Color userColor = CUserSession.getUserColor( connId );
            g1.setColor( userColor );
            g2.setColor( userColor );
        }

        pgRect.Draw( g1 );

        if( pgStereo != null ) pgStereo.Draw( g2 );

        if( pgParamText != null )
        {
            if( pgParamRect == null )
            {
                Rectangle r = pgParamText.GetRegion( g1 );
                r.grow(2,2);
                pgParamRect = new CGrpRectangle( r, true, true );
                pgParamRect.SetFillColor( Color.white );
            }
            pgParamRect.Draw( g1 );
            pgParamText.Draw( g1 );
        }

        pgName.Draw( g2 );

        int visual = pElemController.GetVisual();

        if( visual != 0 ) // Not Minimum
        {

            int y = 32, h;

            Point point = new Point( 5, 0 );
            Font font = new Font( "Helvetica", Font.PLAIN, 10 );
            g2.setFont( font );
            FontMetrics fm = g2.getFontMetrics();
            h = fm.getHeight() + 1;
            fm = null;

            CGrpLabel gtext = new CGrpLabel( point, "", CGrpLabel.LEFT, font );
            CGrpLine line = new CGrpLine( new Point( 0, y ), new Point( pBase.width, y ) );
            line.Draw( g2 );


            String text;

            // Get and show attributes
            y += 2;
            Enumeration e = pClass.GetAttributes();
            while( e.hasMoreElements() )
            {
                CObjAttribute attr = (CObjAttribute) e.nextElement();
                text = attr.GetName();

                if( visual == 2 ) // TOTAL
                {
                    String t = attr.GetType();
                    if( !t.equals("") )  text = text + ": " + t;

                    t = attr.GetInitialValue();
                    if( !t.equals("") )  text = text + " = " + t;
                }

                point.y = y;
                gtext.SetText( text );
                gtext.Draw( g2 );
                y += h;
            }

            line = new CGrpLine( new Point( 0, y ), new Point( pBase.width, y ) );
            line.Draw( g2 );

            // Get and show methods
            y += 2;
            e = pClass.GetMethods();
            while( e.hasMoreElements() )
            {
                CObjMethod met = (CObjMethod) e.nextElement();
                text = met.GetName();

                if( visual == 2 ) // TOTAL
                    text = text + " " + met.GetParameters() + ": " + met.GetType();

                point.y = y;
                gtext.SetText( text );
                gtext.Draw( g2 );
                y += h;
            }

            font = null;
            line = null;
            gtext = null;
        }

        g2.dispose();
        g1.dispose();
    }


    // if the class is changed
    public void Update()
    {
        _CreateGraphicObjects();
    }

    public Rectangle GetBounds()
    {
        return pBase;
    }
}


/*
    public void Move ( int x, int y )
    {
        pBase.move( x, y );
        //pClass.SetRegion( pBase ); ??
        _CreateGraphicObjects();
    }
*/


/*
    public void SetRegiao( Rectangle rect )
    {
        // p_Base = rect;
        p_classe.SetRegiao( rect );
        ProcessaClasse();
    }

*/



