/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: TOOL.UML
 * Classe:  CUMLGraphModel
\******************************************************************************/

package TOOL.UML;

import  java.util.*;

import  FILE.*;
import  FILE.OBJ.*;

import  GRAPH.*;

import  COPSE.CONTROL.*;

/*************************************************************************
 *  Class CUMLGraphModel
 *************************************************************************/
public class CUMLGraphModel
{
    // Data
    protected   CUMLFileController      pFileController;
    //protected   CFile                   pFile;

    // Graphic Elements...
    protected   Vector                  pUMLElements;



    public CUMLGraphModel ( CUMLFileController controller )
    {
        pFileController = controller;

        _CreateUMLElements();
    }

    private void _CreateUMLElements()
    {
        pUMLElements = new Vector();

        Enumeration e = pFileController.GetElementControllers();
        while( e.hasMoreElements() )
        {
            CVisualElementController elemCtrl = (CVisualElementController) e.nextElement();
            CElement elem = elemCtrl.GetElement();

            if( elem instanceof CObjClass )
            {
                pUMLElements.addElement( new CUMLGraphClass( elemCtrl ) );
            }
            else
            if( elem instanceof CObjRelation )
            {
                pUMLElements.addElement( new CUMLGraphRelation( elemCtrl ) );
            }
            else
            {}
        }

    }

    public void Draw( CGraphics g )
    {
        Enumeration e = pUMLElements.elements();
        while( e.hasMoreElements() )
        {
            CUMLGraphElement elem = (CUMLGraphElement) e.nextElement();
            elem.Draw( g );
        }
    }

    public CUMLGraphElement     GetElementAt( int rx, int ry )
    {
        Enumeration e = pUMLElements.elements();
        while( e.hasMoreElements() )
        {
            CUMLGraphElement elem = (CUMLGraphElement) e.nextElement();
            if( elem.Contains( rx, ry ) )
                return elem;
        }
        return null;
    }


    public void AddElement( int elemId )
    {
        CVisualElementController control = (CVisualElementController) pFileController.FindElementController( elemId );
        CElement element = control.GetElement();

        if( element instanceof CObjClass )
        {
            pUMLElements.addElement( new CUMLGraphClass( control ) );
        }
        else
        if( element instanceof CObjRelation )
        {
            pUMLElements.addElement( new CUMLGraphRelation( control ) );
        }
        else
        {}
    }



    public CUMLGraphElement FindElement( int elemId )
    {
        Enumeration e = pUMLElements.elements();
        while( e.hasMoreElements() )
        {
            CUMLGraphElement graphElement = (CUMLGraphElement) e.nextElement();
            if( graphElement.GetElementController().GetElement().GetId() == elemId )
                return graphElement;
        }
        return null;
    }


    public void UpdateElement( int elemId )
    {
        Enumeration e = pUMLElements.elements();
        while( e.hasMoreElements() )
        {
            CUMLGraphElement graphElement = (CUMLGraphElement) e.nextElement();
            if( graphElement.GetElementController().GetElement().GetId() == elemId )
            {
                graphElement.Update();

                if( graphElement instanceof CUMLGraphClass )
                {
                    System.err.println( "[DEBUG] CUMLGraphModel.UpdateElement - element is CUMLGraphClass...");
                    CObjClass aclass = (CObjClass) graphElement.GetElementController().GetElement();
                    Enumeration e2 = aclass.GetRelations();
                    while( e2.hasMoreElements() )
                    {
                        CElement elem = (CElement) e2.nextElement();
                        UpdateElement( elem.GetId() );
                    }
                }

                return;
            }
        }
    }

    public void RemoveElement( int elemId )
    {
        Enumeration e = pUMLElements.elements();
        while( e.hasMoreElements() )
        {
            CUMLGraphElement graphElement = (CUMLGraphElement) e.nextElement();
            if( graphElement.GetElementController().GetElement().GetId() == elemId )
            {
                pUMLElements.removeElement( graphElement );

                if( graphElement instanceof CUMLGraphClass )
                {
                    System.err.println( "[DEBUG] CUMLGraphModel.UpdateElement - element is CUMLGraphClass...");
                    CObjClass aclass = (CObjClass) graphElement.GetElementController().GetElement();
                    Enumeration e2 = aclass.GetRelations();
                    while( e2.hasMoreElements() )
                    {
                        CElement elem = (CElement) e2.nextElement();
                        RemoveElement( elem.GetId() );
                    }
                }

                return;
            }
        }
    }

}