/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Project Server Application
 * Classe: CProjectManager
\******************************************************************************/


import java.io.*;
//import java.net.*;
import java.util.*;


//import COPSE.*;
import FILE.*;

import AGENT.*;
import PROCESS.MODEL.*;
import PROCESS.*;
import PRODUCT.*;

public class CProjectManager
{
    //COPSEManager        pCOPSEManager;

    ProjectEntry        pProjectEntry;



    CProduct            pProduct;

    //CProcess            pProcess;

/*
    CFile               pPrjUsers;
    CFile               pPrjTeams;
    CFile               pTools; // ???


*/



    public CProjectManager( /*COPSEManager parent,*/ String projectId, String host, int port )
        throws Exception
    {
        //pCOPSEManager = parent;


        pProjectEntry = _OpenProject( projectId, host, port );

        //_LoadUsers();
        //_LoadProcess();
        _LoadProduct();

    }


    public String GetProjectDir()
    {
        return pProjectEntry.GetPath();
    }

    public String GetProductDir()
    {
        return new String( pProjectEntry.GetPath() + "/product" );
    }


    public String GetServerNameFor( CLocator locator )
    {
        if( locator.GetDocId() == 0 )
        {
            // Special Files
            switch( locator.GetVerId() )
            {
                case 0: // Process
                    return "ProcessServer";
            //    case 1:
            }
        }
        else
            return "DefaultFileServer";
    }







    private void _LoadProduct()
    {
        try{
            pProduct = (CProduct) CFile.Load( "product.copse", GetProjectDir() );
        } catch (Exception e)
        {
            pProduct = _CreateDefaultProduct();
        }
    }


    private CFile _LoadSpecialProjectFile( int type )
    {
        CFile file;

        switch( type )
        {
            case 0:     // Process
                {
                    CProcess process;
                    try {
                        process = (CProcess) CFile.Load( "process.copse", GetProjectDir() );
                    } catch( Exception e )
                    {
                        process = _CreateDefaultProcess();
                    }
                    return process;
                }


            case 1:     // Users
                try{
                    file = CFile.Load( "users.copse", GetProjectDir() );
                } catch(Exception e)
                {
                    file = _CreateDefaultUsers();
                }
                return file;


            case 2:    // Product
                return pProduct;


            case 3:
                ;

            default:
                file = new CFile();

        }
        return file;
    }



    public CFile LoadFile( CLocator locator )
        throws IOException
    {
        // Special Project "Files"
        if( locator.GetDocId() == 0 )
            return _LoadSpecialProjectFile( locator.GetVersionId() );


        CElement elem = pProduct.GetElement( locator.GetDocId() );
        if( !(elem instanceof CDocEntry) )
            throw new IOException( "Invalid Document reference in Product definition!" );

        CDocEntry entry = (CDocEntry) elem;



        // Load Document
        CDocument doc;
        try {
            doc = entry.Load( GetProductDir() );
        } catch ( Exception e1 )
        {
            // Create the Document
            doc = entry.Create( GetProductDir() );

            new CDocument( entry.GetPath(), GetProductDir() );
            doc.SetTypeId( entry.GetDocTypeId() );
        }

        // Get Version
        CVersion version = doc.GetVersion( locator.GetVersionId() );
        if( version == null )
        {   // Create a new Version
            version = doc.CreateVersion();
            try { doc.Save(); } catch (IOException e){};
            locator.SetVersionId( version.GetId() );
        }

        // Load File
        CFile file = version.GetFile( GetProductDir() );
        if( file == null )
        {
            switch( doc.GetTypeId() )
            {
                case 0:
                default:
                    // Create the file of this version
                    file = new CFile( version.GetFileName(),
                        GetProductDir() + "/" + entry.GetParentPath() );
                    //???
                    break;
            }
        }

        file.Save();
        return file;
    }




    private CProcess _CreateDefaultProcess()
    {
        CProcess process = new CProcess( "process.copse", GetProjectDir() );

        CModelHeader header;
        process.AddElement( header = new CModelHeader() );
        process.SetModelHeader( header );
        header.SetName( "Process" );

        CSubModel   submodel;
        process.AddElement( submodel = new CSubModel() );

        header.SetSubModel( submodel );

        process.SetupActivities();

        try {
            process.Save();
        } catch (IOException e) {};
        return process;
    }


    private CFile _CreateDefaultUsers()
    {
        CFile file = new CFile( "users.copse", GetProjectDir() );

        CMember member = new CMember();
        member.SetLogin("admin");
        member.SetPassword("admin");
        member.SetName("Project Administrator");

        file.AddElement( member );

        try {
            file.Save();
        } catch (IOException e) {};
        return file;
    }


    private CProduct _CreateDefaultProduct()
    {
        CProduct prod = new CProduct( "product.copse", GetProjectDir() );

        try {
            prod.Save();
            File file = new File( GetProductDir() );
            file.mkdirs();
        } catch (IOException e) {};
        return prod;
    }


/*
    private void _LoadUsers()
        throws Exception
    {
        // Load Users...
        try {
            pPrjUsers = CFile.Load( "users.copse", pProjectEntry.GetPath() );
        } catch (Exception e)
        {
            // TEMPORARY ???
            pPrjUsers = new CFile( "users.copse", pProjectEntry.GetPath() );
            CMember member = new CMember();
            member.SetLogin("admin");
            member.SetPassword("admin");
            member.SetName("Project Administrator");

            pPrjUsers.AddElement( member );
            pPrjUsers.Save();
        }

    }


    private void _LoadProcess()
        throws Exception
    {
        // Load Process ...
        try {
            // Try to Load existing process
            pPrjProcess = (CProcess) CFile.Load( "process.copse", pProjectEntry.GetPath() );
        } catch (Exception e)
        {
            // TEMPORARY ???
            // If something wrong happens, it tries to create another process
            CProcess process = new CProcess( "process.copse", pProjectEntry.GetPath() );

            CModelHeader header;
            process.AddElement( header = new CModelHeader() );
            process.SetModelHeader( header );
            header.SetName( "Process" );

            CSubModel   submodel;
            process.AddElement( submodel = new CSubModel() );

            header.SetSubModel( submodel );

            process.SetupActivities();
            process.Save();
        }
    }
*/

/*
    public CFile        GetProjectUsers()       { return pPrjUsers; }
    public CProcess     GetProjectProcess()     { return pPrjProcess; }
*/







    private ProjectEntry _OpenProject( String projectId, String host, int port )
        throws Exception
    {
        ProjectsIndex pIndex;
        ProjectEntry prj;

        // Read Projects Index...
        FileInputStream file = new FileInputStream( "projects.index" );
        ObjectInputStream input = new ObjectInputStream( file );
        pIndex = (ProjectsIndex) input.readObject();
        file.close();

        prj = _FindProject( pIndex, projectId );
        if( prj == null )
        {
            System.out.println( "Project '"+projectId+"' not found!" );
            pIndex.DEBUG_write( System.out );
            throw new Exception( "Project '"+projectId+"' not found!" );
        }

        if( prj.IsOpen() )
        {
            System.out.println( "Project '"+projectId+"' has already been open!" );
            prj.DEBUG_write( System.out );
            throw new Exception( "Project '"+projectId+"' has already been open!" );
        }

        // Change the Project Status...
        prj.Open( host, port );


        // Save this information to ProjectsIndex
        FileOutputStream outputFile = new FileOutputStream( "projects.index" );
        ObjectOutputStream output = new ObjectOutputStream( outputFile );
        output.writeObject( pIndex );
        outputFile.close();

        return prj;
    }




    private void _CloseProject()
    {
        if( pProjectEntry == null )
            return;

        ProjectsIndex pIndex;
        ProjectEntry prj;

        try {
            // Read Projects Index...
            FileInputStream file = new FileInputStream( "projects.index" );
            ObjectInputStream input = new ObjectInputStream( file );
            pIndex = (ProjectsIndex) input.readObject();
            file.close();

            prj = _FindProject( pIndex, new Integer(pProjectEntry.GetId()).toString() );

            // Change the Project Status...
            prj.Close();


            // Save this information to ProjectsIndex
            FileOutputStream outputFile = new FileOutputStream( "projects.index" );
            ObjectOutputStream output = new ObjectOutputStream( outputFile );
            output.writeObject( pIndex );
            outputFile.close();
        }catch (Exception e) {}

        pProjectEntry = null;
    }

/*
    private void _CloseProject()
    {
        if( pProjectEntry != null )
        {
            pProjectEntry.Close();
            pProjectEntry = null;
        }
    }
*/

    private ProjectEntry _FindProject( ProjectsIndex index, String projectId )
    {
        ProjectEntry prj;
        int id = 0;
        try{
            id = new Integer( projectId ).intValue();
        }
        catch( Exception e ){};


        Enumeration e = index.GetProjects();
        if( id != 0 )
            while( e.hasMoreElements() )
            {
                prj = (ProjectEntry) e.nextElement();
                if( prj.GetId() == id )
                    return prj;
            }
        else
            while( e.hasMoreElements() )
            {
                prj = (ProjectEntry) e.nextElement();
                if( prj.GetName().equals(projectId) )
                    return prj;
            }

        return null;
    }


    public void Exit()
    {
        // if any of documents has changed, then save it
        try {
            pProduct.Save();
        } catch(IOException e) {}

        _CloseProject();
    }

}