/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Project Server Application
 * Classe: ProjectManager
\******************************************************************************/


import java.io.*;
//import java.net.*;
import java.util.*;


//import COPSE.*;
import FILE.*;

import AGENT.*;
import PROCESS.MODEL.*;
import PROCESS.*;

public class ProjectManager
{
    COPSEManager        pCOPSEManager;

    ProjectEntry        pProjectEntry;

    CFile               pPrjUsers;

    //ProcessManager    pProcessManager;
    CProcess            pPrjProcess;

    //ProductManager    pProductManager;
    CFile   pPrjProduct;

    CFile   pPrjTeams;

    CFile   pPrjTools; // ???






    public ProjectManager( COPSEManager parent, String projectId, String host, int port )
        throws Exception
    {
        pCOPSEManager = parent;


        pProjectEntry = _OpenProject( projectId, host, port );

        _LoadUsers();
        _LoadProcess();

    }

    private void _LoadUsers()
        throws Exception
    {
        // Load Users...
        try {
            pPrjUsers = CFile.Load( "users.copse", pProjectEntry.GetPath() );
        } catch (Exception e)
        {
            // TEMPORARY ???
            pPrjUsers = new CFile( "users.copse", pProjectEntry.GetPath() );
            CMember member = new CMember();
            member.SetLogin("admin");
            member.SetPassword("admin");
            member.SetName("Project Administrator");

            pPrjUsers.AddElement( member );
            pPrjUsers.Save();
        }

    }


    private void _LoadProcess()
        throws Exception
    {
        // Load Process ...
        try {
            // Try to Load existing process
            pPrjProcess = (CProcess) CFile.Load( "process.copse", pProjectEntry.GetPath() );
        } catch (Exception e)
        {
            // TEMPORARY ???
            // If something wrong happens, it tries to create another process
            CProcess process = new CProcess( "process.copse", pProjectEntry.GetPath() );

            CModelHeader header;
            process.AddElement( header = new CModelHeader() );
            process.SetModelHeader( header );
            header.SetName( "Process" );

            CSubModel   submodel;
            process.AddElement( submodel = new CSubModel() );

            header.SetSubModel( submodel );

            process.SetupActivities();
            process.Save();
        }
    }



    public CFile        GetProjectUsers()       { return pPrjUsers; }
    public CProcess     GetProjectProcess()     { return pPrjProcess; }








    private ProjectEntry _OpenProject( String projectId, String host, int port )
        throws Exception
    {
        ProjectsIndex pIndex;
        ProjectEntry prj;

        // Read Projects Index...
        FileInputStream file = new FileInputStream( "projects.index" );
        ObjectInputStream input = new ObjectInputStream( file );
        pIndex = (ProjectsIndex) input.readObject();
        file.close();

        prj = _FindProject( pIndex, projectId );
        if( prj == null )
        {
            System.out.println( "Project '"+projectId+"' not found!" );
            pIndex.DEBUG_write( System.out );
            throw new Exception( "Project '"+projectId+"' not found!" );
        }

        if( prj.IsOpen() )
        {
            System.out.println( "Project '"+projectId+"' has already been open!" );
            prj.DEBUG_write( System.out );
            throw new Exception( "Project '"+projectId+"' has already been open!" );
        }

        // Change the Project Status...
        prj.Open( host, port );


        // Save this information to ProjectsIndex
        FileOutputStream outputFile = new FileOutputStream( "projects.index" );
        ObjectOutputStream output = new ObjectOutputStream( outputFile );
        output.writeObject( pIndex );
        outputFile.close();

        return prj;
    }


    private void _CloseProject()
    {
        if( pProjectEntry != null )
        {
            pProjectEntry.Close();
            pProjectEntry = null;
        }
    }



    private ProjectEntry _FindProject( ProjectsIndex index, String projectId )
    {
        ProjectEntry prj;
        int id = 0;
        try{
            id = new Integer( projectId ).intValue();
        }
        catch( Exception e ){};


        Enumeration e = index.GetProjects();
        if( id != 0 )
            while( e.hasMoreElements() )
            {
                prj = (ProjectEntry) e.nextElement();
                if( prj.GetId() == id )
                    return prj;
            }
        else
            while( e.hasMoreElements() )
            {
                prj = (ProjectEntry) e.nextElement();
                if( prj.GetName().equals(projectId) )
                    return prj;
            }

        return null;
    }

    public void Exit()
    {
        // if any of documents has changed, the save it
        _CloseProject();
    }

}