/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Project Server Application
 * Classe: COPSEServerEventHandler
\******************************************************************************/


import java.io.*;
import java.net.*;
import java.util.*;


import COPSE.*;

public class COPSEServerEventHandler
    implements CEventHandler
{
    protected   COPSEServer     pServer;

    public COPSEServerEventHandler ( COPSEServer server )
    {
        pServer = server;
    }


    public void ProcessEvent( CEvent event, CConnectionHandler handler )
    {
        COPSEServerConnectionHandler pHandler = (COPSEServerConnectionHandler) handler;
        COPSEServerSessionManager    pSessionManager = pServer.GetSessionManager();

        int type = event.GetType();
        System.out.println( "[DEBUG] COPSEServerEventHandler: Event ["+type+"]" );

        switch( type )
        {
            case CEvent.EvCOPSE_Connection_Close:
                {
                    COPSEServerSession session = pHandler.GetSession();
                    int userId = session.GetUserId();


                    // Notify all clients
                    event = new CEvent( CEvent.EvCOPSE_Connection_Close );
                    event.AddArgument( new Integer(userId) );
                    session.Notify( event );

                    // Remove session from SessionManager
                    pSessionManager.RemoveSession( session );
                    pSessionManager.NotifyAll( event );

                    // Close connection
                    session.Close();

                    System.out.println( "[DEBUG} COPSEServerEventHandler - User ["+userId+"] logout" );
                    break;
                }

            case CEvent.EvCOPSE_SessionManager_Load:
                {
                    COPSESessionManager sMgr = pSessionManager.CloneAsCOPSESessionManager();

                    event = new CEvent( CEvent.EvCOPSE_SessionManager_Load );
                    event.AddArgument( sMgr );
                    try {
                        pHandler.Send( event );
                    } catch(IOException e){};

                    break;
                }

            default:

        }


        /* * /
        if( type <= CEvent.EvCOPSE_Connection_Exit )
            ProcessEvent_Session( event );
        else
        if( (type > CEvent.Ev...) && (type <= CEvent.Ev...) )
            ProcessEvent_Product( event );
        else
            ProcessEvent_Default( event );
        /* */
    }

}