/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Project Server Application
 * Classe: CServerManager
\******************************************************************************/


import java.io.*;
import java.util.*;
import java.net.Socket;

import COPSE.*;
import FILE.*;


class _ServerErrorHandler
    extends Thread
{
    BufferedReader pErrorReader;
    boolean mExit;

    public _ServerErrorHandler ( InputStream errorStream )
    {
        pErrorReader = new BufferedReader( new InputStreamReader(errorStream) );
        mExit = false;
    }

    public void run()
    {
        while( !mExit )
        {
            try{
                String msg = pErrorReader.readLine();
                if( msg != null )
                    System.err.println( "%Server% " + msg );
                else
                    mExit = true;
            } catch( IOException e ){}
        }
    }

    public void Exit()
    {
        mExit = true;
        try{
            pErrorReader.close();
        } catch( IOException e ) {};
    }
}






public class CServerManager
{
    //COPSEServerProjectManager   pProjectManager;

    CLocator                    pFileLocator;
    CFile                       pFile;

    int                         mServerSocketPort;
    CConnectionHandler          pConnection;
    CServerManagerEventHandler  pEventHandler;

    _ServerErrorHandler         pServerErrorHandler;
    boolean                     mExit;

    Process                     BUGFIX_ServerProcess;

    public CServerManager( int serverPort, CLocator locator, CFile file )
        throws IOException
    {
        super();

        mServerSocketPort = serverPort;

        pFileLocator = locator;
        pFile = file;

        ActivateServer();
    }


    protected CServerManagerEventHandler   CreateEventHandler()
    {
        return new CServerManagerEventHandler ( this );
    }

    protected   String      GetServerName()     { return "DefaultServerApp"; }

    public      int         GetServerSocketPort()   { return mServerSocketPort; }

    public      CLocator    GetLocator()        { return pFileLocator; }

    public      CFile       GetFile()           { return pFile; }

    protected void ActivateServer()
        throws IOException
    {
        String serverName = GetServerName();

        System.err.println( "[DEBUG] CServerManager - Starting Server Process ..." );
        Process process = Runtime.getRuntime().exec(
                "java " + serverName + " " + mServerSocketPort );
        if( process == null )
            throw new IOException( new String("[DEBUG] CServerManager - Error on starting "+ serverName) );

        BUGFIX_ServerProcess = process;

        System.err.println( "[DEBUG] CServerManager - Creating ServerErrorHandler..." );
        pServerErrorHandler = new _ServerErrorHandler( process.getErrorStream() );
        pServerErrorHandler.start();


        System.err.println( "[DEBUG] CServerManager - Creating event handler..." );
        pEventHandler = CreateEventHandler();


        System.err.println( "[DEBUG] CServerManager - Creating connector..." );
        int num = 0;
        CProcessStreamConnector connector;
        do {
            connector = null;
            try {
                System.err.println( "[DEBUG] CServerManager - Before ProcessStreamConnector..." );
                connector = CProcessStreamConnector.New( process, true );
                System.err.println( "[DEBUG] CServerManager - After ProcessStreamConnector..." );
            } catch (IOException e)
            {
                System.err.println( "[DEBUG] CServerManager - error on trying ..." );
                connector = null;
                if( ++num >= 10 )
                    throw e;
            }

            try{
                Thread.currentThread().sleep( 10 );
            } catch (InterruptedException ie) {};

        } while( connector == null );

        System.err.println( "[DEBUG] CServerManager - Creating connection handler..." );
        pConnection = new CConnectionHandler( connector, pEventHandler );
        pConnection.start();


/* BUGFIX * /
        // Send Initial Event
        System.err.println( "[DEBUG] CServerManager - Sending initial event..." );
        CEvent event = new CEvent( CEvent.EvCOPSE_Server_Init );
        event.AddArgument( pFile );
        pConnection.Send( event );
/* */



/*
        // ??? Send pPrjProcess
        CEvent event = new CEvent( CEvent.EvServer_Init );
        event.AddArgument( pPrjProcess );
        pConnector.Send( event );

        event = pConnector.Receive();
        Boolean bool = (Boolean) event.GetArgumentAt(0);
        if( bool.booleanValue() )
        {
            this.resume();
        }
        else
        {
            pConnector.SetServerProcess( null );
            throw new IOException( "Error on starting ProcessServer up!" );
        }
        //}
*/
    }


    public CConnectionHandler BUGFIX_CreateNewConnector()
        throws Exception
    {
        System.err.println( "[DEBUG] CServerManager - [BUGFIX] Creating socket to server..." );
        Socket socket = new Socket( "localhost", mServerSocketPort );

        System.err.println( "[DEBUG] CServerManager - [BUGFIX] Creating socketConnector to server..." );
        CSocketStreamConnector connector = CSocketStreamConnector.New( socket, true );

        // Subprocess Input/Output Streams are not going to be used anymore...
        pConnection.stop();

        // Create the connection to use the socket Streams
        System.err.println( "[DEBUG] CServerManager - [BUGFIX] Creating new ConnectionHandler..." );
        pConnection = new CConnectionHandler( connector, pEventHandler );
        pConnection.start();

        return pConnection;
    }


    public void Close()
    {
        if( pConnection != null )
        {
            try{
                CEvent event = new CEvent( CEvent.EvCOPSE_Server_Exit );
                pConnection.Send( event );
            } catch( IOException e )
            {
                pConnection.Close();
            }

            pServerErrorHandler.Exit();

            // pConnection.join(); // ???
        }
    }

        /*
        if( pConnector.IsServerActive() )
        {
            try {
                CEvent event = new CEvent( CEvent.EvCOPSE_Server_Exit );
                pConnector.Send( event );
            } catch ( IOException ioe )
            {
                mExit = true;
                resume();
                // ...

            }
        }
        else
        {
            mExit = true;
            resume();
            // ...
        }
        */




}