/* ****************************************************************************
 * Universidade Federal do Rio de Janeiro
 * COPPE - PESC
 * Mrcio de Sousa Dias
 ******************************************************************************
 * COPSE - Cooperative Project Support Environment
 * Package: TOOL.UML.CLIENT
 * Classe:  UMLRelationControl
\******************************************************************************/

package TOOL.UML.CLIENT;


import  java.awt.*;
import  java.awt.event.*;
import  java.util.*;

import  COPSE.*;
import  COPSE.CLIENT.*;
import  COPSE.CLIENT.GUI.*;
import  COPSE.CONTROL.*;
import  FILE.OBJ.*;
import  TOOL.UML.*;

/* ****************************************** BW *********************************************************** */
import  BW.*;
import  BW.kernel.*;
/* ****************************************** BW *********************************************************** */


public class    UMLRelationControl
    extends     CGUIControl
    implements  ItemListener, ActionListener //, WindowListener ...
{
    //* Tool Manager
    protected   UMLGUIToolManager   pToolManager;

    //* Controller of the Selected Element
    protected   CUMLGraphElement    pSelectedElement;

    //* Menus
    protected   Menu                pRelationMenu;
    protected   MenuItem            mnNew;
    protected   MenuItem            mnEdit;
    protected   MenuItem            mnDelete;
//    protected   MenuItem            mnRename;

    // Isn't it Awareness ???
    protected   Menu                pVisualMenu;
    protected   CheckboxMenuItem    mnVisualTotal;
    protected   CheckboxMenuItem    mnVisualPartial;
    protected   CheckboxMenuItem    mnVisualMinimum;

    //* Dialogs
    protected   UMLDlgRelation      pDlgEdit;
    protected   boolean             mLockRequested;
    protected   boolean             mIsEditing;


    public UMLRelationControl  ( UMLGUIToolManager tool )
    {
        super( tool.GetGUIManager() );
        pToolManager = tool;
        pSelectedElement = null;

        pDlgEdit = null;
        mLockRequested = false;
        mIsEditing = false;
    }


    public void SetEnabled( boolean enabled )
    {
        //super.SetEnabled( enabled );
        mIsEnabled = enabled;

        if( !enabled )
        {
            SetSelectedElement( null );
            /*
            pSelectedElement = null;
            if( pDlgEdit != null )
                CloseDlgEdit( false );
            mLockRequested = false;
            mIsEditing = false;
            */
        }

        ChangeMenuSettings();
    }

    public void     AddMenu( MenuBar menuBar )
    {
        pRelationMenu  = new Menu( "Relation" );

        mnNew       = new MenuItem( "New..." );
//        mnRename    = new MenuItem( "Rename..." );
        mnEdit      = new MenuItem( "Edit..." );
        mnDelete    = new MenuItem( "Delete" );

        pVisualMenu = new Menu( "Visual" );
        mnVisualTotal       = new CheckboxMenuItem( "Total" );
        mnVisualPartial     = new CheckboxMenuItem( "Partial" );
        mnVisualMinimum     = new CheckboxMenuItem( "Minimum" );

        pVisualMenu.add( mnVisualTotal );
        pVisualMenu.add( mnVisualPartial );
        pVisualMenu.add( mnVisualMinimum );


        pRelationMenu.add( mnNew );
//        pRelationMenu.add( mnRename );
        pRelationMenu.add( mnEdit );
        pRelationMenu.add( mnDelete );
        pRelationMenu.addSeparator();
        pRelationMenu.add( pVisualMenu );


        mnNew.addActionListener( this );
//        mnRename.addActionListener( this );
        mnEdit.addActionListener( this );
        mnDelete.addActionListener( this );

        mnVisualTotal.addItemListener( this );
        mnVisualPartial.addItemListener( this );
        mnVisualMinimum.addItemListener( this );


        ChangeMenuSettings();

        menuBar.add( pRelationMenu );
    }


    public synchronized void SetSelectedElement( CUMLGraphElement element )
    {
        // Close the edit dialog if it's open...
        if( mLockRequested )
        {
            try {
                CEvent event = new CEvent( CEvent.EvCOPSE_FileController_Element_Unlock );
                event.AddArgument( new Integer( pToolManager.GetClient().GetSessionManager().GetClientSession().GetConnectionId() ) );
                event.AddArgument( new Integer( pSelectedElement.GetElementController().GetElement().GetId() ) );
                pGUI.Send( event );
            } catch( Exception e ) {}

            mLockRequested = false;
        }
        else
        if( mIsEditing ) //pDlgEdit != null )
        {
            CloseDlgEdit( false );
        }

        pSelectedElement = element;
        ChangeMenuSettings();
    }

    public CUMLGraphElement     GetSelectedElement()
    {
        return pSelectedElement;
    }

    public CObjRelation GetSelectedRelation()
    {
        if( pSelectedElement == null )
            return null;

        return (CObjRelation) pSelectedElement.GetElementController().GetElement();
    }

    public boolean  IsWaitingForLock()
    {   return mLockRequested; }

    public synchronized void     ReceiveLockConfirmation()
    {
        if( mLockRequested )
        {
            pDlgEdit = new UMLDlgRelation( pGUI, false, this );
            pDlgEdit.setVisible( true );
            mIsEditing = true;
            mLockRequested = false;
        }
    }

    public void     CloseDlgEdit(boolean ok)
    {
        // For while... after will send the appropriate event to the server (Update or Unlock)
        if( ok )
        {
            // Send event to update relation...
            /* */
            if( pDlgEdit.HasChanged() )
            {
                CObjRelation relation = pDlgEdit.GetNewRelation();

                // Before sending, disconnect classes from relation
                Enumeration en = relation.GetExtremities();
                while( en.hasMoreElements() )
                {
                    CObjRelationExtremity extr = (CObjRelationExtremity) en.nextElement();
                    extr.SetClass( null );
                }

                System.err.println( "[DEBUG] UMLRelationControl - Sending relation:" );
                relation.DEBUG_write( System.err );

                try {
                    CEvent ev = new CEvent( CEvent.EvCOPSE_FileController_Element_Update );
                    ev.AddArgument( new Integer( pToolManager.GetClient().GetSessionManager().GetClientSession().GetConnectionId()  ) );
                    ev.AddArgument( relation );
                    pGUI.Send( ev );
                } catch( Exception e ) {}
            }
            /* */
        }


        // Unlock class
        try {
            CEvent ev = new CEvent( CEvent.EvCOPSE_FileController_Element_Unlock );
            ev.AddArgument( new Integer( pToolManager.GetClient().GetSessionManager().GetClientSession().GetConnectionId()  ) );
            ev.AddArgument( new Integer( pSelectedElement.GetElementController().GetElement().GetId() ) );
            pGUI.Send( ev );

            mIsEditing = false;
            pDlgEdit.setVisible( false );
            pDlgEdit.dispose();
            pDlgEdit = null;
        } catch( Exception e) {}
    }






    public void ChangeMenuSettings()
    {
        pRelationMenu.setEnabled( IsEnabled() );
        if( IsEnabled() )
        {
            if( pSelectedElement != null )
            {
                CVisualElementController elemController = pSelectedElement.GetElementController();

                // Edit and Delete...
                boolean isLocked = elemController.IsLocked();
                mnEdit.setEnabled( !isLocked );
                mnDelete.setEnabled( !isLocked );

                pVisualMenu.setEnabled( true );

                int visual = elemController.GetVisual();
                mnVisualTotal.setState( visual == 2 );
                mnVisualPartial.setState( visual == 1 );
                mnVisualMinimum.setState( visual == 0 );

                return;
            }

            mnEdit.setEnabled( false );
            mnDelete.setEnabled( false );
            pVisualMenu.setEnabled( false );
        }
    }



    public void actionPerformed( ActionEvent event )
    {
        Object source = event.getSource();

        if( source == mnNew )
        {
            pToolManager.GetCanvas().CreateNewRelation();
        }
        else

        if( source == mnEdit )
        {
            //*
            try {
                CEvent ev = new CEvent( CEvent.EvCOPSE_FileController_Element_Lock );
                ev.AddArgument( new Integer( pToolManager.GetClient().GetSessionManager().GetClientSession().GetConnectionId()  ) );
                ev.AddArgument( new Integer( pSelectedElement.GetElementController().GetElement().GetId() ) );
                pGUI.Send( ev );
                mLockRequested = true;
            } catch(Exception e) {}

            /* * /
            if( pDlgEdit == null )
                pDlgEdit = new UMLDlgRelation( pGUI, false, this );

            pDlgEdit.setVisible(true);
            /* */
        }

        else
        if( source == mnDelete )
        {
            try {
                CEvent ev = new CEvent( CEvent.EvCOPSE_FileController_Element_Delete );
                ev.AddArgument( new Integer( pToolManager.GetClient().GetSessionManager().GetClientSession().GetConnectionId()  ) );
                ev.AddArgument( new Integer( pSelectedElement.GetElementController().GetElement().GetId() ) );
                pGUI.Send( ev );

/* ****************************************** BW *********************************************************** */
		BW_Event BWEvent = new BW_Event (CEvent.EvCOPSE_FileController_Element_Delete+1000,typeIdDefault.UNSET);
		try {			
			CObjRelation BWRelation = (CObjRelation) pSelectedElement.GetElementController().GetElement();
			String BWDetails = new String ( BWRelation.GetName() );
			BWEvent.setDescription ("Relation Removed - "+BWDetails);
			
			BWDetails += ", entre ";
			BWDetails += BWRelation.GetExtremity(0).GetClass().GetName();
			BWDetails += " e ";
			BWDetails += BWRelation.GetExtremity(1).GetClass().GetName();
			BWDetails += ", ";
			
			BWEvent.setDetails ("Relacao "+BWDetails+" removida do diagrama");							
		}catch (Exception ex) {
			System.out.println ("[DEBUG]TOOL.UML.CLIENT.UMLClassControl.actionPerformed: exception "+ex);
			BWEvent.setDetails ("Relacionamento removido do diagrama");
		}

		GW_BW_Mediator.BeginEvent (CEvent.EvCOPSE_FileController_Element_Delete+1000,BWEvent);
		GW_BW_Mediator.EndEvent   (CEvent.EvCOPSE_FileController_Element_Delete+1000);
/* ****************************************** BW *********************************************************** */

            } catch(Exception e) {}
        }

    }


    public void itemStateChanged( ItemEvent event )
    {
/* */
        if( pSelectedElement == null )  return;

        CheckboxMenuItem source = (CheckboxMenuItem) event.getSource();
        CVisualElementController elemController = pSelectedElement.GetElementController();
        int currentVisual = elemController.GetVisual();

        int newVisual = 0;
        //if( source == mnVisualMinimum )     newVisual = 0;
        if( source == mnVisualPartial )     newVisual = 1;
        if( source == mnVisualTotal )       newVisual = 2;

        if( currentVisual == newVisual )
            source.setState( true );
        else
        {
            try {
                CEvent ev = new CEvent( CEvent.EvCOPSE_FileController_Element_Visual );
                ev.AddArgument( new Integer( elemController.GetElement().GetId() ) );
                ev.AddArgument( new Integer( newVisual ) );
                pGUI.Send( ev );

                elemController.SetVisual( newVisual );
                switch( currentVisual )
                {
                    case 0:     mnVisualMinimum.setState( false );  break;
                    case 1:     mnVisualPartial.setState( false );  break;
                    case 2:     mnVisualTotal.setState( false );    break;
                }

                pSelectedElement.Update();
                pToolManager.GetCanvas().repaint();
            }
            catch( Exception e )
            {}
        }
    }

}


